<?php
/**
 * UTF-8
 * Created by PhpStorm.
 * User: pavel.rudol
 * Date: 21.1.2016
 * Time: 13:51
 */

namespace Ron\RonPortal\Zamestnanci;

require_once('ron/controls.php');

use Exception;
use Ron\Controls\ComboBox;
use Ron\Controls\ICreateItem;
use Ron\Controls\Option;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\MessageManager;
use RonPortal\Log;
use RonPortal\RonPortalDBRecord;
use function Ron\ronText;
use RonPortal\Zamestnanci\Prukaz;
use RonPortal\Zamestnanci\ZdravotniPojisteniOsoby;


/**
 * Class RodinnyPrislusnik
 * @package Ron\RonPortal\Zamestnanci
 * @deprecated
 */
class RodinnyPrislusnik
{
    public $jmeno;
    public $prijmeni;
    public $rodnecislo;
    public $noverodnecislo;
    public $datumnarozeni;
    public $rodinnyvztah;
    public $rodinnyvztahstr;
    public $zdravotnistav;
    public $zdravotnistavstr;
    public $drzitelZTPod;
    public $vyzivovanaosoba;
    public $vyzivovanaosobastr;
    public $stejnaadresa;
    public $kontaktniosoba;
    public $telefon;
    public $email;


    /**
     * @param Database $database
     * @param array $row
     */
    public function nactiDataZRadku($database, $row)
    {
        $this->jmeno = $row[2];
        $this->prijmeni = $row[1];
        $this->rodnecislo = $row[0];
        if ($row[7]) {
            $this->datumnarozeni = $database->stringToTimestamp($row[7]);
        }
        $this->rodinnyvztah = $row[8];
        $this->rodinnyvztahstr = $row[11];
        $this->zdravotnistav = $row[4];
        $this->zdravotnistavstr = $row[12];
        if ($row[6]) {
            $this->drzitelZTPod = $database->stringToTimestamp($row[6]);
        }
        $this->vyzivovanaosoba = $row[10];
        $this->vyzivovanaosobastr = $row[16];
        $this->stejnaadresa = $row[9];
        $this->kontaktniosoba = $row[13];
        $this->telefon = $row[14];
        $this->email = $row[15];
    }

    /**
     * @param Database $database
     * @param string $rc
     * @return RodinnyPrislusnik[]
     * @throws DBException
     */
    public static function nactiRodinuZamestnance($database, $rc)
    {
        $params = $database->createParameters();

        $sql =
            "SELECT RodinnyPrislusnik.RC_RodinnehoPrislusnika, RodinnyPrislusnik.Prijmeni, RodinnyPrislusnik.Jmeno,". //0 - 2
            " RodinnyPrislusnik.Titul, RodinnyPrislusnik.ZdravotniStav, RodinnyPrislusnik.DrzitelPrukazuZTTP, RodinnyPrislusnik.DatumPriznaniZTTP, RodinnyPrislusnik.DatumNarozeni," . //3 - 7
            " ClenoveRodiny.RodinnyVztah, ClenoveRodiny.StejnaAdresa, ClenoveRodiny.VyzivovanaOsoba, S_RodinnyVztah.Popis, S_ZdravotniStav.Popis, Kontakt.Popis," . //8 - 13
            " Telefon, Email, VyzivovanaOsobaPopis.Popis" .     //14..16

            "\nFROM RodinnyPrislusnik JOIN ClenoveRodiny ON (RodinnyPrislusnik.RC_RodinnehoPrislusnika = ClenoveRodiny.RC_RodinnehoPrislusnika) " .
            "  JOIN S_RodinnyVztah ON (S_RodinnyVztah.Kod = ClenoveRodiny.RodinnyVztah) " .
            "  JOIN S_ZdravotniStav ON (S_ZdravotniStav.Kod = RodinnyPrislusnik.ZdravotniStav) " .
            "  LEFT JOIN S_AnoNe Kontakt ON (Kontakt.Kod = RodinnyPrislusnik.KontaktniOsoba) " .
            "  LEFT JOIN S_AnoNe VyzivovanaOsobaPopis ON (VyzivovanaOsobaPopis.Kod = ClenoveRodiny.VyzivovanaOsoba) " .
            "\nWHERE ClenoveRodiny.RC  = " . $params->addParameter($rc) .
            ' AND (ClenoveRodiny.PartnerDatumOd IS NULL OR ClenoveRodiny.PartnerDatumOd <= ' . $params->addDateParameter(time()) . ')' .
            ' AND (ClenoveRodiny.PartnerDatumDo IS NULL OR ClenoveRodiny.PartnerDatumDo >= ' . $params->addDateParameter(time()) . ')' .
            " ORDER BY RodinnyPrislusnik.DatumNarozeni";

        MessageManager::addDebugLog($sql);

        $cursor = $database->getSqlResult($sql,$params);

        $prislusnici = [];

        while ($row = $cursor->fetch()) {
            $prislusnik = new RodinnyPrislusnik();
            $prislusnik->nactiDataZRadku($database,$row);

            $prislusnici[] = $prislusnik;
        }

        return $prislusnici;
    }

    /**
     * @param Database $database
     * @param string $rc rc osoby, ktere se prirazuji clenove
     * @param int $index index prvku v postu
     * @return bool
     * @throws Exception
     */
    public function ulozRodinnehoPrislusnika($database, $rc, $index = null)
    {
        $this->nactiPost($index);

        if (!$this->rodnecislo) {
            return false;
        }

        $rec = new RonPortalDBRecord($database,'RodinnyPrislusnik');

        $rec->logDescription = 'Editace zamestnance - portal';

        $rec->addField('RC_RodinnehoPrislusnika','',false, 15, $this->noverodnecislo ?: $this->rodnecislo);
        $rec->addField('Prijmeni', '', false, 30, $this->prijmeni);
        $rec->addField('Jmeno','', false, 30, $this->jmeno);
        $rec->addField('ZdravotniStav', 'int', false, 0, $this->zdravotnistav);
        if ($this->drzitelZTPod > 0) {
            $rec->addField('DrzitelPrukazuZTTP', 'int', false, 0, 1);
            $rec->addField('DatumPriznaniZTTP', 'timestamp', false, 0, $this->drzitelZTPod);
        }
        $rec->addField('DatumNarozeni', 'timestamp', false, 0, $this->datumnarozeni);

        $params = $database->createParameters();

        $rec->setWhere(" RC_RodinnehoPrislusnika = " . $params->addParameter($this->rodnecislo), $params);

        $rec->loadRecord();
        
        try {
            $rec->updateOrInsert();
        }
        catch (Exception $exception) {
            throw new Exception('Chyba pri vkladani/aktualizaci rodinneho prislusnika.' . "\n" . $exception->getMessage());
        }

        //prirazeni clena rodiny k osobe se provede jen pri vlozeni, update je zajisten v databazi kaskadne!
        if (!isset($this->noverodnecislo)) {
            //přiřazení člena rodiny osobě
            $recClenove = new RonPortalDBRecord($database, 'ClenoveRodiny');

            $recClenove->logDescription = 'Editace zamestnance - portal';

            $recClenove->addField('RC', '', false, '15', $rc);
            $recClenove->addField('RC_RodinnehoPrislusnika', '', false, 15, $this->noverodnecislo ?: $this->rodnecislo);
            $recClenove->addField('RodinnyVztah', 'int', false, 0, $this->rodinnyvztah);
            $recClenove->addField('StejnaAdresa', 'int', false, 0, $this->stejnaadresa);
            $recClenove->addField('VyzivovanaOsoba', 'int', false, 0, $this->vyzivovanaosoba);

            $params = $database->createParameters();

            $recClenove->setWhere(" RC = " . $params->addParameter($rc) .
                " AND RC_RodinnehoPrislusnika = " . $params->addParameter($this->rodnecislo), $params);

            if ($recClenove->loadRecord() == 0) {
                try {
                    $recClenove->insertRecord();
                }
                catch (Exception $exception) {
                    throw new Exception('Chyba pri vkladani clena rodiny (prirazeni rodinneho prislusnika osobe).' . "\n" . $exception->getMessage());
                }
            }
        }

        return true;
    }
    
    /**
     * 
     * @param Database $database
     * @param int $index index prvku v postu
     * @return boolean
     * @throws Exception
     */
    public function smazRodinnehoPrislusnika($database, $index = null)
    {
        $this->nactiPost($index);

        if (!$this->rodnecislo) {
            return false;
        }
        
        $rec = new RonPortalDBRecord($database,'RodinnyPrislusnik');

        $rec->logDescription = 'Editace zamestnance - portal';

        $rec->addField('RC_RodinnehoPrislusnika','',false, 15, $this->noverodnecislo ?: $this->rodnecislo);

        $params = $database->createParameters();

        $rec->setWhere(" RC_RodinnehoPrislusnika = " . $params->addParameter($this->rodnecislo), $params);

        if (!$rec->deleteRecord()) {
            throw new Exception('Chyba pri mazani rodinneho prislusnika.');
        }

        return true;
    }

    /**
     * @param int $index index prvku v postu
     * @throws Exception
     */
    private function nactiPost($index = null)
    {
        $poradiPrislusnika = '';

        if (!is_null($index)) {
            $poradiPrislusnika = '_' . $index;
        }

        $this->jmeno = filter_input(INPUT_POST,'rodina_prijmeni'.$poradiPrislusnika,FILTER_SANITIZE_SPECIAL_CHARS);
        $this->prijmeni = filter_input(INPUT_POST, 'rodina_jmeno'.$poradiPrislusnika, FILTER_SANITIZE_SPECIAL_CHARS);
        $rc = filter_input(INPUT_POST, 'rodina_rc'.$poradiPrislusnika, FILTER_SANITIZE_SPECIAL_CHARS);
        if (is_null($this->rodnecislo)) {
            $this->rodnecislo = $rc;
        }
        elseif ($rc != $this->rodnecislo) {
            $this->noverodnecislo = $rc;
        }
        $this->datumnarozeni = DateUtils::dateStringToTimestamp(filter_input(INPUT_POST, 'rodina_datumnarozeni'.$poradiPrislusnika, FILTER_SANITIZE_SPECIAL_CHARS));
        $this->rodinnyvztah = filter_input(INPUT_POST,'rodina_vztah'.$poradiPrislusnika, FILTER_SANITIZE_SPECIAL_CHARS);
        $this->zdravotnistav = filter_input(INPUT_POST, 'rodina_zdravotni_stav'.$poradiPrislusnika, FILTER_SANITIZE_SPECIAL_CHARS);
        if ($_POST['rodina_drzitelztp'.$poradiPrislusnika] != '') {
            $this->drzitelZTPod = DateUtils::dateStringToTimestamp(filter_input(INPUT_POST, 'rodina_drzitelztp'.$poradiPrislusnika, FILTER_SANITIZE_SPECIAL_CHARS));
        }
        $this->vyzivovanaosoba = isset($_POST['rodina_vyzivovanaosoba'. $poradiPrislusnika]);
        $this->stejnaadresa = isset($_POST['rodina_stejnaadresa' . $poradiPrislusnika]);
    }

}


/**
 * Class RodinnyStav
 * @package Ron\RonPortal\Zamestnanci
 * @deprecated
 */
class RodinnyStav extends \RonPortal\Zamestnanci\RodinnyStav
{

    /**
     * @param Database $database
     * @param int $platnostOd
     * @param string $rc
     * @param bool $povinne
     * @param bool $ukoncitPredchozi
     * @return bool
     * @throws Exception
     */
    public static function ulozStav($database, $platnostOd = 0, $rc = '', $povinne = false, $ukoncitPredchozi = false)
    {
        $rodinnyStav = new RodinnyStav();
        $rodinnyStav->nactiPost();

        if ($platnostOd) {
            $rodinnyStav->platnostOd = $platnostOd;
        }

        if ($rc != '') {
            $rodinnyStav->rc = $rc;
        }

        if ($povinne) {
            if ($rodinnyStav->rc == '' || $rodinnyStav->platnostOd == 0 || $rodinnyStav->rodinnyStav == '') {
                MessageManager::addErrorMessage(ronText('Rodinny_stav_chybi_pole'));
                return false;
            }
        }
        elseif (!$rodinnyStav->rodinnyStav) {
            return true;
        }


        $rec = new RonPortalDBRecord($database,'RodinnyStav');

        $rec->logDescription = 'Editace zamestnance - portal';

        $rec->addField('RC','',false, '15', $rodinnyStav->rc);
        $rec->addField('PlatnostOd', 'timestamp', false, 0, $rodinnyStav->platnostOd);
        $rec->addField('KodRodinnehoStavu','int', false, 0, $rodinnyStav->rodinnyStav);
        $rec->addField('PlatnostDo', 'timestamp', false, 0, $rodinnyStav->platnostDo ?: null);

        $params = $database->createParameters();

        $rec->setWhere(" RC = " . $params->addParameter($rodinnyStav->rc) .
                        ' AND PlatnostOd = ' . $params->addParameter($database->timestampToParam($rodinnyStav->platnostOd)), $params);

        try {
            $rec->updateOrInsert();
        }
        catch (Exception $exception) {
            throw new Exception("Chyba pri aktualizaci/vlozeni rodinneho stavu \n" . $exception->getMessage());
        }

        if ($ukoncitPredchozi)
            $rodinnyStav->ukonciPredchozi($database);

        return true;
    }

    /**
     * @param Database $database
     * @param string $rc
     * @param int $puvodniPlatnostOd
     * @param int $novaPlatnostOd
     * @return bool
     * @throws DBException
     */
    public static function aktualizujPlatnostStavu($database, $rc, $puvodniPlatnostOd, $novaPlatnostOd)
    {
        $rec = new RonPortalDBRecord($database,'RodinnyStav');

        $rec->logDescription = 'Editace zamestnance - portal';

        $rec->addField('RC','',true, '15');
        $rec->addField('PlatnostOd', 'timestamp', true);
        $rec->addField('KodRodinnehoStavu','int', true);

        $params = $database->createParameters();

        $rec->setWhere(" RC = " . $params->addParameter($rc) .
            ' AND PlatnostOd = ' . $params->addParameter($database->timestampToParam($puvodniPlatnostOd)), $params);

        if (!$rec->loadRecord())
            return false;

        if ($rec->findFieldByName('PlatnostOd')->value > $novaPlatnostOd)
            $rec->findFieldByName('PlatnostOd')->setNewValue($novaPlatnostOd);

        try {
            $rec->updateRecord();
        } catch (Exception $exception) {
            throw new Exception("Chyba pri aktualizaci platnosti rodinneho stavu \n " . $exception->getMessage());
        }

        return true;
    }

    /**
     *
     */
    private function nactiPost()
    {
        $this->rc = filter_input(INPUT_POST,'RC',FILTER_SANITIZE_SPECIAL_CHARS);
        $this->platnostOd = filter_input(INPUT_POST, 'RodinnyStavPlatnostOd', FILTER_SANITIZE_SPECIAL_CHARS);
        $this->platnostDo = filter_input(INPUT_POST, 'RodinnyStavPlatnostDo', FILTER_SANITIZE_SPECIAL_CHARS);
        $this->rodinnyStav = filter_input(INPUT_POST, 'RodinnyStav', FILTER_SANITIZE_SPECIAL_CHARS);
    }

    /**
     * @param Database $database
     * @throws Exception
     */
    private function ukonciPredchozi($database)
    {
        $params = $database->createParameters();

        $sql =
            "UPDATE RodinnyStav ".
            "SET PlatnostDo = " . $params->addParameter($database->timestampToParam(DateUtils::incDay($this->platnostOd, -1))) .
            " WHERE RC = " . $params->addParameter($this->rc) .
            "  AND PlatnostOd < " . $params->addParameter($database->timestampToParam($this->platnostOd));

        $database->sql($sql, $params);
    }
}

/**
 * Class ZivotniPojisteniOsoby
 * @package Ron\RonPortal\Zamestnanci
 * @deprecated
 */
class ZivotniPojisteniOsoby extends \RonPortal\Zamestnanci\ZivotniPojisteniOsoby
{
}




/**
 * Class PenzijniPripojisteniOsoby
 * @package Ron\RonPortal\Zamestnanci
 * @deprecated
 */
class PenzijniPripojisteniOsoby extends \RonPortal\Zamestnanci\PenzijniPripojisteniOsoby
{
}


/**
 * Class PracovniPomuckaOsoby
 * @package Ron\RonPortal\Zamestnanci
 * @deprecated
 */
class PracovniPomuckaOsoby
{
    public $typVelikosti;
    public $typVelikostiPopis;
    public $velikost;

    /**
     * @param Database $database
     * @param string $rc
     * @return PracovniPomuckaOsoby[]
     * @throws DBException
     */
    public static function nactiVelikostiPomucekOsoby($database, $rc)
    {
        $params = $database->createParameters();
        $sql =  "SELECT TypVelikosti, Velikost " .
                "FROM VelikostPPOsoby " .
                "WHERE (RCOsoby = " . $params->addParameter($rc) . ") ".
                "ORDER BY TypVelikosti";
        
        MessageManager::addDebugLog($sql);
        
        $cursor = $database->getSqlResult($sql,$params);
        
        $pracovniPomucky = [];
        
        while ($row = $cursor->fetch()) {
            $pracovniPomucka = new PracovniPomuckaOsoby();
            $pracovniPomucka->typVelikosti = $row[0];
            $pracovniPomucka->velikost = $row[1];
            
            $pracovniPomucky[$row[0]] = $pracovniPomucka;
        }
        
        return $pracovniPomucky;
    }

    /**
     * @param Database $database
     * @return PracovniPomuckaOsoby[]
     * @throws DBException
     */
    public static function nactiTypyVelikostiPomucek($database)
    {
        $sql =  "SELECT Kod, Popis " .
                "FROM TypVelikosti " .
                "ORDER BY Kod";
        
        $cursor = $database->getSqlResult($sql);
        
        $seznamVelikosti = [];
        
        while ($row = $cursor->fetch()) {
            $typVelikosti = new PracovniPomuckaOsoby();
            $typVelikosti->typVelikosti = $row[0];
            $typVelikosti->typVelikostiPopis = $row[1];
            
            $seznamVelikosti[] = $typVelikosti;
        }
        
        return $seznamVelikosti;
    }

    /**
     *
     * @param Database $database
     * @param string $rc
     * @param bool $povinne
     * @return bool
     * @throws Exception
     */
    public function ulozVelikostPomuckyOsoby($database, $rc, $povinne = false)
    {
        $this->nactiPost();

        if ($povinne and (($rc == '') or ($this->typVelikosti == '') or ($this->velikost == ''))) {
            throw new Exception(ronText('Pomucky_chybi_povinne_pole'));
        }

        $rec = new RonPortalDBRecord($database,'VelikostPPOsoby');
        
        $rec->logDescription = 'Editace zamestnance - portal';
        
        $rec->addField('RCOsoby', '', false, 15, $rc);
        $rec->addField('TypVelikosti', '', false, 10, $this->typVelikosti);
        $rec->addField('Velikost', '', false, 20, $this->velikost);

        $params = $database->createParameters();

        $rec->setWhere(" RCOsoby = " . $params->addParameter($rc) .
                        " AND TypVelikosti = " . $params->addParameter($this->typVelikosti), $params);
        
        
        if (!$this->velikost) {
            if ($rec->loadRecord()) {
                //pokud je u osoby zaznam o velikosti, smaze se
                if (!$rec->deleteRecord()) {
                    throw new Exception('Chyba pri mazani velikosti pomucky osoby');
                }
                // smazani probehlo uspesne
                else {
                    return true;
                }
            }
            //zaznam velikosti neexistuje, nic se neprovede
            else {
                return false;
            }
        }
        
        try {
            $rec->updateOrInsert();
        }
        catch (Exception $exception) {
            throw new Exception('Chyba pri vkladani/aktualizaci velikosti pomucky osoby' . "\n" . $exception->getMessage());
        }

       return true;
    }
    
    private function nactiPost()
    {
        $velikost = explode(';', $_POST['velikost_pomucky_'.$this->typVelikosti]);
        $this->velikost = $velikost[1];
    }
}

/**
 * Class ZdravotniPojistovnaOsoby
 * @package Ron\RonPortal\Zamestnanci
 * @deprecated
 */
class ZdravotniPojistovnaOsoby extends ZdravotniPojisteniOsoby
{
    /**
     *
     * @param Database $database
     * @param null|string $cisloPojistence
     * @param int $platnostOd
     * @param bool $platnostOdPrvnihoDneMesice pokud je true, nastavi se platnostod na prvni den v mesici zadane platnosti od
     * @return bool
     * @throws Exception
     */
    public function ulozZdravotniPojistovnuOsoby($database, $cisloPojistence = null, $platnostOd = 0, $platnostOdPrvnihoDneMesice = true)
    {
        $this->nactiPost();
        
        if ($platnostOd > 0)
            $this->platnostOd = $platnostOd;

        if (!is_null($cisloPojistence))
            $this->cisloPojistence = $cisloPojistence;

        if (!$this->rc)
            return false;

        $this->platnostOd = $platnostOdPrvnihoDneMesice ? DateUtils::firstDayOfMonth($this->platnostOd) : $this->platnostOd;

        $this->ukonciPredchoziPojistovnu($database);

        $rec = new RonPortalDBRecord($database,'PojisteniOsobyU_ZP');
        
        $rec->logDescription = 'Editace zamestnance - portal';
        
        $rec->addField('RC', '', false, 15, $this->rc);
        $rec->addField('KodZdravotniPojistovny', '', false, 10, $this->kodZP);
        $rec->addField('PlatnostOd', 'timestamp', false, 0, $this->platnostOd);
        $rec->addField('CisloPojistence', '', false, 15, $this->cisloPojistence);

        $params = $database->createParameters();

        $rec->setWhere(" RC = " . $params->addParameter($this->rc) .
                        " AND PlatnostOd = " . $params->addParameter($database->timestampToParam($this->platnostOd)), $params);
        
        $rec->loadRecord();
        
        try {
            $rec->updateOrInsert();
        }
        catch (Exception $exception) {
            throw new Exception('Chyba pri zapisu pojistovny zamestnance.' . "\n" . $exception->getMessage());
        }
        return true;
    }

    /**
     *
     * @param Database $database
     * @param string $rc RC osoby
     * @param int $puvodniPlatnostOd
     * @param int $novaPlatnostOd
     * @param bool $platnostOdPrvnihoDneMesice
     * @param string $kodZP
     * @return bool
     * @throws Exception
     */
    public static function aktualizujPlatnostZdravotniPojistovnyOsoby($database, $rc, $puvodniPlatnostOd, $novaPlatnostOd, $platnostOdPrvnihoDneMesice = true, $kodZP = null)
    {
        $novaPlatnostOd = $platnostOdPrvnihoDneMesice ? DateUtils::firstDayOfMonth($novaPlatnostOd) : $novaPlatnostOd;

        $rec = new RonPortalDBRecord($database,'PojisteniOsobyU_ZP');

        $rec->logDescription = 'Editace zamestnance - portal';

        $rec->addField('RC', '', true, 15);
        if (!is_null($kodZP)) {
            $rec->addField('KodZdravotniPojistovny', '', true, 10);
        }
        $rec->addField('PlatnostOd', 'timestamp', true);

        $params = $database->createParameters();

        $rec->setWhere(" RC = " . $params->addParameter($rc) .
                        " AND PlatnostOd = " . $params->addParameter($database->timestampToParam($puvodniPlatnostOd)) .
                        (is_null($kodZP) ? "" : " AND KodZdravotniPojistovny = " . $params->addParameter($kodZP)), $params);

        $rec->loadRecord();

        if ($rec->findFieldByName('PlatnostOd')->value > $novaPlatnostOd)
            $rec->findFieldByName('PlatnostOd')->setNewValue($novaPlatnostOd);

        try {
            $rec->updateRecord();
        }
        catch (Exception $exception) {
            throw new Exception('Chyba pri aktualizaci platnosti pojistovny zamestnance.' . "\n" . $exception->getMessage());
        }

        return true;
    }

    /**
     * @throws Exception
     */
    private function nactiPost()
    {
        //$this->rc = filter_input(INPUT_POST, 'RC', FILTER_SANITIZE_SPECIAL_CHARS);
        $this->kodZP = filter_input(INPUT_POST, 'ZdravotniPojistovna', FILTER_SANITIZE_SPECIAL_CHARS);
        $this->cisloPojistence = filter_input(INPUT_POST, 'ZdravotniPojistovnaCisloPojistence', FILTER_SANITIZE_SPECIAL_CHARS);
        if ($_POST['ZdravotniPojistovnaPlatnostOd'] != '') {
            $this->platnostOd = DateUtils::dateStringToTimestamp(filter_input(INPUT_POST, 'ZdravotniPojistovnaPlatnostOd', FILTER_SANITIZE_SPECIAL_CHARS));
        }
    }

    /**
     * @param Database $database
     * @throws Exception
     */
    private function ukonciPredchoziPojistovnu($database)
    {
        $params = $database->createParameters();

        $sql =
            "UPDATE PojisteniOsobyU_ZP ".
            "SET PlatnostDo = " . $params->addParameter($database->timestampToParam(DateUtils::incDay($this->platnostOd, -1))) .
            " WHERE RC = " . $params->addParameter($this->rc) .
            "  AND PlatnostOd < " . $params->addParameter($database->timestampToParam($this->platnostOd));
        
        $database->sql($sql, $params);
    }
}


/**
 * Class AdresaOsoby
 * @package Ron\RonPortal\Zamestnanci
 * @deprecated
 */
class AdresaOsoby extends \RonPortal\Zamestnanci\AdresaOsoby
{
    /**
     * @param Database $database
     * @param int $platnostOd
     * @param int $index index prvku v postu
     * @return bool
     * @throws Exception
     */
    public function ulozAdresuOsoby($database, $platnostOd = 0, $index = null)
    {
        $this->nactiPost($index);

        if (is_null($this->platnostOd) && $platnostOd > 0)
            $this->platnostOd = $platnostOd;
        
        if (!($this->rc && $this->platnostOd))
            return false;
        
        $rec = new RonPortalDBRecord($database,'AdresaOsoby');

        $rec->logDescription = 'Editace zamestnance - portal';

        $rec->addField('RC', '', true, 15, $this->rc);
        $rec->addField('TrvaleBydliste', 'int', false, 0, $this->trvaleBydliste);
        $rec->addField('PlatnostOd', 'timestamp', false, 0, $this->platnostOd);
        $rec->addField('PlatnostDo', 'timestamp', false, 0, $this->platnostDo ?: null);
        $rec->addField('Ulice', '', false, 40, $this->ulice ?: null);
        $rec->addField('Mesto', '', false, 40, $this->mesto ?: null);
        $rec->addField('Obec', '', false, 10, $this->obec ?: null);
        $rec->addField('PSC', '', false, 10, $this->psc ?: null);
        $rec->addField('Stat', '', false, 10, $this->stat ?: null);
        $rec->addField('Telefon1', '', false, 20, $this->telefon1 ?: null);
        $rec->addField('Telefon2', '', false, 20, $this->telefon2 ?: null);
        $rec->addField('Mobil', '', false, 20, $this->mobil ?: null);
        $rec->addField('ZpusobDopravy', 'int', false, 0, $this->zpusobDopravy ? 1 : 0);

        $params = $database->createParameters();

        $rec->setWhere(" RC = " . $params->addParameter($this->rc) .
                        " AND PlatnostOd = " . $params->addParameter($database->timestampToParam($this->platnostOd)) .
                        " AND TrvaleBydliste = " . $params->addParameter($this->trvaleBydliste), $params);

        try {
            $rec->updateOrInsert();
        }
        catch (Exception $exception) {
            throw new Exception('Chyba pri vkladani/aktualizaci adresy osoby.' . "\n" . $exception->getMessage());
        }

        return true;
    }

    /**
     * @param Database $database
     * @param string $rc
     * @param int $puvodniPlatnostOd
     * @param int $novaPlatnostOd
     * @return bool
     * @throws Exception
     */
    public static function aktualizujPlatnostAdresyOsoby($database, $rc, $puvodniPlatnostOd, $novaPlatnostOd)
    {
        $params = $database->createParameters();
        $sql =
            'UPDATE AdresaOsoby SET PlatnostOd = ' . $params->addParameter($database->dateToParam($novaPlatnostOd)) .
            ' WHERE RC = ' . $params->addParameter($rc) .
            '   AND PlatnostOd = ' . $params->addParameter($database->dateToParam($puvodniPlatnostOd));

        $database->sql($sql, $params);

        Log::addRecordToDBLog($database, 'AdresaOsoby', 'UPDATE', 'Editace zamestnance - portal', "$sql");

        return true;
    }

    /**
     * @param int $index index prvku v postu
     * @throws Exception
     */
    private function nactiPost($index = null)
    {
        $poradiAdresy = '';

        if (!is_null($index)) {
            $poradiAdresy = '_' . $index;
        }

        $this->trvaleBydliste = filter_input(INPUT_POST,'adresa_trvale'.$poradiAdresy,FILTER_SANITIZE_SPECIAL_CHARS);
        $this->kontaktniAdresa = filter_input(INPUT_POST, 'adresa_kontaktni'.$poradiAdresy, FILTER_SANITIZE_SPECIAL_CHARS);
        
        if ($_POST['adresa_platnostOd'.$poradiAdresy] != '') {
            $this->platnostOd = DateUtils::dateStringToTimestamp(filter_input(INPUT_POST,'adresa_platnostOd'.$poradiAdresy,FILTER_SANITIZE_SPECIAL_CHARS));
        }
        if ($_POST['adresa_platnostDo'.$poradiAdresy] != '') {
            $this->platnostDo = DateUtils::dateStringToTimestamp(filter_input(INPUT_POST,'adresa_platnostDo'.$poradiAdresy,FILTER_SANITIZE_SPECIAL_CHARS));
        }
        
        $this->ulice = filter_input(INPUT_POST,'adresa_ulice'.$poradiAdresy,FILTER_SANITIZE_SPECIAL_CHARS);
        $this->mesto = filter_input(INPUT_POST,'adresa_mesto'.$poradiAdresy,FILTER_SANITIZE_SPECIAL_CHARS);
        $this->obec = filter_input(INPUT_POST,'adresa_obec'.$poradiAdresy,FILTER_SANITIZE_SPECIAL_CHARS);
        $this->psc = filter_input(INPUT_POST,'adresa_psc'.$poradiAdresy,FILTER_SANITIZE_SPECIAL_CHARS);
        $this->stat = filter_input(INPUT_POST,'adresa_stat'.$poradiAdresy,FILTER_SANITIZE_SPECIAL_CHARS);
        $this->telefon1 = filter_input(INPUT_POST,'adresa_telefon1'.$poradiAdresy,FILTER_SANITIZE_SPECIAL_CHARS);
        $this->telefon2 = filter_input(INPUT_POST,'adresa_telefon2'.$poradiAdresy,FILTER_SANITIZE_SPECIAL_CHARS);
        $this->mobil = filter_input(INPUT_POST,'adresa_mobil'.$poradiAdresy,FILTER_SANITIZE_SPECIAL_CHARS);
        $this->zpusobDopravy = filter_input(INPUT_POST,'adresa_zpusob'.$poradiAdresy,FILTER_SANITIZE_SPECIAL_CHARS);
        
    }
}


/**
 * Class ComboBoxObec
 * @package Ron\RonPortal\Zamestnanci
 */
class ComboBoxObec extends ComboBox implements ICreateItem
{
    public function __construct($varName)
    {
        parent::__construct($varName);
        $this->emptyRow = ' ';
        $this->className .= ' adresaObec';
    }

    /**
     * @param $database
     * @param $selectedValue
     * @throws DBException
     */
    public function loadDataCustom($database, $selectedValue)
    {
        $sql = "SELECT Obec.Kod, Obec.Popis, Okres.Popis  FROM Obec JOIN Okres ON (Okres.Kod = Obec.Okres) ORDER BY Obec.Popis";

        parent::loadData($database, $sql, $selectedValue);
        $this->setSelectedOption($selectedValue);
    }

    public function createItem($row)
    {
        if (!$row) {
            return new Option('', ' ');
        }

        $key = $row[0];
        $caption = $row[1] . ' (' . ronText('okres') . ' ' . $row[2] . ')';

        return new Option($key, $caption);
    }

}

/**
 * Class ComboBoxMesto
 * @package Ron\RonPortal\Zamestnanci
 */
class ComboBoxMesto extends ComboBox implements ICreateItem
{
    public function __construct($varName, $data = null, $keyFieldName = 'key', $valueFieldName = 'caption')
    {
        parent::__construct($varName, $data, $keyFieldName, $valueFieldName);
        $this->emptyRow = ' ';
        $this->className .= ' adresaMesto';
    }

    /**
     * @param $database
     * @param $selectedValue
     * @throws DBException
     */
    public function loadDataCustom($database, $selectedValue)
    {
        $sql =
            "SELECT Mesto.PSC, Mesto.Mesto, Mesto.Posta, Mesto.Stat, Obec.Kod, Obec.Popis, Okres.Popis 
             FROM Mesto LEFT JOIN Obec ON (Mesto.Obec = Obec.Kod) 
             LEFT JOIN Okres ON (Okres.Kod = Obec.Okres) 
             ORDER BY Mesto.Mesto, Obec.Popis";

        parent::loadData($database, $sql, $selectedValue);
        $this->setSelectedOption($selectedValue);
    }

    public function createItem($row)
    {
        if (!$row) {
            return new Option('', ' ');
        }

        $key = $row[1];
        $caption = $row[1] . ' - ' . ronText('posta') . ' ' . $row[2]; // . ' (' . \Ron\ronText('obec') . ' ' . $row[5] . ' - ' . \Ron\ronText('okres') . ' ' . $row[6];

        $option = new Option($key, $caption);
        $option->params = [
            'data-adresa-obec'  => $row[4],
            'data-adresa-psc'   => $row[0],
            'data-adresa-stat'  => $row[3]
        ];

        return $option;
    }
}

/**
 * Class PrukazOsoby
 * @package Ron\RonPortal\Zamestnanci
 * @deprecated
 */
class PrukazOsoby extends Prukaz
{
    /**
     *
     * @param Database $database
     * @return PrukazOsoby[]
     * @throws DBException
     */
    public static function nactiTypyPrukazu($database)
    {
        $sql =  "SELECT Kod, Popis " .
                "FROM TypPrukazu " .
                "ORDER BY Kod";

        $cursor = $database->getSqlResult($sql);

        $seznamTypu = [];

        while ($row = $cursor->fetch()) {
            $typPrukazu = new PrukazOsoby();
            $typPrukazu->typPrukazu = $row[0];
            $typPrukazu->typPrukazuPopis = $row[1];

            $seznamTypu[] = $typPrukazu;
        }

        return $seznamTypu;
    }

    /**
     *
     * @param Database $database
     * @param string $rc
     * @throws Exception
     * @throws DBException
     */
    public function ulozPrukazOsoby($database, $rc)
    {
        $this->nactiPost();

        $rec = new RonPortalDBRecord($database,'Prukaz');

        $rec->logDescription = 'Editace zamestnance - portal';

        $rec->addField('RC', '', true, 15, $rc);
        $rec->addField('TypPrukazu', 'int', true, 0, $this->typPrukazu);
        $rec->addField('CisloPrukazu', '', false, 20, $this->cisloPrukazu);
        $rec->addField('PlatnostOd', 'timestamp', false, 0, $this->platnostOd);
        $rec->addField('PlatnostDo', 'timestamp', false, 0, $this->platnostDo ?: null);
        $rec->addField('Poznamka', 'memo', false, 0, $this->poznamka);

        $params = $database->createParameters();

        $rec->setWhere(" RC = " . $params->addParameter($rc) .
                        " AND TypPrukazu = " . $params->addParameter($this->typPrukazu) .
                        " AND CisloPrukazu = " . $params->addParameter($this->cisloPrukazu), $params);


        $rec->loadRecord();

        try {
            $rec->updateOrInsert();
        }
        catch (Exception $exception) {
            throw new Exception('Chyba pri vkladani/aktualizaci prukazu osoby' . "\n" . $exception->getMessage());
        }
    }

    /**
     * @throws Exception
     */
    private function nactiPost()
    {
        $this->cisloPrukazu = filter_input(INPUT_POST, 'prukaz_' . $this->typPrukazu . '_cislo', FILTER_SANITIZE_SPECIAL_CHARS);
        if ($_POST['prukaz_' . $this->typPrukazu . '_platnostOd'] !== '') {
            $this->platnostOd = DateUtils::dateStringToTimestamp($this->platnostOd = filter_input(INPUT_POST, 'prukaz_' . $this->typPrukazu . '_platnostOd', FILTER_SANITIZE_SPECIAL_CHARS));
        }
        if (isset($_POST['prukaz_' . $this->typPrukazu . '_platnostDo']) && ($_POST['prukaz_' . $this->typPrukazu . '_platnostDo'] !== '')) {
            $this->platnostDo = DateUtils::dateStringToTimestamp($this->platnostDo = filter_input(INPUT_POST, 'prukaz_' . $this->typPrukazu . '_platnostDo', FILTER_SANITIZE_SPECIAL_CHARS));
        }
        $this->poznamka = filter_input(INPUT_POST, 'prukaz_' . $this->typPrukazu . '_poznamka', FILTER_SANITIZE_SPECIAL_CHARS);
    }

}

/**
 * Class ZdravotniStav
 * @package Ron\RonPortal\Zamestnanci
 * @deprecated
 */
class ZdravotniStav
{
    public $rc;
    public $platnostOd;
    public $platnostDo;

    public $zdravotniStav;
    public $zdravotniStavStr;
    public $drzitelPrukazuZTTP;
    public $osetrujiciLekar;
    public $osetrujiciLekarJmenoPrijmeni;
    public $zps;
    public $zps_tzp;

    /** Vraci vsechny zdravotni stavy osoby, pokud je zadana i platnost od, pak zvoleny rodinny stav
     *  a pokud je $aktualni = true, tak jen aktualni.
     * @param Database $database
     * @param string $rc
     * @param int $platnostOd
     * @param bool $aktualni
     * @return ZdravotniStav[]|ZdravotniStav
     * @throws Exception
     */
    public static function nactiStav($database, $rc, $platnostOd = 0, $aktualni = false)
    {
        $params = $database->createParameters();

        $sql =
            'SELECT PlatnostOd, PlatnostDo, ZdravotniStav, S_ZdravotniStav.Popis, DrzitelPrukazuZTTP, ZdravotniStavOsoby.OsetrujiciLekar, ' . //0 - 5
            '  OsetrujiciLekar.JmenoLekare, OsetrujiciLekar.Prijmeni, ZPS, ZPS_TZP ' . //6 - 9
            ' FROM ZdravotniStavOsoby JOIN S_ZdravotniStav ON (S_ZdravotniStav.Kod = ZdravotniStavOsoby.ZdravotniStav) ' .
            '   LEFT JOIN OsetrujiciLekar ON (OsetrujiciLekar.Kod = ZdravotniStavOsoby.OsetrujiciLekar) ' .
            ' WHERE ZdravotniStavOsoby.RC = ' . $params->addParameter($rc);
        if ($platnostOd) {
            $sql .=
                ' AND ZdravotniStavOsoby.PlatnostOd = ' . $params->addParameter($database->timestampToParam($platnostOd));
        } else if ($aktualni) {
            $sql .=
                ' AND ZdravotniStavOsoby.PlatnostOd <= ' . $params->addParameter($database->dateToParam(time())) .
                ' AND (ZdravotniStavOsoby.PlatnostDo >= ' . $params->addParameter($database->dateToParam(time())) . ' OR ZdravotniStavOsoby.PlatnostDo IS NULL)';
        }

        $sql .= ' ORDER BY PlatnostOd';

        MessageManager::addDebugLog($sql);

        $cursor = $database->getSqlResult($sql,$params);

        if ($platnostOd) {
            $row = $cursor->fetch();
            if (!$row) {
                return null;
            }

            $zdravotniStav = new ZdravotniStav();
            $zdravotniStav->nactiStavZRadku($database, $row);

            return $zdravotniStav;
        }

        $seznamStavu = [];

        while ($row = $cursor->fetch()) {
            $zdravotniStav = new ZdravotniStav();
            $zdravotniStav->nactiStavZRadku($database, $row);

            $seznamStavu[] = $zdravotniStav;
        }

        return $seznamStavu;
    }

    /**
     * @param Database $database
     * @param int $platnostOd
     * @param string $rc
     * @param bool $povinne
     * @param bool $ukoncitPredchozi
     * @return bool
     * @throws Exception
     */
    public static function ulozStav($database, $platnostOd = 0, $rc = '', $povinne = false, $ukoncitPredchozi = false)
    {
        $zdravotniStav = new ZdravotniStav();
        $zdravotniStav->nactiPost();

        if ($platnostOd) {
            $zdravotniStav->platnostOd = $platnostOd;
        }

        if ($rc != '') {
            $zdravotniStav->rc = $rc;
        }

        if ($povinne) {
            if ($zdravotniStav->rc == '' || $zdravotniStav->platnostOd == 0 || $zdravotniStav->zdravotniStav == '') {
                MessageManager::addErrorMessage(ronText('Zdravotni_stav_chybi_pole'));
                return false;
            }
        }
        elseif (!$zdravotniStav->zdravotniStav) {
            return true;
        }


        $rec = new RonPortalDBRecord($database,'ZdravotniStavOsoby');

        $rec->logDescription = 'Editace zamestnance - portal';

        $rec->addField('RC','',false, '15', $zdravotniStav->rc);
        $rec->addField('PlatnostOd', 'timestamp', false, 0, $zdravotniStav->platnostOd);
        $rec->addField('ZdravotniStav','int', false, 0, $zdravotniStav->zdravotniStav);
        $rec->addField('PlatnostDo', 'timestamp', false, 0, $zdravotniStav->platnostDo ?: null);

        $params = $database->createParameters();

        $rec->setWhere(" RC = " . $params->addParameter($zdravotniStav->rc) .
            ' AND PlatnostOd = ' . $params->addParameter($database->timestampToParam($zdravotniStav->platnostOd)), $params);

        try {
            $rec->updateOrInsert();
        }
        catch (Exception $exception) {
            throw new Exception("Chyba pri aktualizaci/vlozeni zdravotniho stavu \n" . $exception->getMessage());
        }

        if ($ukoncitPredchozi)
            $zdravotniStav->ukonciPredchozi($database);

        return true;
    }

    /**
     * @param Database $database
     * @param string $rc
     * @param int $puvodniPlatnostOd
     * @param int $novaPlatnostOd
     * @return bool
     * @throws DBException
     */
    public static function aktualizujPlatnostStavu($database, $rc, $puvodniPlatnostOd, $novaPlatnostOd)
    {
        $rec = new RonPortalDBRecord($database,'ZdravotniStavOsoby');

        $rec->logDescription = 'Editace zamestnance - portal';

        $rec->addField('RC','',true, 15);
        $rec->addField('PlatnostOd', 'timestamp', true);
        $rec->addField('ZdravotniStav','int', true);
        $rec->addField('PlatnostDo', 'timestamp', true);

        $params = $database->createParameters();

        $rec->setWhere(" RC = " . $params->addParameter($rc) .
            ' AND PlatnostOd = ' . $params->addParameter($database->timestampToParam($puvodniPlatnostOd)), $params);

        if (!$rec->loadRecord())
            return false;

        if ($rec->findFieldByName('PlatnostOd')->value > $novaPlatnostOd)
            $rec->findFieldByName('PlatnostOd')->setNewValue($novaPlatnostOd);

        try {
            $rec->updateRecord();
        } catch (Exception $exception) {
            throw new Exception("Chyba pri atkualizaci zdravotniho stavu osobu \n" . $exception->getMessage());
        }

        return true;
    }

    /**
     * @param Database $database
     * @param array $row
     */
    private function nactiStavZRadku($database, $row)
    {
        $this->platnostOd = $database->stringToTimestamp($row[0]);
        $this->platnostDo = $database->stringToTimestamp($row[1]);

        $this->zdravotniStav = $row[2];
        $this->zdravotniStavStr = $row[3];

        $this->drzitelPrukazuZTTP = $row[4];
        $this->osetrujiciLekar = $row[5];
        $this->osetrujiciLekarJmenoPrijmeni = $row[6] . ' ' . $row[7];
        $this->zps = $row[8];
        $this->zps_tzp = $row[9];
    }

    /**
     *
     */
    private function nactiPost()
    {
        $this->rc = filter_input(INPUT_POST,'RC',FILTER_SANITIZE_SPECIAL_CHARS);
        $this->platnostOd = filter_input(INPUT_POST, 'ZdravotniStavPlatnostOd', FILTER_SANITIZE_SPECIAL_CHARS);
        $this->platnostDo = filter_input(INPUT_POST, 'ZdravotniStavPlatnostDo', FILTER_SANITIZE_SPECIAL_CHARS);
        $this->zdravotniStav = filter_input(INPUT_POST, 'ZdravotniStav', FILTER_SANITIZE_SPECIAL_CHARS);
        //TODO: doplnit dalsi pole
    }

    /**
     * @param Database $database
     * @throws Exception
     */
    private function ukonciPredchozi($database)
    {
        $params = $database->createParameters();

        $sql =
            "UPDATE ZdravotniStavOsoby ".
            "SET PlatnostDo = " . $params->addParameter($database->timestampToParam(DateUtils::incDay($this->platnostOd, -1))) .
            " WHERE RC = " . $params->addParameter($this->rc) .
            "  AND PlatnostOd < " . $params->addParameter($database->timestampToParam($this->platnostOd));

        $database->sql($sql, $params);
    }
}

/**
 * Class EvidencniStav
 * @package Ron\RonPortal\Zamestnanci
 * @deprecated
 */
class EvidencniStav extends \RonPortal\Zamestnanci\EvidencniStav
{
}