<?php
/**
 * UTF8
 * stanislav.elbl, 18.09.2015
 */

namespace Ron\Http;

use Ron\GuiUtils;

class HtmlHeaders
{
    /** @var string[] */
    protected $headers = [];

    /** @var string[] */
    protected $bodyAttributes = [];

    /** @var string   Pripojuje se za jmeno souboru (css, script) pro zabraneni cachovani */
    public $versionId;

    private static $htmlHeaders;


    public function __construct($versionId = null)
    {
        $this ->versionId = $versionId;
    }


    /**
     * Vraci globalni instanci objektu HtmlHeaders
     * @return HtmlHeaders
     */
    public static function getInstance()
    {
        if (!self::$htmlHeaders) {
            self::$htmlHeaders = new HtmlHeaders();
        };

        return self::$htmlHeaders;
    }


    /**
     * Prida radek do HTML head.
     * @param string $header
     * @param string $id        Pokud je zadan $id a uz existuje radek se stejnym $id, tak jej nahradi
     * @return HtmlHeaders      Vraci $this pro moznost zretezeni
     */
    public function addHeader($header, $id = null)
    {
        if ($id) {
            $this->headers[$id] = $header;
        } else {
            $this->headers[] = $header;
        }

        return $this;
    }


    public function addCss($filename, $id = null, $params = '')
    {
        if (!$params  &&  $this ->versionId) {
            $params = '?ver=' . $this ->versionId;
        }

        $this ->addHeader(
            '<link rel="stylesheet" type="text/css" href="' .
            $filename . $params .
            '">',
            $id
        );

        return $this;
    }


    /**
     * Prida do HTML hlavicky zadany skript.
     * @param string $filename
     * @param string $id        viz {@see HtmlHeaders::addHeader}
     * @param string $params    Parametry, ktere se pridaji k url. Pokud nejsou zadany, tak se automaticky prida parametr ver=...
     * @return HtmlHeaders      Vraci $this pro moznost zretezeni
     */
    public function addScript($filename, $id = null, $params = '')
    {
        if (!$params  &&  $id  &&  $id[0] === '?') {
            // pro kompatibilitu se starym kodem (puvodne bylo $params druhy parametr a $id treti)
            $params = $id;
            $id = null;
        }

        if (!$params  &&  $this ->versionId) {
            $params = '?ver=' . $this ->versionId;
        }

        $this ->addHeader(
            '<script type="text/javascript" src="' .
            $filename . $params .
            '"></script>',
            $id
        );

        return $this;
    }


    public function setTitle($title)
    {
        $this ->addHeader("<title>$title</title>", 'title');
        return $this;
    }


    /**
     * Nastavi ikonu zobrazovanou v prohlizeci
     * @param $iconFile
     * @return $this
     */
    public function setIcon($iconFile)
    {
        $this ->addHeader("<link rel='shortcut icon' sizes='16x16' type='image/png' href='$iconFile'>", 'shortcut_icon');
        //$this ->addHeader("<link rel='apple-touch-icon' sizes='180x180' href='images/D_180x180.png'>", 'shortcut_icon_180'); - toto by bylo potreba, aby se zobrazovala i na plose Androidu
        return $this;
    }


    public function render()
    {
        foreach ($this ->headers as $header) {
            echo $header . "\n";
        }

        if (isset($this ->title)) {
            echo "<title>" . $this ->title . "</title>";
        }
    }


    /**
     * Prida atribut, ktery se vyrendruje u tagu <body> v html strance
     * @param string $name  Jmeno atributu
     * @param string $value Hodnota atributu
     * @return $this
     */
    public function addBodyAttribute($name, $value)
    {
        $this ->bodyAttributes[$name] = $value;
        return $this;
    }


    public function getBodyAttributesHtml()
    {
        return GuiUtils::getAttributesHtml($this ->bodyAttributes);
    }
}
