<?php
/**
 * Roman.Szablatura RON Software 6.4.2018
 */

namespace RonPortal\Benefity;

use Ron\Controls\BrowserField;
use Ron\Controls\ComboBox;
use Ron\Controls\DateInput;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\GuiUtils;
use Ron\MessageManager;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\ComboBoxOsoba;
use RonPortal\Controls\JmenoPrijmeniBrowserField;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\Security;
use function Ron\ronText;

/**
 * Class BenefityFiltrForm
 * @package RonPortal\Benefity
 */
class BenefityFiltrForm extends RonPortalFilterForm
{
    /** @var  PravaNaOsoby $pravo_rezervovat */
    public $pravo_rezervovat;

    /** @var  PravaNaOsoby $pravo_schvalit */
    public $pravo_schvalit;

    /** @var  PravaNaOsoby $pravo_vydat */
    public $pravo_vydat;

    /** @var  PravaNaOsoby $pravo_vznik */
    public $pravo_vznik;


    public $osobaBenefitu;

    /** @var int    Ke kteremu datu se ma nacitat stav (omezeni benefitu) */
    public $kDatu;

    public $filtrDlePrava;

    /** @var int        Zakladni zaskrtavatka (jen rezervace atd.) */
    public $omezeniBenefitu = 0;

    /** @var string     Vybrany typ benefitu (null=vse) */
    public $typBenefitu = null;

    /** @var integer    Vybrany benefit (null=vse) */
    public $benefit = null;

    /** @var integer    Vybrany druh pohybu (null=vse) */
    public $druhPohybu = null;

    public $jenNeschvalene = false;
    public $jenNecerpane = false;


    /**
     * BenefityFiltrForm constructor.
     * @param Database $database
     */
    public function __construct($database)
    {
        parent::__construct($database, 'benefity', true);

        $this->pravo_rezervovat = Prava::vratPrava('benefit_rezervace');
        $this->pravo_schvalit = Prava::vratPrava('benefit_schvaleni');
        $this->pravo_vydat = Prava::vratPrava('benefit_vydani');
        $this->pravo_vznik = Prava::vratPrava('benefit_vznik_bodu');

        //$this->skrytTlacitkoNastavit = true;
        //$this->f_classname = 'filtercontainer leftcaptions';

        $this ->setUseGet(true, 'benefity');
    }


    function renderSchvaleniRezervaceCerpani()
    {
        echo '<div class="filterrow">';

        echo
            '<span class="filtercaption">' .
            "<label><input type='radio' name='omezeni' value='0' " . ($this ->omezeniBenefitu == 0 ? 'checked="checked"' : "") . " class='clickAutoSubmit'> " . TextManager::_('Vse') . '</label>' .
            '</span>';

        echo
            '<span class="margin-right-2 padding-right-2">' .
            '<label class="margin-right-2"><input type="radio" name="omezeni" value="1" ' . ($this ->omezeniBenefitu == 1 ? 'checked="checked"' : '') . ' class="clickAutoSubmit"> ' . TextManager::_('jenRezervaci') . '</label>' .
            '<label class="margin-right-2"><input type="radio" name="omezeni" value="2" ' . ($this ->omezeniBenefitu == 2 ? 'checked="checked"' : '') . ' class="clickAutoSubmit"> ' . TextManager::_('jenSchvaleni') . '</label>' .
            '<label class="margin-right-2"><input type="radio" name="omezeni" value="3" ' . ($this ->omezeniBenefitu == 3 ? 'checked="checked"' : '') . ' class="clickAutoSubmit"> ' . TextManager::_('jenCerpani') . '</label>' .
            '</span>';
//        echo "&nbsp;&nbsp;&nbsp;";
//        echo "&nbsp;&nbsp;&nbsp;";

        echo TextManager::_('benefity_k_datu') . ' ';
        DateInput::renderDateSelect('kdatu', date('j.n.Y', $this->kDatu), true);

        echo '</div>';
    }


    /**
     * @throws \Ron\DB\DBException
     */
    function renderDruhPohybu()
    {
        echo '<div class="filterrow">';
        echo '<span class="filtercaption">';
        echo TextManager::_('druh_pohybu');
//            GuiUtils::renderCheckbox('jenDruh', \Ron\ronText('druh_pohybu'). '&nbsp;&nbsp;', $_SESSION['jenDruh'],
//                'onClick="submit()"');
        echo '</span>';

        $sql =
            'SELECT Kod, Popis' .
            ' FROM S_DruhPohybuBenefitu' .
            ' ORDER BY Popis';

        $combo = new ComboBox('druh');
        $combo->className = 'form-input adsfilterforminput_full';
        $combo->htmlParams = ['data-ron-autosubmit' => '1'];
        $combo->dataTypeComparison = false;
        $combo->emptyRow = TextManager::_('_vse_');
        $combo->loadData($this->database, $sql, $this ->druhPohybu);
        $combo->render();
        echo '</div>';
    }


    /**
     * Zobrazi filtr pro vyber typu benefitu
     * @throws \Ron\DB\DBException
     */
    function renderTypBenefitu()
    {
        echo '<div class="filterrow">';
        echo '<span class="filtercaption">';
        echo TextManager::_('typ_benefitu');
//            GuiUtils::renderCheckbox('jenTypBenefitu', \Ron\ronText('typ_benefitu'). '&nbsp;&nbsp;', $this ->jenTypBenefitu,
//                'onClick="submit()"');
        echo '</span>';

        $sql =
            'SELECT DISTINCT TypBenefitu.Kod, TypBenefitu.Popis' .
            ' FROM TypBenefitu Left JOIN Benefit ON (TypBenefitu.Kod = Benefit.TypBenefitu)' .
            ' ORDER BY TypBenefitu.Popis';

        $combo = new ComboBox('typbenefitu');
        $combo->className = 'form-input adsfilterforminput_full';
        $combo->htmlParams = ['data-ron-autosubmit' => '1'];
        $combo->dataTypeComparison = false;
        $combo->emptyRow = TextManager::_('_vse_');
        $combo->loadData($this->database, $sql, $this ->typBenefitu);
        $combo->render();
        echo '</div>';
    }



    /**
     * Zobrazi filtr pro vyber benefitu
     * @throws \Ron\DB\DBException
     * @throws \Exception
     */
    function renderBenefit()
    {
        echo '<div class="filterrow">';
        echo '<span class="filtercaption">';
        echo TextManager::_('benefit');
//            GuiUtils::renderCheckbox('jenBenefit', \Ron\ronText('benefit'). '&nbsp;&nbsp;', $_SESSION['jenBenefit'],
//                'onClick="submit()"');
        echo '</span>';

        $prava = array($this->pravo_rezervovat, $this->pravo_schvalit, $this->pravo_vydat, $this->pravo_vznik);
        $this->filtrDlePrava = Prava::sqlFiltrPodlePrav($prava, 'PracovniPomer', time(), time());

        $sql =
            'SELECT DISTINCT Benefit.ID, Benefit.Popis' .
            ' FROM Benefit' .
            ' LEFT JOIN PohybBenefitu ON (Benefit.ID = PohybBenefitu.IdBenefitu)' .
            ' LEFT JOIN PracovniPomer ON (PracovniPomer.RC = PohybBenefitu.RC)' .
            ' WHERE PohybBenefitu.IdBenefitu IS NOT NULL' .
            ($this ->filtrDlePrava ? ' AND ' . $this->filtrDlePrava : '') .
            ' ORDER BY Benefit.Popis';

        $combo = new ComboBox('benefit');
        $combo->className = 'form-input adsfilterforminput_full';
        $combo->htmlParams = ['data-ron-autosubmit' => '1'];
        $combo->dataTypeComparison = false;
        $combo->emptyRow = TextManager::_('_vse_');
        $combo->loadData($this->database, $sql, $this ->benefit);
        $combo->render();
        echo '</div>';
    }


    function renderOsobaBenefitu()
    {
        echo '<div class="filterrow">';
        echo '<span class="filtercaption">';
        echo TextManager::_('osoba_benefitu');
//            GuiUtils::renderCheckbox('jenOsoba', \Ron\ronText('osoba_benefitu'). ' ', $_SESSION['jenOsoba'],
//                'onClick="submit()"');
        echo '</span>';

        $datumOd = DateUtils::today();
        $datumDo = DateUtils::today();

        $combo = new ComboBoxOsoba('osoba');
        $combo->className = 'form-input adsfilterforminput_full';
        $combo->dataTypeComparison = false;
        $combo->htmlParams = ['data-ron-autosubmit' => '1'];
//        $combo->emptyRow = TextManager::_('_vsichni_');
        $combo->filtr = ' EXISTS (Select PohybBenefitu.RC From PohybBenefitu Where PohybBenefitu.RC = Osoba.RC)';
        $prava = array($this->pravo_rezervovat, $this->pravo_schvalit, $this->pravo_vydat, $this->pravo_vznik);
        $combo->loadDataPP($this->database, $datumOd, $datumDo, $prava, TextManager::_('_vsichni_'), $this ->osobaBenefitu, false);
        $combo->render();

        echo '</div>';
    }


    function renderFiltryBenefitu()
    {
        echo '<div class="filterrow">';
        echo '<span class="filtercaption">';
        echo "</span>";

        GuiUtils::renderCheckbox('neschvalene', TextManager::_('neschvalene_benefity') . ' ', $this ->jenNeschvalene,
            '', 'clickAutoSubmit');

        GuiUtils::renderCheckbox('necerpane', TextManager::_('necerpane_benefity') . ' ', $this ->jenNecerpane,
            '', 'clickAutoSubmit');


        echo '</div>';
    }


    function renderOdkazy()
    {
        echo '<div class="margin-top-2">';
        echo '<ul class="linkslist">';

        if ($this->pravo_rezervovat->jsouNejakePrava()) {
            echo "<li><a href='?menuid=benefity&action=rezervacebodu'>" . TextManager::_('rezervace_benefitu') . "</a></li>";
        }

        if ($this->pravo_vznik->jsouNejakePrava()) {
            echo "<li><a href='?menuid=benefity&action=vznikbodu'>" . TextManager::_('vznik') . "</a></li>";
        }

        /*if ($this->pravo_rezervovat->jsouNejakePrava()) {
            echo "<li><a href='?menuid=benefity&action=sablony'>" . TextManager::_('sablony_benefitu') . "</a></li>";
        }*/

        echo "</ul>";
        echo "</div>";
    }

    private function loadOmezeniBenefitu()
    {
        if (isset($_GET['omezeni'])) {
            $this ->omezeniBenefitu = (int)$_GET['omezeni'];
            $_SESSION['omezeniBenefitu'] = $this ->omezeniBenefitu;
        } else {
            $this ->omezeniBenefitu = isset($_SESSION['omezeniBenefitu']) ? (int)$_SESSION['omezeniBenefitu'] : 0;
        }

        if (isset($_GET['kdatu'])) {
            try {
                $this->kDatu = ($_GET['kdatu'] != '') ? DateUtils::dateStringToTimestamp($_GET['kdatu']) : time();
            } catch(\Exception $e) {
                MessageManager::addErrorMessage($e);
                $this->kDatu = time();
            }
            $_SESSION['datumFiltruBenefitu'] = $this ->kDatu;
        } else {
            $this ->kDatu = isset($_SESSION['datumFiltruBenefitu']) ? $_SESSION['datumFiltruBenefitu'] : time();
        }
    }


    function loadTypBenefitu()
    {
        if (isset($_GET['typbenefitu'])) {
            $this ->typBenefitu = $_GET['typbenefitu'];
            if ($this ->typBenefitu === '') {
                $this ->typBenefitu = null;
            }
            $_SESSION['filtr_typ_benefitu'] = $this ->typBenefitu;
        } else {
            $this ->typBenefitu = isset($_SESSION['filtr_typ_benefitu']) ? $_SESSION['filtr_typ_benefitu'] : null;
        }
    }

    function loadBenefit()
    {
        if (isset($_GET['benefit'])) {
            $this ->benefit = $_GET['benefit'];
            if ($this ->benefit === '') {
                $this->benefit = null;
            }
            $_SESSION['filtr_benefit'] = $this ->benefit;
        } else {
            $this ->benefit = isset($_SESSION['filtr_benefit']) ? $_SESSION['filtr_benefit'] : null;
        }
    }

    function loadDruhPohybu()
    {
        if (isset($_GET['druh'])) {
            $this ->druhPohybu = $_GET['druh'];
            if ($this ->druhPohybu === '') {
                $this->druhPohybu = null;
            }
            $_SESSION['filtr_druh_pohybu_benefitu'] = $this ->druhPohybu;
        } else {
            $this ->druhPohybu = isset($_SESSION['filtr_druh_pohybu_benefitu']) ? $_SESSION['filtr_druh_pohybu_benefitu'] : null;
        }
    }


    function loadOsobaBenefitu(){
        if (isset($_GET['osoba'])) {
            $this ->osobaBenefitu = $_GET['osoba'];
            if ($this ->osobaBenefitu === '') {
                $this ->osobaBenefitu = null;
            }
            $_SESSION['osoba'] = $this ->osobaBenefitu;
        } else {
            $this ->osobaBenefitu = null;
        }
//        if (isset($_POST['adsfilterform'])){
//            $_SESSION['jenOsoba'] = isset($_POST['jenOsoba']);
//            $_SESSION['osoba'] = $_POST['osoba'];
//        }
    }


    function loadFiltryBenefitu()
    {
        if (isset($_GET['kdatu']) ||  isset($_GET['neschvalene'])  ||  isset($_GET['necerpane'])) {
            $this ->jenNeschvalene = isset($_GET['neschvalene']) ? (int)$_GET['neschvalene'] : null;
            if ($this ->jenNeschvalene) {
                $_SESSION['jen_neschvalene_benefity'] = $this->jenNeschvalene;
            } else {
                unset($_SESSION['jen_neschvalene_benefity']);
            }

            $this ->jenNecerpane = isset($_GET['necerpane']) ? (int)$_GET['necerpane'] : null;
            if ($this ->jenNecerpane) {
                $_SESSION['jen_necerpane_benefity'] = $this->jenNecerpane;
            } else {
                unset($_SESSION['jen_necerpane_benefity']);
            }
        }
    }

    function loadData()
    {
        $this->loadOmezeniBenefitu();
        $this->loadTypBenefitu();
        $this->loadBenefit();
        $this->loadOsobaBenefitu();
        $this->loadDruhPohybu();
        $this->loadFiltryBenefitu();

        parent::loadData();
    }


    /**
     * @throws \Ron\DB\DBException
     */
    function renderFilters()
    {
        $this->renderSchvaleniRezervaceCerpani();
        $this->renderOsobaBenefitu();
        $this->renderTypBenefitu();
        $this->renderBenefit();
        $this->renderDruhPohybu();
        $this->renderFiltryBenefitu();


        $this->renderOdkazy();
    }
}


/**
 * Class DatumPlatnostiRezervaceField
 * @package RonPortal\Benefity
 */
class DatumPlatnostiRezervaceField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($rowData[8]) { // DatumRezervaceDo
            return ronText('rezervace_do') . ': <span class="rezervace-do">' . parent::getViewText($rowData[8], $rowNumber, $recordId, $rowData) . '</span>';
        } else if ($rowData[15]) { // DatumOmezeniPlatnosti
            return ronText('platnost') . ': <span class="omezeni-platnosti-do">' . parent::getViewText($rowData[15], $rowNumber, $recordId, $rowData) . '</span>';
        }

        return '';
    }
}


/**
 * Class DatumSchvaleniField
 * @package RonPortal\Benefity
 */
class DatumSchvaleniField extends BrowserField
{
    /** @var PravaNaOsoby $pravo_schvalit */
    public $pravo_schvalit;

    /** @var PravaNaOsoby $pravo_vznik_bodu */
    public $pravo_vznik_bodu;


    /**
     * DatumSchvaleniField constructor.
     * @param Database $database
     * @param PravaNaOsoby $pravo_schvalit
     * @param PravaNaOsoby $pravo_vznik_bodu
     */
    function __construct($database, $pravo_schvalit, $pravo_vznik_bodu)
    {
        parent::__construct($database);

        $this->pravo_schvalit = $pravo_schvalit;
        $this->pravo_vznik_bodu = $pravo_vznik_bodu;
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws \Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($rowData[6]) { // DatumSchvaleni
            return parent::getViewText($value, $rowNumber, $recordId, $rowData);
        }

        if ($rowData[2] != 31 && $rowData[2] != 51) { // druh pohybu
            $zobrazitOdkaz = true;
            //Odkaz pro splneni jen pokud je platnostDo nebo RezervaceDo vetsi nez akt. datum
            if ($rowData[8]) { // DatumRezervaceDo
                $zobrazitOdkaz = $this->database->stringToTimestamp($rowData[8]) >= DateUtils::today();
            } else if ($rowData[15]) { // DatumOmezeniPlatnosti
                $zobrazitOdkaz = $this->database->stringToTimestamp($rowData[15]) >= DateUtils::today();
            }

            if ($zobrazitOdkaz) {
                if ($this->pravo_schvalit->lzeZobrazitRC($rowData[1]) || ($this->pravo_vznik_bodu->lzeZobrazitRC($rowData[1]) && $rowData[2] == 11)) {
                    return GuiUtils::changeLocationQuest(ronText('schvalit_benefit'), ronText('Opravdu_schvalit_benefit'),
                        '?menuid=benefity&action=schvalenirezervace&record=' . $rowData[0] . Security::csrfUrlParameter());
                }
            }

        }

        return '';
    }
}


/**
 * Class DatumCerpaniField
 * @package RonPortal\Benefity
 */
class DatumCerpaniField extends BrowserField
{

    /** @var PravaNaOsoby $pravo_vydat */
    public $pravo_vydat;

    /**
     * DatumCerpaniField constructor.
     * @param null|Database $database
     * @param PravaNaOsoby $pravo
     */
    function __construct($database, $pravo)
    {
        parent::__construct($database);
        $this->pravo_vydat = $pravo;
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws \Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($rowData[2] != 31 && $rowData[2] != 51 && $rowData[2] != 11) {
            if ($rowData[9]) { // DatumCerpani => jiz cerpano
                return parent::getViewText($value, $rowNumber, $recordId, $rowData);
            }

            if ($this->pravo_vydat->lzeZobrazitRC($rowData[1])) {
                $zobrazitOdkaz = true;
                // Odkaz pro splneni pokud je platnostDo nebo RezervaceDo vetsi nez akt. datum
                if ($rowData[8]) { // DatumRezervaceDo
                    $zobrazitOdkaz = $this->database->stringToTimestamp($rowData[8]) >= DateUtils::today();
                } elseif ($rowData[15]) { // DatumOmezeniPlatnosti
                    $zobrazitOdkaz = $this->database->stringToTimestamp($rowData[15]) >= DateUtils::today();
                }

                if ($zobrazitOdkaz) {
                    $zobrazitDanyTyp = true;
                    if (trim($this->pravo_vydat->priznak3) != '') {
                        $povoleneTypyBenefitu = explode(';', $this->pravo_vydat->priznak3);
                        $zobrazitDanyTyp = in_array($rowData[27], $povoleneTypyBenefitu);
                    }

                    if ($zobrazitDanyTyp) {
                        return GuiUtils::changeLocationQuest(ronText('vydat_benefit'), ronText('opravdu_vlozit_cerpani'),
                            '?menuid=benefity&action=cerpanirezervace&record=' . $rowData[0] . Security::csrfUrlParameter());
                    }
                }
            }
        }

        return '';
    }
}


/**
 * Class StavBenefituField
 * @package RonPortal\Benefity
 */
class StavBenefituField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = '';

//        "PB.ID, PB.RC, DruhPohybu, DruhZdroje, IdBenefitu, DatumVlozeni, DatumSchvaleni, Schvalil, " .        //0..7
//        "DatumRezervaceDo, DatumCerpani, Cerpani_Username, PocetBodu, PocetBenefitu, TypDoplatku, " .       //8..13


        switch ($rowData[2]) {

            case PohybBenefitu::DRUH_POHYBU_VZNIK_BODU:
                $result .= $rowData[6] // je schvaleno
                    ? ronText('vznik_bodu') . " (" . $rowData[26] . ")" // pocet bodu
                    : ronText('vznik') . " (" . ronText('neschvaleno') . ")";
                break;

            case PohybBenefitu::DRUH_POHYBU_ZANIK_BODU;
                $result .= ronText('zanik_bodu') . " (" . $rowData[26] .  ")"; // pocet bodu
                break;

            case PohybBenefitu::DRUH_POHYBU_CERPANI_BODU:
            case PohybBenefitu::DRUH_POHYBU_REZERVACE_BODU:
                if ($rowData[6]) { // schvaleno
                    $result .=
                        TextManager::_($rowData[9] ? 'cerpano' : 'necerpano') . // datum cerpani
                        ' - ' . StringUtils::htmlTextConvert($rowData[23]) . ' (' . StringUtils::htmlTextConvert($rowData[25]) . ')';  // popis benefitu (typ benefitu)
                } else {
                    $result .= StringUtils::htmlTextConvert($rowData[23]) . " (" . ronText('neschvaleno') . ")"; // popis benefitu
                }
                break;

            case PohybBenefitu::DRUH_POHYBU_ZANIK_REZERVACE_BODU:
                $title = StringUtils::htmlTextConvert($rowData[23])  . ' (' . StringUtils::htmlTextConvert($rowData[25]) . ')';
                $result .= "<div title='$title' >" . ronText('zanik_rezervace') . ' (' . $rowData[26] .  ')' . "</div>" ;
                break;
        }

        return $result
            ? "<a href='?menuid=benefity&action=pohyb&record=$rowData[0]'>" . $result . '</a>'
            : '';
    }
}


/**
 * Class BenefityBrowser
 * @package RonPortal\Benefity
 */
class BenefityBrowser extends RonPortalBrowser
{

    /** @var  PravaNaOsoby $prava_rezervovat */
    public $pravo_rezervovat;

    /** @var  PravaNaOsoby $prava_schvalit */
    public $pravo_schvalit;

    /** @var  PravaNaOsoby $pravo_vydat */
    public $pravo_vydat;

    /** @var  PravaNaOsoby $pravo_vznik */
    public $pravo_vznik;

//    /** @var  BenefityFiltrForm $filtrForm */
//    public $filtrForm;


    /**
     * BenefityBrowser constructor.
     * @param \Ron\DB\Database $database
     * @throws \Exception
     */
    public function __construct($database)
    {
        parent::__construct($database, 'benefity');

        Prava::nastavObdobiProPrava(time(), time());
        $this->pravo_rezervovat = Prava::vratPrava('benefit_rezervace');
        $this->pravo_schvalit = Prava::vratPrava('benefit_schvaleni');
        $this->pravo_vydat = Prava::vratPrava('benefit_vydani');
        $this->pravo_vznik = Prava::vratPrava('benefit_vznik_bodu');


        /*if ($this->pravo_schvalit->prava == 1 || $this->pravo_rezervovat->prava == 1 || $this->pravo_vznik->prava == 1 || $this->pravo_vydat->prava == 1) {
            $this->title = ronText('prehled_pohybu_benefitu');
        } else {
            $this->title = ronText('prehled_pohybu_vlastnich_benefitu');
        }*/

        $this->code = 'browserbenefity';
        $this->tableId = 'browserbenefity';
        $this->browserUrl = "?menuid=benefity";

        $this->sqlSelectPart =
            'PB.ID, PB.RC, DruhPohybu, S_DruhPohybuBenefitu.Popis, IdBenefitu, PB.DatumVlozeni, DatumSchvaleni, Schvalil,' .    //0..7
            ' DatumRezervaceDo, DatumCerpani, Cerpani_Username, PocetBodu, PocetBenefitu, TypDoplatku,' .       //8..13
            ' PB.DoplatekCastka, DatumOmezeniPlatnosti, ' . $database->castWideMemo('PB.Poznamka') .            //14..16

            ', PB.Insert_Username, PB.Insert_Timestamp, PB.Update_Timestamp, PB.Update_Username,' .             //17..20
            " Osoba.Jmeno, Osoba.Prijmeni," .    //21..22
            ' Benefit.Popis, PracovniPomer.ID, TypBenefitu.Popis, PB.PocetBodu, TypBenefitu.Kod';        //23..27

        $this->sqlFromPart =
            "PohybBenefitu PB 
                LEFT JOIN S_DruhPohybuBenefitu ON (S_DruhPohybuBenefitu.Kod = PB.DruhPohybu) 
                 LEFT JOIN Osoba ON (PB.RC = Osoba.RC) 
                  LEFT JOIN PracovniPomer ON (PracovniPomer.RC = Osoba.RC) 
                   LEFT JOIN Benefit ON (Benefit.ID = PB.IdBenefitu)
                    LEFT JOIN TypBenefitu ON  (TypBenefitu.Kod = Benefit.TypBenefitu)";


        $this->addField()
            ->setSqlFieldName('PB.ID', 0) ->setHeaderText('ID') ->setOrdering(1)
            ->setCode('col-id');

        $this->addField()
            ->setSqlFieldName('PB.DatumVlozeni', 5) ->setHeaderText('datum_vlozeni') ->setOrdering(1) ->setFormat('date')
            ->setCode('col-datumvlozeni');

        $this ->addField(new JmenoPrijmeniBrowserField($this ->database, 21, 22))
            ->setCode('col-osoba')
            ->setHeaderText('osoba_benefitu') ->setOrdering(1)
            //Pokud ma pravo pouze na sebe, pak se mu zobrazuji pouze vlastni a pole osoba benefitu nema smysl
            ->setVisible($this->pravo_rezervovat->prava > 1 || $this->pravo_vydat->prava > 1 || $this->pravo_vznik->prava > 1 || $this->pravo_schvalit->prava > 1);

        $this->addField(new StavBenefituField($this->database))
            ->setHeaderText('stav_benefitu')
            ->setCode('col-stav');

        $this->addField()
            ->setSqlFieldName('S_DruhPohybuBenefitu.Popis', 3)
            ->setCode('col-druhpohybu')
            ->setHeaderText('druh_pohybu_benefitu') ->setAlign('center') ->setOrdering(1);

        $this->addField(new DatumPlatnostiRezervaceField($this->database))
            ->setSortSQL('DatumOmezeniPlatnosti, DatumRezervaceDo') ->setSortSQLDesc('DatumOmezeniPlatnosti DESC, DatumRezervaceDo DESC')
            ->setCode('col-datumrezervacedo') ->setFormat('date')
            ->setHeaderText('datum_platnosti_rezervace') ->setAlign('center') ->setOrdering(1);

        $this->addField()
            ->setSqlFieldName('PB.Insert_Timestamp', 18) ->setSqlFieldIndex2(17) // jmeno v zavorce
            ->setCode('col-insert_timestamp')
            ->setFormat('date') ->setHeaderText('datum_osoba_vlozeni') ->setAlign('center') ->setOrdering(1);

        $this->defaultSort = -count($this->fields);

        $this->addField(new DatumSchvaleniField($this->database, $this->pravo_schvalit, $this->pravo_vznik))
            ->setSqlFieldName('PB.DatumSchvaleni', 6) ->setSqlFieldIndex2(7) ->setOrdering(1) ->setFormat('date')
            ->setCode('col-datumschvaleni')
            ->setHeaderText('datum_osoba_schvaleni') ->setAlign('center');


        $this->addField(new DatumCerpaniField($this->database, $this->pravo_vydat))
            ->setSqlFieldName('PB.DatumCerpani', 9) ->setSqlFieldIndex2(10) ->setFormat('date')
            ->setCode('col-datumcerpani')
            ->setAlign('center') ->setHeaderText('datum_osoba_cerpani') ->setOrdering(1);

        $this->addField()
            ->setSqlFieldName('PB.Update_Timestamp', 19) ->setSqlFieldIndex2(20) // jmeno v zavorce
            ->setCode('col-update_timestamp')
            ->setFormat('date') ->setHeaderText('posledni_aktualizace') ->setAlign('center') ->setOrdering(1);
    }


    function setFilterForm()
    {
        $this->filterForm = new BenefityFiltrForm($this->database);
    }


//    function renderFilter()
//    {
//        parent::renderFilter();
//        $this->filtrForm->render();
//    }


    /**
     * @param array $values
     * @return bool
     * @throws \Exception
     */
    function enableLine($values)
    {
        if ($this->pravo_rezervovat->lzeZobrazitRC($values[1]) || $this->pravo_schvalit->lzeZobrazitRC($values[1])
            || $this->pravo_vydat->lzeZobrazitRC($values[1]) || $this->pravo_vznik->lzeZobrazitRC($values[1])) {

            return true;
        }

        return false;
    }



    /**
     * @return string
     */
    function getSQLFilter()
    {
        $params = $this ->database ->createParameters();
        //$datum = ($_SESSION['datumFiltruBenefitu']);
        $datum = ($this ->filterForm instanceof BenefityFiltrForm)
            ? $this ->filterForm ->kDatu
            : time();

        $result = 'PracovniPomer.ID = (SELECT Max(PP.ID) FROM PracovniPomer PP Where PP.RC = PB.RC)';

        if ($this ->filterForm instanceof BenefityFiltrForm) {
            if (isset($this ->filterForm ->typBenefitu)) {
                $result .=
                    ($result ? ' AND ' : '') .
                    'TypBenefitu.Kod = ' . $params->addParameter($this->filterForm->typBenefitu);
            }

            if (isset($this ->filterForm ->benefit)) {
                $result .=
                    ($result ? ' AND ' : '') .
                    ' PB.idBenefitu = ' . $params->addParameter((int)$this ->filterForm ->benefit);
            }

            if (isset($this ->filterForm ->druhPohybu)) {
                $result .=
                    ($result ? ' AND ' : '') .
                    'PB.DruhPohybu = ' . $params->addParameter((int)$this ->filterForm ->druhPohybu);
            }

            if ($this ->filterForm ->omezeniBenefitu == 1) {
                // Jen rezervace
                $result .=
                    ($result ? ' AND ' : '') .
                    "( PB.DruhPohybu IN (41))" .
                    " AND ((PB.DatumRezervaceDo IS NOT NULL) AND (PB.DatumRezervaceDo IS NOT NULL AND PB.DatumRezervaceDo <= " . $params->addDateParameter($datum) . "))";


            }  else if ($this ->filterForm ->omezeniBenefitu == 2) {
                // Jen schvalene
                $result .=
                    ($result ? ' AND ' : '') .
                    " (PB.DatumSchvaleni IS NOT NULL AND PB.DatumSchvaleni <= " . $params->addDateParameter($datum) . "" .
                    " AND PB.DruhPohybu NOT IN (51) )";


            } else if ($this ->filterForm ->omezeniBenefitu == 3) {
                //Jen cerpani
                $result .=
                    ($result ? ' AND ' : '') .
                    " (PB.DatumCerpani <= " . $params->addDateParameter($datum) . " )";
            }

        }

        if ($this ->filterForm ->osobaBenefitu) {
            $result .=
                ($result ? ' AND ' : '') .
                'PracovniPomer.ID = ' . $params->addParameter($this ->filterForm ->osobaBenefitu);
        }

        if ($this ->filterForm ->jenNeschvalene){
            $result .=
                ($result ? ' AND ' : '') .
                '(PB.DatumSchvaleni IS NULL AND PB.DruhPohybu IN (11, 21, 41))';
        }

        if ($this->filterForm ->jenNecerpane){
            $result .=
                ($result ? ' AND ' : '') .
                '(PB.DatumSchvaleni IS NOT NULL AND PB.DatumCerpani IS NULL AND PB.DruhPohybu IN (21, 41))';
        }


        return new SqlStatement($result, $params);
    }
}