<?php
/**
 * Roman.Szablatura RON Software 6.11.2018
 */

namespace RonPortal\Benefity;
use Ron\Controls\BrowserField;
use Ron\Controls\ComboBox;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\GuiUtils;
use Ron\TextManager;
use RonPortal\Controls\JmenoPrijmeniBrowserField;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\Security;
use RonPortal\User;


/**
 * Class SablonyBenefituFiltrForm
 * @package Ron\RonPortal\Benefity
 */
class SablonyBenefituFilterForm extends RonPortalFilterForm
{
    /** @var string     Vybrana hodnota pro filtru "typ benefitu" */
    public $typBenefitu = null;

    /** @var int        Vybrana hodnota pro filtr "casova jednotka" */
    public $casovaJednotkaOpakovani = null;

    /** @var  PravaNaOsoby $pravo_sablony */
    public $pravo_sablony;

    /**
     * SablonyBenefituFilterForm constructor.
     * @param Database $database
     * @param string $action
     * @param bool $useGet
     */
    public function __construct(Database $database, $action, $useGet = false)
    {
        parent::__construct($database, $action, $useGet);
        $this ->setUseGet(true, ['menuid' => 'benefity', 'action' => 'sablony']);
    }


    /**
     * @throws \Ron\DB\DBException
     */
    function renderTypBenefitu()
    {
        echo '<div class="filterrow">';

        echo
            '<span class="filtercaption">' .
            TextManager::_('typ_benefitu') .
            '</span>';

        $sql =
            'SELECT DISTINCT TypBenefitu.Kod, TypBenefitu.Popis ' .
            ' FROM TypBenefitu Left JOIN Benefit ON (TypBenefitu.Kod = Benefit.TypBenefitu) LEFT JOIN SablonaRezervaceBenefitu ON (Benefit.ID = SablonaRezervaceBenefitu.IdBenefitu)' .
            ' ORDER BY TypBenefitu.Popis';

        $combo = new ComboBox('typbenefitu');
        $combo->className = 'adsfilterforminput_full';
        $combo->htmlParams = ['data-ron-autosubmit' => 1];
        $combo->emptyRow = TextManager::_('_vse_');
        $combo->loadData($this->database, $sql, $this ->typBenefitu);
        $combo->render();

        echo '</div>';
    }


    function renderTypCasoveJednotkySablonyRezervaceBenefitu()
    {
        echo '<div class="filterrow">';
        echo
            '<span class="filtercaption">' .
            TextManager::_('pouze_casova_jednotka_opakovani') .
            '</span>';

        $combo = new ComboBox('jednotka', [
            1 => TextManager::_('casova_jednotka_benefitu_1'),
            2 => TextManager::_('casova_jednotka_benefitu_2'),
            3 => TextManager::_('casova_jednotka_benefitu_3'),
            4 => TextManager::_('casova_jednotka_benefitu_4'),
            5 => TextManager::_('casova_jednotka_benefitu_5'),
            6 => TextManager::_('casova_jednotka_benefitu_6')]);
        $combo->className = 'form-input adsfilterforminput_full';
        $combo->htmlParams = ['data-ron-autosubmit' => 1];
        $combo->dataTypeComparison = false;
        $combo->showCode = 0;
        $combo->emptyRow = TextManager::_('_vse_');
        $combo->selectedOption = $combo->findOption($this ->casovaJednotkaOpakovani);
        $combo->render();
        echo '</div>';
    }


    function renderVytvoreniSablonyBenefitu()
    {
        echo
            '<div class="filterrow">' .

//            '<span class="filtercaption"><a href="?menuid=benefity">' . TextManager::_('zpet_pohyby') . '</a></span>' .
            '<span class="margin-all-2">' .
            '<a href="?menuid=benefity&action=sablony&formaction=new">' . TextManager::_('vytvorit_sablonu') . '</a>' .
            '</span>' .

            "</div>\n";
    }



    function loadTypBenefitu()
    {
        if (isset($_GET['typbenefitu'])) {
            $this ->typBenefitu = $_GET['typbenefitu'];
            $_SESSION['filtr_typ_benefitu_sablony'] = $this ->typBenefitu;
        } else {
            if (isset($_SESSION['filtr_typ_benefitu_sablony'])) {
                $this ->typBenefitu = $_SESSION['filtr_typ_benefitu_sablony'];
            }
        }
    }



    function loadTypCasoveJednotkySablonyRezervaceBenefitu()
    {
        if (isset($_GET['jednotka'])) {
            $this ->casovaJednotkaOpakovani = ($_GET['jednotka'] == '')
                ? null
                : (int)$_GET['jednotka'];
            $_SESSION['filtr_casova_jednotka_sablony_benefitu'] = $this ->casovaJednotkaOpakovani;
        } else {
            if (isset($_SESSION['filtr_casova_jednotka_sablony_benefitu'])) {
                $this ->casovaJednotkaOpakovani = $_SESSION['filtr_casova_jednotka_sablony_benefitu'];
            }
        }
    }


    /**
     * @throws \Exception
     */
    function loadData()
    {
        $this->loadTypCasoveJednotkySablonyRezervaceBenefitu();
        $this->loadTypBenefitu();
        parent::loadData();
    }


    /**
     * @throws \Ron\DB\DBException
     */
    function renderFilters()
    {
        parent::renderFilters();

        $this->renderTypBenefitu();
        $this->renderTypCasoveJednotkySablonyRezervaceBenefitu();
        $this->renderVytvoreniSablonyBenefitu();
    }
}



/**
 * Class SmazatSablonuField
 * @package Ron\RonPortal\Benefity
 */
class SmazatSablonuField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        //Momentalne na portale lze rusit pouze sve sablony
        if ($rowData[1] == User::getRC()) {
            $text = TextManager::_("odstranit_sablonu_info");

            return
                "<span class='confirmQuestion js' data-ron-confirm-text='$text'" .
                " data-ron-href='?menuid=benefity&action=odstranitsablonu&record=$rowData[0]" . Security::csrfUrlParameter() . "' >" .
                TextManager::_('odstranit_sablonu') .
                "</span>";
        }

        return '';
    }
}



/**
 * Class BrowserSablonyBenefitu
 * @package Ron\RonPortal\Benefity
 */
class SablonyBenefituBrowser extends RonPortalBrowser
{

    /** @var SablonyBenefituFilterForm $filterForm */
    public $filterForm;

    /** @var PravaNaOsoby $pravo_rezervace */
    public $pravo_sablony;


    /**
     * BrowserSablonyBenefitu constructor.
     * @param Database $database
     * @throws \Exception
     */
    public function __construct($database)
    {
        parent::__construct($database, 'benefity');

//        $this->title = TextManager::_('sablony_benefitu');
        $this->code = 'browsersablonybenefitu';
        $this->tableId = 'browsersablonybenefitu';
        $this->browserUrl = '?menuid=benefity&action=sablony';

        $this->pravo_sablony = Prava::vratPrava('benefit_sablona');

        $this->sqlSelectPart =
            'SablonaRezervaceBenefitu.ID, SablonaRezervaceBenefitu.RC, DruhPohybu, IdBenefitu, Poradi,' .   //0..4
            ' PocetBenefitu, PocetBodu, TypDoplatku, SablonaRezervaceBenefitu.DoplatekCastka,' .        //5..8
            ' CasovaJednotka, PocetCasJednotek,' .                                                      //9,10
            ' SablonaRezervaceBenefitu.Insert_UserName, SablonaRezervaceBenefitu.Insert_Timestamp,' .   //11,12
            ' SablonaRezervaceBenefitu.Update_UserName, SablonaRezervaceBenefitu.Update_Timestamp,' .   //12,14
            ' TypKontrolySablony, Benefit.Popis, Osoba.Jmeno, Osoba.Prijmeni';                          //15..18

        $this ->primaryField = 0;

        $this->sqlFromPart =
            'SablonaRezervaceBenefitu LEFT JOIN Benefit ON (Benefit.ID = SablonaRezervaceBenefitu.IdBenefitu)' .
            ' LEFT JOIN Osoba ON (SablonaRezervaceBenefitu.RC = Osoba.RC)';


        $this->addField()
            ->setSqlFieldName('Id', 0) ->setHeaderText('ID') ->setOrdering(1)
            ->setCode('col-id')
            ->setVisible(false);

        $this->addField()
            ->setSqlFieldName('Poradi', 4) ->setHeaderText('Poradi') ->setOrdering(1)
            ->setCode('col-poradi');

        $this->addField()
            ->setSqlFieldName('Benefit.Popis', 16) ->setHeaderText('benefit_popis') ->setOrdering(1)
            ->setCode('col-popis');

        $this ->addField(new JmenoPrijmeniBrowserField($this->database, 17, 18))
            ->setCode('col-osoba')
            ->setHeaderText('osoba_benefitu') ->setOrdering(1)
            ->setVisible($this->pravo_sablony->prava > 1);

        $this->addField()
            ->setSqlFieldName('PocetBenefitu', 5) ->setHeaderText('pocet_benefitu')
            ->setCode('col-pocetbenefitu') ->setOrdering(1) ->setAlign('right');

        $this ->addField(BrowserField::FIELD_TYPE_TEXT)
            ->setText(TextManager::_('detail_sablony'))
            ->setCode('col-detail')
            ->setLink('?menuid=benefity&action=sablony&record=');

        $this ->addField(BrowserField::FIELD_TYPE_SELECT)
            ->setSqlFieldName('CasovaJednotka', 9) ->setHeaderText('casova_jednotka')
            ->setCode('col-casovajednotka')
            ->setOrdering(1)
            ->setOptions([
                1 => TextManager::_('casova_jednotka_benefitu_1'),
                2 => TextManager::_('casova_jednotka_benefitu_2'),
                3 => TextManager::_('casova_jednotka_benefitu_3'),
                4 => TextManager::_('casova_jednotka_benefitu_4'),
                5 => TextManager::_('casova_jednotka_benefitu_5'),
                6 => TextManager::_('casova_jednotka_benefitu_6')
            ]);


        $this->addField()
            ->setSqlFieldName('PocetCasJednotek', 10) ->setHeaderText('opakovani')
            ->setCode('col-pocetcasjednotek')
            ->setOrdering(1) ->setAlign('right');

        $this->addField(BrowserField::FIELD_TYPE_SELECT)
            ->setSqlFieldName('TypKontrolySablony', 15) ->setHeaderText('povolit_vlozit_mene')
            ->setCode('col-typkontrolysablony')
            ->setAlign('center') ->setOrdering(1)
            ->setOptions([0 => TextManager::_('Str_No'), 1 => TextManager::_('Str_Yes')]);

        $this->addField(new SmazatSablonuField($this->database))
            ->setHeaderText('odstraneni_sablony') ->setAlign('center')
            ->setCode('col-odstranit');

        $this ->addField()
            ->setSqlFieldName('SablonaRezervaceBenefitu.Insert_Timestamp', 12) ->setSqlFieldIndex2(11)
            ->setCode('col-insert_timestamp') ->setFormat('date')
            ->setHeaderText('datum_osoba_vlozeni')
            ->setOrdering(1);

        $this->defaultSort = -count($this->fields);

        $this ->addField()
            ->setSqlFieldName('SablonaRezervaceBenefitu.Update_Timestamp', 14) ->setSqlFieldIndex2(13)
            ->setCode('col-update_timestamp') ->setFormat('date')
            ->setHeaderText('posledni_aktualizace')
            ->setOrdering(1);
    }



    function setFilterForm()
    {
        $this->filterForm = new SablonyBenefituFilterForm($this ->database, 'benefity');
    }


    /**
     * @param array $values
     * @return bool
     * @throws \Exception
     */
    function enableLine($values)
    {
        if ($this->pravo_sablony->lzeZobrazitRC($values[1])){
            return true;
        }

        return false;
    }


    /**
     * @return SqlStatement|string
     */
    function getSQLFilter()
    {
        $result = '';
        $params = $this->database->createParameters();

        if ($this ->filterForm instanceof SablonyBenefituFilterForm) {
            if ($this->filterForm->typBenefitu) {
                $result .=
                    ($result ? ' AND ' : '') .
                    ' Benefit.TypBenefitu = ' . $params->addParameter($this->filterForm->typBenefitu);
            }


            if ($this ->filterForm ->casovaJednotkaOpakovani) {
                $result .=
                    ($result ? ' AND ' : '') .
                    ' SablonaRezervaceBenefitu.CasovaJednotka = ' . $params->addParameter((int)$this ->filterForm ->casovaJednotkaOpakovani);
            }
        }

        return $result
            ? new SqlStatement($result, $params)
            : '';
    }
}





