<?php
/**
 * Roman.Szablatura RON Software 27.3.2017
 */

namespace RonPortal\Common;

use Exception;
use Ron\Controls\BrowserField;
use Ron\DB\DBException;
use Ron\DB\SqlStatement;
use Ron\DB\Database;
use Ron\DateUtils;
use Ron\GuiUtils;
use Ron\Http\Csrf;
use Ron\TextManager;
use RonPortal\Controls\FilterFormFilter;
use RonPortal\Controls\FilterFormFilterItem;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\RonPortalSession;
use RonPortal\RonPortalSettings;
use Tracy\Debugger;


/**
 * Class NavstevyFilter
 */
class NavstevyFilter extends RonPortalFilterForm
{
    /** @var  PravaNaOsoby $prava */
    public $prava;

//    /** @var int    0 = od data, 1 = ocekavane, 2 = pritomni */
//    public $omezeniNavstev = 0;
    /** @var FilterFormFilter */
    public $omezeniNavstevFilter;

//    /** @var int */
//    public $pocetDnu = 1;

    public $omezeniVratnice = 1; // 0=vsechny, 1=pouze moje



//    public $jenTypNavstevy = 0;
//    public $typNavstevy = 0;
//
//    public $jenPrijmeniNavstevy = 0;
//    public $prijmeniNavstevy = null;
//
//    public $jenNavstivenaOsoba = 0;
//    public $navstivenaOsoba = null;

//    public $jenFirma = 0;
//    public $firma = null;

//    public $jenDuvod = 0;
//    public $duvod = null;

//    public $jenVozidlo = 0;
//    public $sVozidlem = 0; // bez vozidla/s vozidlem

//    public $filtrovatSchvaleni = 0;
//    public $schvalene = 0;

//    public $jenMedium = 0;
//    public $kodMedia = null;

    //public $zobrazitUzivatelskePoznamky = 0;

    /**
     * NavstevyFilter constructor.
     * @param Database $database
     * @param string $action
     * @param bool $useGet
     */
    public function __construct(Database $database)
    {
        parent::__construct($database, 'navstevy', true);

        $this->prava = Prava::vratPrava('navstevy');
        Prava::nastavObdobiProPrava(time(), time());

        $this ->nastavDatumOdDo(true, true, TextManager::_('Vybrane_obdobi_od'));
        $this ->datumOdDoSessionPrefix = 'navstevy_';
        $this ->defaultDatumOdDo = DateUtils::today();
        $this ->povolitPrazdneDatumDo = true;
        $this ->datumOdDoLinks = [
            [
                'datumOd' => DateUtils::today(),
                'datumDo' => DateUtils::today(),
                'text' => TextManager::_('Navstevy_pro_dnesni_den')
            ],
            [
                'datumOd' => DateUtils::today(),
                'datumDo' => null,
                'text' => TextManager::_('Navstevy_aktualni_a_budouci')
            ],
        ];

        // Vytvori jednotlive filtry
        // vsechny vratnice / pouze moje vratnice
        $filtrVratnice = new FilterFormFilter('omezeniVratnice');
        $filtrVratnice
            ->setSessionName('navstevy_omezenivratnice')
            ->setRowId('filterrow_omezenivratnice')
            ->addItem('0', TextManager::_('vsechnyVratnice'))
            ->addItem( (new FilterFormFilterItem('1', TextManager::_('pouzeMojeVratnice'))) );
        $this ->addFilter($filtrVratnice, true);
        $filtrVratnice ->load($this ->database);


        $this ->omezeniNavstevFilter = new FilterFormFilter('omezeniNavstev');
        $this ->omezeniNavstevFilter
            //->setSessionName('omezeniNavstev')
            ->setRowId('filterrow_omezeninavstev')
            ->addItem('0', TextManager::_('zobrazit_navstevy_za_obdobi'))
            ->addItem('1', TextManager::_('jenOcekavany'))
            ->addItem('2', TextManager::_('pritomni'))
            ->setDefaultValue('0');
        $this ->addFilter($this ->omezeniNavstevFilter, true);

//        //Combo volby dni a datumu
//        $zvolenaDoba =
//            TextManager::_('zvolena_doba') . ': ' .

        //$this->f_formaction = '?menuid=navstevy&';
//        $this->f_classname = 'filtercontainer';

        // jmeno navstivene osoby
        $filter = new FilterFormFilter('osoba', FilterFormFilter::TYPE_EDIT);
        $filter
            ->setCaption(TextManager::_('podle_navstivene_osoby'))
            ->setSqlFilter('Navsteva.JmenoNavstiveneOsoby LIKE ')
            ->setSessionName('navstivenaOsoba')
            ->setRowId('filterrow_osobanavstevy');
        $this ->addFilter($filter);

        // prijmeni navstevy
        $filterPrijmeni = new FilterFormFilter('prijmeni', FilterFormFilter::TYPE_EDIT);
        $filterPrijmeni
            ->setCaption(TextManager::_('podle_prijmeni'))
            ->setSqlFilter('Navsteva.Prijmeni LIKE ')
            ->setSessionName('prijmeniNavstevy')
            ->setRowId('filterrow_prijmeninavstevy');
        $this ->addFilter($filterPrijmeni);

        // typ navstevy (zamestnanec/cizi)
        $filtrTyp = new FilterFormFilter('typ', FilterFormFilter::TYPE_OPTIONS);
        $filtrTyp
            ->setCaption(TextManager::_('podle_typu_navstevy'))
            ->setItems(['' => TextManager::_('_vse_'), 0 => TextManager::_('cizi'), 1 => TextManager::_('vlastni')])
            ->setSqlFilter('Navsteva.TypNavstevy = ')
            ->setSessionName('typNavstevy')
            ->setRowId('filterrow_typnavstevy');
        $this ->addFilter($filtrTyp);

        // firma
        $filterFirma = new FilterFormFilter('firma', FilterFormFilter::TYPE_EDIT);
        $filterFirma
            ->setCaption(TextManager::_('podle_firmy'))
            ->setSqlFilter('Navsteva.Firma LIKE ')
            ->setSessionName('firmaNavstevy')
            ->setRowId('filterrow_firmanavstevy');
        $this ->addFilter($filterFirma);

        // schvaleni navstevy (vse/neschvalene/schvalene)
        $filtrSchvaleni = new FilterFormFilter('schvaleni', FilterFormFilter::TYPE_OPTIONS);
        $filtrSchvaleni
            ->setCaption(TextManager::_('podle_schvaleni'))
            ->setItems([
                '' => new FilterFormFilterItem('', TextManager::_('_vse_')),
                '0' => (new FilterFormFilterItem('0', TextManager::_('navsteva_neschvalena'))) ->setSqlFilter('Navsteva.Schvalil IS NULL'),
                '1' => (new FilterFormFilterItem('1', TextManager::_('navsteva_schvalena'))) ->setSqlFilter('Navsteva.Schvalil IS NOT NULL')
            ])
            // Navsteva.Schvalil IS NOT NULL
            ->setSessionName('schvaleniNavstevy')
            ->setRowId('filterrow_schvaleninavstevy');
        $this ->addFilter($filtrSchvaleni);


        // vyhledani podle ID media
        $params = $this->database->createParameters();
        $sql =
            'SELECT DISTINCT Karta.Kod, Karta.OznaceniMedia' .
            ' FROM Navsteva LEFT JOIN PrirazeniKarty' .
            ' ON (Navsteva.osobaRC = PrirazeniKarty.RC' .
            '  AND Navsteva.osobniCislo = PrirazeniKarty.osobniCislo' .
            '  AND Navsteva.datumNastupu = PrirazeniKarty.datumNastupu)' .
            ' LEFT JOIN Karta ON (Karta.Kod = PrirazeniKarty.kodKarty)' .
            ' WHERE Navsteva.Prichod IS NOT NULL' .
            ' AND Navsteva.Odchod IS NULL AND Karta.Kod IS NOT NULL';

        if ($filtrVratnice ->value  &&  RonPortalSession::getValue('rozdelenivratnice')) {
            $sql .= ' AND Navsteva.Rozdeleni_Vratnice = ' . $params->addParameter(RonPortalSession::getValue('rozdelenivratnice'));
        }

        $sql .= ' ORDER BY Karta.OznaceniMedia';

        $filtrKarta = new FilterFormFilter('medium', FilterFormFilter::TYPE_OPTIONS);
        $filtrKarta
            ->setCaption(TextManager::_('vyhledat_zapujcena_media'))
            ->setItems(['' => new FilterFormFilterItem('', TextManager::_('_vse_'))])
            ->setOptionsSql(new SqlStatement($sql, $params))
            ->setSessionName('mediumNavstevy')
            ->setRowId('filterrow_mediumnavstevy')
            ->setCaptionHtmlAttributes(['title' => TextManager::_('filtr_zapujceneho_media')]);
        $this ->addFilter($filtrKarta);

        $filtrDuvod = new FilterFormFilter('duvod', FilterFormFilter::TYPE_OPTIONS);
        $filtrDuvod
            ->setCaption(TextManager::_('podle_duvodu_navstevy'))
            ->setItems(['' => new FilterFormFilterItem('', TextManager::_('_vse_'))])
            ->setOptionsSql('SELECT Kod, Popis FROM DuvodNavstevy WHERE %FILTER% ORDER BY %ORDER%', '1 = 1', 'Popis')
            ->setSessionName('duvodNavstevy')
            ->setRowId('filterrow_duvodnavstevy')
            ->setSqlFilter('Navsteva.duvodNavstevy2 = ');
        $this ->addFilter($filtrDuvod);

        $filtrSchvaleni = new FilterFormFilter('svozidlem', FilterFormFilter::TYPE_OPTIONS);
        $filtrSchvaleni
            ->setCaption(TextManager::_('podle_typu_vozidla'))
            ->setItems([
                '' => new FilterFormFilterItem('', TextManager::_('_vse_')),
                '0' => (new FilterFormFilterItem('0', TextManager::_('bez_vozidla'))) ->setSqlFilter('Navsteva.Vozidlo IS NULL'),
                '1' => (new FilterFormFilterItem('1', TextManager::_('s_vozidlem'))) ->setSqlFilter('Navsteva.Vozidlo IS NOT NULL')
            ])
            // Navsteva.Schvalil IS NOT NULL
            ->setSessionName('FiltrNavstevySVozidlem')
            ->setRowId('filterrow_vozidlonavstevy]');
        $this ->addFilter($filtrSchvaleni);

        $filtrZobrazitPoznamky = new FilterFormFilter('poznamky', FilterFormFilter::TYPE_CHECKBOX);
        $filtrZobrazitPoznamky
            ->setSessionName('uzivatelskePoznamkyNavstev')
            ->setCaption2(TextManager::_('navstevyUzivatelskePoznamky'));
        $this ->addFilter($filtrZobrazitPoznamky);

//        GuiUtils::renderCheckbox('uzivatelskePoznamky', ,
//            $this ->zobrazitUzivatelskePoznamky,
//            '', 'clickAutoSubmit');

        //Nastaveni timestamp (pokud je datum ve spatnem formatu nebo neni v session nebo neni, nastavi se na aktu. datum)
//        if (isset($_POST['datum'])){
//            try {
//                $this->timestamp = DateUtils::dateStringToTimestamp($_POST['datum']);
//            } catch(Exception $e) {
//                MessageManager::addErrorText( 'Bad_time_format' );
//                $this->timestamp = time();
//            }
//        } else {
//            if (isset($_SESSION['datum'])) {
//                $this->timestamp = $_SESSION['datum'];
//            } else {
//                $this->timestamp = time();
//            }
//        }

//        if (isset($_SESSION['navstevy_pocetdnu'])) {
//            $this ->pocetDnu = $_SESSION['navstevy_pocetdnu'];
//        }

//        $_SESSION['datum'] = DateUtils::startOfTheDay($this->timestamp);
//        $_SESSION['datumdodata'] = ((DateUtils::incDay($_SESSION['datum'], $this ->pocetDnu)) - 1);
    }

//    /**
//     * @param $pocetDni
//     * @return string
//     */
//    private function textTitulkuZaznamyOd($pocetDni)
//    {
//        switch ($pocetDni){
//            case 1:
//                $titulek = TextManager::_('1_den');
//                break;
//
//            case 10:
//                $titulek = TextManager::_('10_dni');
//                break;
//
//            case 20:
//                $titulek = TextManager::_('20_dni');
//                break;
//
//            case 31:
//                $titulek = TextManager::_('31_dni');
//                break;
//
//            default:
//                $titulek = "";
//        }
//        return $titulek;
//    }

//    function renderOmezeniNavstev()
//    {
//        echo
//            "<div class='filterrow {$this ->omezeniNavstevFilter ->rowClass}' id='{$this ->omezeniNavstevFilter ->rowId}'>" .
//            '<span class="filtercaption">' . $this ->omezeniNavstevFilter ->caption . '</span>';
//        $this ->omezeniNavstevFilter ->render();
//        echo
//            "</div>\n";

//        echo '<div class="filterrow">';
//        echo
//            '<span class="filtercaption">' .
//            // jen ocekavane
//            '<label class="margin-right-2">' .
//            "<input type='radio' class='clickAutoSubmit' name='omezeniNavstev' value='0' " . ($this ->omezeniNavstev == 0 ? 'checked="checked"' : "") . '> ' .
//            TextManager::_('jenOcekavany') .
//            '</label>' .
//            // pritomne
//            '<label>' .
//            "<input type='radio' class='clickAutoSubmit' name='omezeniNavstev' value='1' " . ($this ->omezeniNavstev == 1 ? 'checked="checked"' : "") . '> ' .
//            TextManager::_('pritomni') .
//            '</label>' .
//            '</span>';

//        //Combo volby dni a datumu
//        $zvolenaDoba =
//            TextManager::_('zvolena_doba') . ': ' .
//            date('j.n.Y', $this ->datumOd) . ' - ' . date('j.n.Y', $this ->datumDo);
//
//        echo "<span title='$zvolenaDoba'>";

//        echo
//            '<label class="zaznamy_od">' .
//            "<input type='radio' class='clickAutoSubmit' name='omezeniNavstev' value='2' " .
//            ($this ->omezeniNavstev == 2 ? 'checked="checked"' : "") . '> ' .
//            TextManager::_('zobrazit_navstevy_za') .
//            '</label> ';

//        if ($this ->omezeniNavstev == 2) {
//            $combo = new ComboBox('vyberpocetdnu', [
//                    1 => TextManager::_('1_den'),
//                    10 => TextManager::_('10_dni'),
//                    20 => TextManager::_('20_dni'),
//                    31 => TextManager::_('31_dni')]
//            );
//            $combo->className = 'vyber_dni_combo adsfilterforminput_kod';
//            $combo->htmlParams = 'data-ron-autosubmit="1"';
//            $combo->selectedOption = $combo->findOption($this ->pocetDnu ? $this ->pocetDnu : 1);
//            $combo->render();
//
//            $pouzeDnesniDen = TextManager::_('pouze_dnesni_den');
//            echo
//                "<span class='margin-left-2' title='$pouzeDnesniDen'>" .
//                TextManager::_('Od') . ' ';
//            DateInput::renderDateSelect('datum', date('j.n.Y', $this->datumOd), true);
//            echo
//                '<i class="inline-glyph glyph-button fa glyph-today setValue" data-selector="input[name=\'datum\']" data-value="' . date('j.n.Y') . '" data-submit="1"></i>' .
//                '</span>';
//        } else {
//
//            $text =
//                "<input type='text' size='10' class='input_pocet_dni' disabled='disabled' value='" . $this->textTitulkuZaznamyOd($this ->pocetDnu) . "'/>" .
//                ' ' . TextManager::_('od') . ' ' .
//                "<input type='text' size='10' class='input_pocet_dni' disabled='disabled' value='" . date('j.n.Y', $this ->datumOd) . "'/>";
//            echo $text;
//        }
//        echo '</span>';
//
//        echo '</div>';


//        echo
//            '<div class="filterrow" id="filterrow_omezenivratnice">' .
//            // vsechny vratnice
//            '<label class="margin-right-2">' .
//            "<input type='radio' class='clickAutoSubmit' name='omezeniVratnice' value='0' " . (!$this ->omezeniVratnice ? 'checked="checked"' : '') . '> ' .
//            TextManager::_('vsechnyVratnice') .
//            '</label>' .
//            // jen aktualni vratnice
//            '<label>' .
//            "<input type='radio' class='clickAutoSubmit' name='omezeniVratnice' value='1' " . ($this ->omezeniVratnice? 'checked="checked"' : '') . '> ' .
//            TextManager::_('pouzeMojeVratnice') .
//            '</label>' .
//            '</div>';
//    }

//    function renderTypNavstevy()
//    {
//        echo '<div class="filterrow" id="filterrow_typnavstevy">';
//        echo '<span class="filtercaption">';
//        GuiUtils::renderCheckbox('jenTypNavstevy', TextManager::_('podle_typu_navstevy'), $this ->jenTypNavstevy,
//            ['class' => 'clickAutoSubmit', 'id' => 'jen_typ_navstevy_checkbox']);
//        echo '</span>';
//
//        $combo = new ComboBox('typNavstevy', [
//            new Option(0, TextManager::_('cizi')),
//            new Option(1, TextManager::_('vlastni'))
//        ]);
//        $combo->className = 'adsfilterforminput_full';
//        $combo->htmlParams = ['data-ron-autosubmit' => '1', 'data-ron-autosubmit-check' => '#jen_typ_navstevy_checkbox'];
//        $combo->selectedOption = $combo->findOption($this ->typNavstevy); // $_SESSION['typNavstevy'] != '' ? (int)$_SESSION['typNavstevy'] : 0);
//        $combo->showCode = 0;
//        $combo->render();
//        echo '</div>';
//    }

//    /**
//     * @throws DBException
//     */
//    function renderDuvodNavstevy()
//    {
//        echo '<div class="filterrow" id="filterrow_duvodnavstevy">';
//        echo '<span class="filtercaption">';
//        GuiUtils::renderCheckbox('jenDuvodNavstevy', TextManager::_('podle_duvodu_navstevy'), $this ->jenDuvod,
//            ['class' => 'clickAutoSubmit', 'id' => 'jen_duvod_navstevy_checkbox']);
//        echo '</span>';
//
//        $combo = new ComboBox('duvodNavstevy');
//        $combo->className = 'adsfilterforminput_full';
//        $combo->htmlParams = ['data-ron-autosubmit' => '1', 'data-ron-autosubmit-check' => '#jen_duvod_navstevy_checkbox'];
//        $combo->loadData($this->database,
//            'SELECT Kod, Popis FROM DuvodNavstevy ORDER BY Popis',
//            $this ->duvod);
//        $combo->render();
//
//        echo '</div>';
//    }

//    /**
//     * @throws DBException
//     */
//    function renderOsobaNavstevy()
//    {
//        echo '<div class="filterrow" id="filterrow_osobanavstevy">';
//
//        // checkbox
//        echo '<span class="filtercaption">';
//        GuiUtils::renderCheckbox('jenOsoba', TextManager::_('podle_navstivene_osoby'), $this ->jenNavstivenaOsoba,
//            ['class' => 'clickAutoSubmit', 'id' => 'jen_navstivena_osoba_checkbox']);
//        echo '</span>';
//
//        echo '<input name="osoba" class="adsfilterforminput_full form-control" value="' . StringUtils::htmlTextConvert($this->navstivenaOsoba) . '"/>';
//
////        $params = null;
////        if ($this ->omezeniVratnice  &&  RonPortalSession::getValue('rozdelenivratnice') != '') {
////            $params = $this->database->createParameters();
////            $sql =
////                'SELECT DISTINCT JmenoNavstiveneOsoby, JmenoNavstiveneOsoby Jmeno FROM Navsteva' .
////                ' WHERE Navsteva.Rozdeleni_Vratnice = ' . $params->addParameter(RonPortalSession::getValue('rozdelenivratnice')) .
////                ' ORDER BY JmenoNavstiveneOsoby';
////        } else {
////            $sql =
////                'SELECT DISTINCT JmenoNavstiveneOsoby, JmenoNavstiveneOsoby Jmeno FROM Navsteva ORDER BY jmenoNavstiveneOsoby';
////        }
////
////        $combo = new ComboBox('osoba');
////        $combo->className = 'adsfilterforminput_full';
////        $combo->htmlParams = ['data-ron-autosubmit' => '1', 'data-ron-autosubmit-check' => '#jen_navstivena_osoba_checkbox'];
////        $combo->showCode = 0;
////        $combo->loadData($this->database, $sql, $this ->navstivenaOsoba, $params);
////        $combo->render();
//        echo '</div>';
//    }

//    /**
//     * @throws DBException
//     */
//    function renderFirma()
//    {
//        echo '<div class="filterrow"  id="filterrow_firmanavstevy">';
//        echo '<span class="filtercaption">';
//        GuiUtils::renderCheckbox('jenFirma', TextManager::_('podle_firmy'), $this ->jenFirma,
//            ['class' => 'clickAutoSubmit', 'id' => 'jen_firma_checkbox']);
//        echo '</span>';
//
//        echo '<input name="firma" class="adsfilterforminput_full form-control" value="' . StringUtils::htmlTextConvert($this->firma) . '"/>';
//
////        $params = null;
////        if ($this ->omezeniVratnice  &&  RonPortalSession::getValue('rozdelenivratnice') != '') {
////            $params = $this->database->createParameters();
////            $sql =
////                'SELECT DISTINCT Firma FROM Navsteva' .
////                ' WHERE Navsteva.Rozdeleni_Vratnice = ' . $params->addParameter(RonPortalSession::getValue('rozdelenivratnice')) .
////                ' ORDER BY Firma';
////        } else {
////            $sql =
////                'SELECT DISTINCT Firma FROM Navsteva ORDER BY Firma';
////        }
////        $combo = new ComboBox('firma');
////        $combo->className = 'adsfilterforminput_full';
////        $combo->htmlParams = ['data-ron-autosubmit' => '1', 'data-ron-autosubmit-check' => '#jen_firma_checkbox'];
////        $combo->showCode = 0;
////        $combo ->captionIndex = 0;
////        $combo->loadData($this->database, $sql, $this ->firma, $params);
////        $combo->render();
//
//        echo '</div>';
//    }


//    /**
//     * @throws Exception
//     */
//    function renderSchvaleniNavstevy()
//    {
//        echo '<div class="filterrow" id="filterrow_schvaleninavstevy">';
//        echo '<span class="filtercaption">';
//        GuiUtils::renderCheckbox('jenSchvaleno', TextManager::_('podle_schvaleni'),
//            $this ->filtrovatSchvaleni,
//            ['id' => 'checkbox_jen_schvaleno'], 'clickAutoSubmit');
//        echo '</span>';
//
//        $combo = new ComboBox(
//            'schvaleniNavstevy',
//            [0 => TextManager::_('navsteva_neschvalena'), 1 => TextManager::_('navsteva_schvalena')]
//        );
//        $combo->className = 'adsfilterforminput_full';
//        //$combo->htmlParams = 'onChange="if (jenSchvaleno.checked) submit();"';
//        $combo->htmlParams = 'data-ron-autosubmit="1" data-ron-autosubmit-check="#checkbox_jen_schvaleno"';
//        $combo->selectedOption = $combo->findOption($this ->schvalene ? 1 : 0);
//        $combo->render();
//        echo '</div>';
//    }

//    /**
//     * @throws DBException
//     */
//    function renderPrijmeni()
//    {
//        echo '<div class="filterrow">';
//
//        echo '<span class="filtercaption">';
//        GuiUtils::renderCheckbox('jenPrijmeni', TextManager::_('podle_prijmeni'),
//            $this ->jenPrijmeniNavstevy,
//            ['id' => 'checkbox_jen_prijmeni'], 'clickAutoSubmit');
//        echo '</span>';
//
//        echo '<input name="prijmeni" class="adsfilterforminput_full form-control" value="' . StringUtils::htmlTextConvert($this->prijmeniNavstevy) . '"/>';
//
////        $params = null;
////        if ($this ->omezeniVratnice  &&  RonPortalSession::getValue('rozdelenivratnice') != '') {
////            $params = $this->database->createParameters();
////            $sql =
////                'SELECT DISTINCT prijmeni, prijmeni p FROM Navsteva' .
////                ' WHERE Navsteva.Rozdeleni_Vratnice = ' . $params->addParameter(RonPortalSession::getValue('rozdelenivratnice')) .
////                ' ORDER BY p';
////        } else {
////            $sql =
////                'SELECT DISTINCT prijmeni, prijmeni p FROM Navsteva ORDER BY p';
////        }
////        $combo = new ComboBox('prijmeni');
////        $combo->className = 'adsfilterforminput_full';
////        $combo->htmlParams = 'data-ron-autosubmit="1" data-ron-autosubmit-check="#checkbox_jen_prijmeni"';
////        //$combo->htmlParams = 'onChange="if (jenPrijmeni.checked) submit();"';
////        $combo->showCode = 0;
////        $combo->loadData($this->database, $sql, $this ->prijmeniNavstevy, $params);
////        $combo->render();
//
//
//        echo '</div>';
//    }


//    /**
//     * @throws DBException
//     */
//    function renderMedium()
//    {
//        echo '<div class="filterrow">';
//        $titulekFiltruZapujcenehoMedia = TextManager::_('filtr_zapujceneho_media');
//        echo "<span class='filtercaption' title='$titulekFiltruZapujcenehoMedia'>";
//        GuiUtils::renderCheckbox('jenMedium', TextManager::_('vyhledat_zapujcena_media'), $this ->jenMedium,
//            ['id' => 'jen_medium_checkbox'], 'clickAutoSubmit');
//        echo '</span>';
//
//        $params = $this->database->createParameters();
//
//        $sql =
//            'SELECT DISTINCT Karta.Kod, Karta.OznaceniMedia' .
//            ' FROM Navsteva LEFT JOIN PrirazeniKarty' .
//            ' ON (Navsteva.osobaRC = PrirazeniKarty.RC' .
//            '  AND Navsteva.osobniCislo = PrirazeniKarty.osobniCislo' .
//            '  AND Navsteva.datumNastupu = PrirazeniKarty.datumNastupu)' .
//            ' LEFT JOIN Karta ON (Karta.Kod = PrirazeniKarty.kodKarty)' .
//            ' WHERE Navsteva.Prichod IS NOT NULL' .
//            ' AND Navsteva.Odchod IS NULL AND Karta.Kod IS NOT NULL';
//
//        if ($this ->omezeniVratnice  &&  RonPortalSession::getValue('rozdelenivratnice') != '') {
//            $sql .= ' AND Navsteva.Rozdeleni_Vratnice = ' . $params->addParameter(RonPortalSession::getValue('rozdelenivratnice'));
//        }
//
//        $sql .= ' ORDER BY Karta.OznaceniMedia';
//
//        $combo = new ComboBox('medium');
//        $combo->className = 'adsfilterforminput_full';
//        $combo->emptyRow = TextManager::_("nevybrano");
//        $combo->htmlParams = ['data-ron-autosubmit' => '1', 'data-ron-autosubmit-check' => '#jen_medium_checkbox'];
//        $combo->showCode = 0;
//        $combo->loadData($this->database, $sql, $this ->kodMedia, $params);
//        $combo->render();
//        echo '</div>';
//    }


//    public function renderUzivatelskePolozkyVPoznamceBrowseru()
//    {
//        echo '<div class="filterrow">';
//        echo '<label>';
//        GuiUtils::renderCheckbox('uzivatelskePoznamky', TextManager::_('navstevyUzivatelskePoznamky'),
//            $this ->zobrazitUzivatelskePoznamky,
//            '', 'clickAutoSubmit');
//        echo '</label>';
//        echo '</div>';
//    }

//    function renderVozidloNavstevy()
//    {
//        echo '<div class="filterrow">';
//        echo '<span class="filtercaption">';
//        GuiUtils::renderCheckbox('jenVozidloNavstevy', TextManager::_('podle_typu_vozidla'), $this ->jenVozidlo,
//            ['id' => 'jen_vozidlo_navstevy_checkbox'], 'clickAutoSubmit');
//        echo '</span>';
//
//        $combo = new ComboBox('typVozidla', [0 => TextManager::_("bez_vozidla"), 1 => TextManager::_("s_vozidlem")]);
//        $combo->className = 'adsfilterforminput_full';
//        $combo->htmlParams = ['data-ron-autosubmit' => '1', 'data-ron-autosubmit-check' => '#jen_vozidlo_navstevy_checkbox'];
//        $combo->selectedOption = $combo->findOption($this ->sVozidlem ? 1 : 0);
//        $combo->render();
//
//        echo '</div>';
//    }


//    private function loadTypNfavstevy()
//    {
//        if (isset($_POST['adsfilterform'])) {
//            $this ->jenTypNavstevy = isset($_POST['jenTypNavstevy']) ? (int)$_POST['jenTypNavstevy'] : 0;
//            $this ->typNavstevy = (int)$_POST['typNavstevy'];
//
//            $_SESSION['jenTypNavstevy'] = $this ->jenTypNavstevy;
//            $_SESSION['typNavstevy'] = $this ->typNavstevy;
//        } else {
//            $this ->jenTypNavstevy = (int)RonPortalSession::getValue('jenTypNavstevy');
//            $this ->typNavstevy = RonPortalSession::getValue('typNavstevy');
//        }
//    }

//    private function loadOsobaNavstevy()
//    {
//        if (isset($_POST['adsfilterform'])) {
//            $this ->jenNavstivenaOsoba = isset($_POST['jenOsoba']) ? (int)$_POST['jenOsoba'] : 0;
//            $this ->navstivenaOsoba = $_POST['osoba'];
//            $_SESSION['jenNavstivenaOsoba'] = $this ->jenNavstivenaOsoba;
//            $_SESSION['navstivenaOsoba'] = $this ->navstivenaOsoba;
//        } else {
//            $this ->jenNavstivenaOsoba = (int)RonPortalSession::getValue('jenNavstivenaOsoba');
//            $this ->navstivenaOsoba = RonPortalSession::getValue('navstivenaOsoba');
//        }
//    }

//    private function loadUzivatelskePoznamky()
//    {
//        if (isset($_POST['adsfilterform'])) {
//            $this ->zobrazitUzivatelskePoznamky = isset($_POST['uzivatelskePoznamky']) ? (int)$_POST['uzivatelskePoznamky'] : 0;
//
//            $_SESSION['uzivatelskePoznamkyNavstev'] = $this ->zobrazitUzivatelskePoznamky;
//        } else {
//            $this ->zobrazitUzivatelskePoznamky = (int)RonPortalSession::getValue('uzivatelskePoznamkyNavstev');
//        }
//    }

//    private function loadVozidloNavstevy()
//    {
//        if (isset($_POST['adsfilterform'])) {
//            $this ->jenVozidlo = isset($_POST['jenVozidloNavstevy']) ? (int)$_POST['jenVozidloNavstevy'] : 0;
//            $this ->sVozidlem = $_POST['typVozidla'] ? 1 : 0;
//
//            $_SESSION['jenVozidloNavstevy'] = $this ->jenVozidlo;
//            $_SESSION['vozidloNavstevy'] = $this ->sVozidlem;
//        } else {
//            $this ->jenVozidlo = (int)RonPortalSession::getValue('jenVozidloNavstevy');
//            $this ->sVozidlem = (int)RonPortalSession::getValue('vozidloNavstevy');
//        }
//    }

//    private function loadDuvodNavstevy()
//    {
//        if (isset($_POST['adsfilterform'])) {
//            $this ->jenDuvod = isset($_POST['jenDuvodNavstevy']) && (int)$_POST['jenDuvodNavstevy'];
//            $this ->duvod = $_POST['duvodNavstevy'];
//
//            $_SESSION['jenDuvodNavstevy'] = $this ->jenDuvod;
//            $_SESSION['duvodNavstevy'] = $this ->duvod;
//        } else {
//            $this ->jenDuvod = (int)RonPortalSession::getValue('jenDuvodNavstevy');
//            $this ->duvod = RonPortalSession::getValue('duvodNavstevy');
//        }
//    }

//    private function loadFirma()
//    {
//        if (isset($_POST['adsfilterform'])) {
//            $this ->jenFirma = isset($_POST['jenFirma']) ? (int)$_POST['jenFirma'] : 0;
//            $this ->firma = $_POST['firma'];
//            $_SESSION['jenFirmaNavstevy'] = $this ->jenFirma;
//            $_SESSION['firmaNavstevy'] = $this ->firma;
//        } else {
//            $this ->jenFirma = (int)RonPortalSession::getValue('jenFirmaNavstevy');
//            $this ->firma = RonPortalSession::getValue('firmaNavstevy');
//        }
//    }


//    private function loadSchvaleniNavstevy()
//    {
//        if (isset($_POST['adsfilterform'])) {
//            $this ->filtrovatSchvaleni = isset($_POST['jenSchvaleno']) && (int)$_POST['jenSchvaleno'];
//            $this ->schvalene = (int)$_POST['schvaleniNavstevy'];
//
//            $_SESSION['filtrovatSchvaleniNavstevy'] = $this ->filtrovatSchvaleni;
//            $_SESSION['schvaleniNavstevy'] = $this ->schvalene;
//        } else {
//            $this ->filtrovatSchvaleni = (int)RonPortalSession::getValue('filtrovatSchvaleniNavstevy');
//            $this ->schvalene = (int)RonPortalSession::getValue('schvaleniNavstevy');
//        }
//    }


//    private function loadPrijmeni()
//    {
//        if (isset($_POST['adsfilterform'])) {
//            $this ->jenPrijmeniNavstevy = (isset($_POST['jenPrijmeni']) && (int)$_POST['jenPrijmeni']) ? 1 : 0;
//            $this ->prijmeniNavstevy = $_POST['prijmeni'];
//
//            $_SESSION['jenPrijmeniNavstevy'] = $this ->jenPrijmeniNavstevy;
//            $_SESSION['prijmeniNavstevy'] = $this ->prijmeniNavstevy;
//        } else {
//            $this ->jenPrijmeniNavstevy = (int)RonPortalSession::getValue('jenPrijmeniNavstevy');
//            $this ->prijmeniNavstevy = RonPortalSession::getValue('prijmeniNavstevy');
//        }
//    }

//    private function loadMedium()
//    {
//        if (isset($_POST['adsfilterform'])) {
//            $this ->jenMedium = isset($_POST['jenMedium']) && (int)$_POST['jenMedium'];
//            $this ->kodMedia = $_POST['medium'];
//
//            $_SESSION['jenMediumNavstevy'] = $this ->jenMedium;
//            $_SESSION['mediumNavstevy'] = $this ->kodMedia;
//        } else {
//            $this ->jenMedium = (int)RonPortalSession::getValue('jenMediumNavstevy');
//            $this ->kodMedia = RonPortalSession::getValue('mediumNavstevy');
//        }
//    }


//    private function loadOmezeniNavstev()
//    {
//        if (isset($_GET['omezeniNavstev'])) {
//            $this ->omezeniNavstev = (int)$_GET['omezeniNavstev'];
//            $_SESSION['omezeniNavstev'] = $this ->omezeniNavstev;
//        } else if (isset($_SESSION['omezeniNavstev'])) {
//            $this ->omezeniNavstev = $_SESSION['omezeniNavstev'];
//        }
//
//        if (isset($_GET['vyberpocetdnu'])  &&  $_GET['vyberpocetdnu'] != '') {
//            $this ->pocetDnu = (int)$_GET['vyberpocetdnu'];
//            $_SESSION['navstevy_pocetdnu'] = $this ->pocetDnu;
//        } else if (isset($_SESSION['navstevy_pocetdnu'])) {
//            $this ->pocetDnu = $_SESSION['navstevy_pocetdnu'];
//        }
//
//        if (isset($_GET['datum'])) {
//            try{
//                //V pripade ze se zada neexistujici den v mesici tak skript skoncil chybou, ze ktere se jen tak neda odejit - proto jsem toto osetril
//                // a nastavuji v pripade chyby aktualni datum
//                $this ->datumOd = DateUtils::dateStringToTimestamp($_GET['datum']);
//            } catch (Exception $ex) {
//                MessageManager::addErrorText( 'Bad_time_format' );
//                $this ->datumOd = DateUtils::today();
//            }
//            $_SESSION['navstevy_datumod'] = $this ->datumOd;
//        } else if (isset($_SESSION['navstevy_datumod'])) {
//            $this ->datumOd = $_SESSION['navstevy_datumod'];
//        } else {
//            $this ->datumOd = DateUtils::today();
//        }
//
//        $this ->datumDo = DateUtils::incDay($this ->datumOd, $this->pocetDnu) - 1;
//
////        $_SESSION['datum'] = DateUtils::startOfTheDay($this->timestamp);
////        $_SESSION['datumdodata'] = ((DateUtils::incDay($_SESSION['datum'], $this->pocetDnu)) - 1);
//    }


//    private function loadOmezeniVratnice()
//    {
//        if (isset($_GET['omezeniVratnice'])) {
//            $this ->omezeniVratnice = (int)$_GET['omezeniVratnice'];
//            $_SESSION['omezeniVratnice'] = $this ->omezeniVratnice;
//        } else if (isset($_SESSION['omezeniVratnice'])) {
//            $this ->omezeniVratnice = (int)RonPortalSession::getValue('omezeniVratnice');
//        }
//    }

    function loadData()
    {
        //$this->loadOmezeniVratnice();
//        $this ->omezeniNavstevFilter ->load($this ->database);

//        $this->loadOmezeniNavstev();
//        $this->loadTypNavstevy();
//        $this->loadOsobaNavstevy();
//        $this->loadPrijmeni();
//        $this->loadFirma();
//        $this->loadSchvaleniNavstevy();
//        $this->loadMedium();
//        $this->loadDuvodNavstevy();
//        $this->loadVozidloNavstevy();
//        $this->loadUzivatelskePoznamky();

        parent::loadData();
        $this ->omezeniVratnice = (int)$this ->getFilterValue('omezeniVratnice');
    }


//    /**
//     * @throws DBException
//     * @throws Exception
//     */
//    function renderFilters()
//    {
//        $this->renderOmezeniNavstev();
//        parent::renderFilters();
//        $this->renderOsobaNavstevy();
//        $this->renderPrijmeni();
//        $this->renderTypNavstevy();
//        $this->renderFirma();
//        $this->renderSchvaleniNavstevy();
//        $this->renderMedium();
//        $this->renderDuvodNavstevy();
//        $this->renderVozidloNavstevy();
//        $this->renderUzivatelskePolozkyVPoznamceBrowseru();
//    }
}


/**
 * Class TypNavstevy
 */
class TypNavstevy extends BrowserField
{

    /**
     * TypNavstevy constructor.
     * @param Database $database
     * @param $options
     */
    public function __construct($database, $options)
    {
        parent::__construct($database, $options);
        $this->header = TextManager::_('typ_navstevy');
        $this->ordering = 1;
        $this->align = 'center';
        $this->sortSQL = 'TypNavstevy, Navsteva.OcekavanyPrichod, prichod';
        $this->sortSQLDesc = 'TypNavstevy DESC, Navsteva.OcekavanyPrichod DESC, prichod DESC';
        $this->sqlFieldIndex = 1;
        $this->sqlFieldName = 'TypNavstevy';
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        return
            '<a href="?menuid=navstevy&formaction=edit&record=' . (int)$rowData[0] . '" title="' . TextManager::_($rowData[1] ? 'vlastni' : 'cizi') . '">' .
            ($rowData[1] ? '<strong>Z</strong>' : '-->') .
            '</a>';
    }
}

/**
 * Class CasNavstevy
 */
class CasNavstevyBrowserField extends BrowserField
{
    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return int|string
     * @throws DBException
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = parent::getViewText($value, $rowNumber, $recordId, $rowData);

        $time = $this->database->stringToTimestamp($value);
        if (DateUtils::startOfTheDay($time) == DateUtils::today()) {
            $result =
                '<strong>' . $result . '</strong>';
        }

        return $result;
    }
}


/**
 * Class PotvrzeniOdchoduNavstevyField
 */
class PotvrzeniOdchoduNavstevyField extends BrowserField
{
    var $lzeZapsatOdchod;

    /**
     * PotvrzeniPrijetiField constructor.
     * @param Database $database
     * @param array|null $options
     */
    public function __construct($database, $options)
    {
        parent::__construct($database, $options);
        $this->lzeZapsatOdchod = !Prava::vratPrava('navstevy') ->priznak;
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        // Jestlize je zadan prichod a odchod jeste ne
        if (!$rowData[$this ->sqlFieldIndex]  &&  $this ->lzeZapsatOdchod  &&  $rowData['col-prichod']) {
            return GuiUtils::changeLocationQuest(
                TextManager::_('vlozit_odchod_navstevy'), TextManager::_('opravdu_ukoncit_navstevu'),
                '?menuid=navstevy&action=potvrzeniodchodu&id=' . $rowData[0]);
        }

        return parent::getViewText($value, $rowNumber, $recordId, $rowData);
    }
}


/**
 * Class NavstevuSchvalilaOsoba
 */
class NavstevuSchvalilaOsoba extends BrowserField
{
    /** @var PravaNaOsoby $pravo_schvaleni */
    private $pravo_schvaleni;


    /**
     * NavstevuSchvalilaOsoba constructor.
     * @param Database $database
     */
    public function __construct($database)
    {
        parent::__construct($database);
        $this->pravo_schvaleni = Prava::vratPrava('navstevy_schvaleni');
    }


    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if (!$value) {
            // jeste neschvaleno
            $rcNavstiveneOsoby = $rowData[2]; // Navsteva.NavstivenaOsobaRC
            $maPravoNaOsobu = $this->pravo_schvaleni->lzeZobrazitRC($rcNavstiveneOsoby);
            if ($maPravoNaOsobu && $rowData['col-ocekavany']) {
                return GuiUtils::changeLocationQuest(TextManager::_('Schvalit'), TextManager::_('opravdu_schvalujete_navstevu'),
                    ' ?menuid=navstevy&action=schvaleninavstevy&id=' . $rowData[0] .
                    '&browser=1' . Csrf::csrfUrlParameter());
            }

            // neni pravo schvalovat
            return '';
        }

        return parent::getViewText($value, $rowNumber, $recordId, $rowData);
    }
}



/**
 * Class NavstevyBrowser
 */
class NavstevyBrowser extends RonPortalBrowser
{

    /** @var  PravaNaOsoby $prava */
    private $prava;

    /**
     * NavstevyBrowser constructor.
     * @param Database $database
     * @throws Exception
     */
    public function __construct(Database $database)
    {
        parent::__construct($database, 'navstevy');

        Prava::nastavObdobiProPrava(time(),time());
        $this->prava = Prava::vratPrava('navstevy');

        $this->code = 'browsernavstevy';
        $this->tableId = 'browsernavstevy';

        $this ->maxPageRows = 200;

        $omezeniVratnice = (int)($_GET['omezeniVratnice'] ?? $_SESSION['navstevy_omezenivratnice'] ?? 0);
        $this->title = TextManager::_('navstevy') . ' ' .
            ($omezeniVratnice == 1
                ? $this->rozdeleniVratnice($database)
               : (!$omezeniVratnice ? TextManager::_('vsechnyVratnice') : '')
            );

        $this->sqlSelectPart =
            'Navsteva.Id, TypNavstevy, Navsteva.NavstivenaOsobaRC';   //0 - 2

        $this->primaryField = 0;

        $this->sqlFromPart =
            'Navsteva LEFT JOIN Osoba NavstivenaOsoba ON (Navsteva.NavstivenaOsobaRC = NavstivenaOsoba.RC) ' .
            ' LEFT JOIN Osoba Zapsal ON (Navsteva.prichodZapsalRC = Zapsal.RC)' .
            ' LEFT JOIN PrirazeniKarty' .
            '  ON (Navsteva.OsobaRC = PrirazeniKarty.RC) AND (Navsteva.OsobniCislo = PrirazeniKarty.OsobniCislo) AND (Navsteva.DatumNastupu = PrirazeniKarty.DatumNastupu)' .
            '  AND PrirazeniKarty.DatumPrideleni <= Navsteva.Prichod' .
            '  AND (PrirazeniKarty.DatumVraceni >= Navsteva.Prichod  OR  PrirazeniKarty.DatumVraceni IS NULL)' .
            ' LEFT JOIN Karta ON (Karta.Kod = PrirazeniKarty.kodKarty)' .
            ' LEFT JOIN DuvodNavstevy ON ( (DuvodNavstevy.Kod = Navsteva.duvodNavstevy2))';

        // Typ navstevy
        $this->addField( new TypNavstevy($database, null))
            ->setCode('col-typnavstevy');

        // Vratnice
        $this->addField()
            ->setSqlFieldName(
                '(SELECT Rozdeleni.Popis FROM Rozdeleni' .
                ' WHERE Rozdeleni.KodTypuRozdeleni = Navsteva.KodTypuRozdeleni_Vratnice AND Rozdeleni.Kod = Rozdeleni_Vratnice) AS Rozdeleni_vratnice_popis')
            ->setSortSQL('Rozdeleni_vratnice_popis') ->setOrdering(true)
            ->setHeaderText('vratnice')
            ->setCode('col-vratnice')
            ->setVisible(!$omezeniVratnice);

        // Duvod navstevy
        $this->addField()
            ->setSqlFieldName('DuvodNavstevy.Popis') ->setHeaderText('duvod_navstevy')
            ->setCode('col-duvodnavstevy2');

        //Ocekavany?
        $this->addField(BrowserField::FIELD_TYPE_SELECT)
            ->setSqlFieldName('Navsteva.PredemOcekavanaNavsteva') ->setHeaderText('ocekavany') ->setOrdering(true)
            ->setCode('col-ocekavany')
            ->setOptions([0 => TextManager::_('Str_No'), 1 => TextManager::_('Str_Yes')])
            ->setAlign('center');

        //Navsteva.prijmeni
        $this->addField()
            ->setSqlFieldName('Navsteva.Prijmeni') ->setHeaderText('Prijmeni')
            ->setCode('col-prijmeni')
            ->setOrdering(true) ->setClassName('font-weight-bold')
            ->setLink('?menuid=navstevy&action=zaznam');

        //Navsteva.jmeno
        $this->addField()
            ->setSqlFieldName('Navsteva.Jmeno') ->setHeaderText('Jmeno')
            ->setCode('col-jmeno')
            ->setOrdering(true)
            ->setLink('?menuid=navstevy&action=zaznam');

        //Ocekavany prichod
        $this->addField( new CasNavstevyBrowserField($database))
            ->setSqlFieldName('OcekavanyPrichod') ->setHeaderText('ocekavany_datum') ->setFormat('weektimestamp')
            ->setCode('col-ocekavanyprichod')
            ->setOrdering(true)
            ->setLink('?menuid=navstevy&formaction=edit');

        //doklad
        $this->addField()
            ->setSqlFieldName('Navsteva.Doklad') ->setHeaderText('doklad')
            ->setCode('col-doklad');

        //vozidlo
        $this->addField()
            ->setSqlFieldName('Vozidlo') ->setHeaderText('vozidlo')
            ->setCode('col-vozidlo');

        //firma
        $this->addField()
            ->setSqlFieldName('firma') ->setHeaderText('firma') ->setOrdering(true)
            ->setCode('col-firma');

        //OznaceniMedia
        $this->addField()
            ->setSqlFieldName('OznaceniMedia') ->setHeaderText('oznaceni_media') ->setOrdering(true)
            ->setCode('col-oznacenimedia');

        //Prichod
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName("COALESCE(Zapsal.Jmeno,'') {fn_concat()} ' ' {fn_concat()} COALESCE(Zapsal.Prijmeni,'')")
            ->setCode('col-prichodzapsal');

        $this->addField( new CasNavstevyBrowserField($database))
            ->setSqlFieldName('Prichod') ->setHeaderText('prichod') ->setFormat('weektimestamp') ->setOrdering(true)
            ->setCode('col-prichod')
            ->setTitleFieldIndex('col-prichodzapsal', TextManager::_('prichod_zapsal') . ': ') ->setLink('?menuid=navstevy&formaction=edit');

        $this->defaultSort = '-col-prichod'; // -count($this ->fields);

        //Odchod
        $this->addField(new PotvrzeniOdchoduNavstevyField($database, null))
            ->setSqlFieldName('Odchod') ->setHeaderText('Odchod') ->setFormat('weektimestamp') ->setOrdering(true)
            ->setCode('col-odchod');

        $this ->addField(BrowserField::FIELD_TYPE_COMMENT)
            ->setSqlFieldName('{fn_castwidememo(duvodNavstevy)}') ->setHeader(TextManager::_('duvod_navstevy') . ': ')
            ->setCode('col-duvodnavstevy')
            ->setClassName('browserpoznamka') ->setHeaderColSpan(-1);

        //NavstivenaOsoba
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('NavstivenaOsoba.Telefon')
            ->setCode('col-telefon');

        $this->addField()
            ->setSqlFieldName('JmenoNavstiveneOsoby') ->setHeaderText('navstivena_osoba')
            ->setCode('col-navstivenaosoba')
            ->setTitleFieldIndex('col-telefon', TextManager::_('telefon') . ': ');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Navsteva.DT_Schvaleni') ->setFormat('datetime')
            ->setCode('col-dt_schvaleni');

        $this->addField( new NavstevuSchvalilaOsoba($database))
            ->setSqlFieldName('Navsteva.Schvalil') ->setSqlFieldIndex2('@col-dt_schvaleni') // vola getViewText hidden pole s kodem col-dt_schvaleni
            ->setCode('col-schvalil')
            ->setHeaderText('navstevu_schvalil');

        $zobrazitPoznamky = (int)($_GET['poznamky'] ?? ($_SESSION['uzivatelskePoznamkyNavstev'] ?? 0));

        for ($x = 1; $x <= 10; $x++) {
            $this->addField(BrowserField::FIELD_TYPE_COMMENT)
                ->setSqlFieldName("Navsteva.Uzivatelska$x")
                ->setVisible($zobrazitPoznamky  &&  RonPortalSettings::getValue("navstevy_uzivatelska$x"))
                ->setHeader(RonPortalSettings::getValue('navstevy_uzivatelska' . $x) . ': ')
                ->setCode("col-uzivatelska$x")
                ->setClassName('browserpoznamka')    // puvodne mel navic cely radek class uzivatelska-pole-navstev
                ->setHeaderColSpan(-1);
        }
    }


    function setFilterForm()
    {
        $this->filterForm = new NavstevyFilter($this ->database);
        //$this->filterForm->loadData();
        // nedela nic - filter nastaven v constructoru
    }

    function renderFilter()
    {
        parent::renderFilter();

        echo
            '<ul class="linkslist margin-top-3">' .
            "<li><a href='?menuid=navstevy&formaction=new&ocekavana=1'>" . TextManager::_('ocekavana_navsteva') . "</a></li>" .
            (!$this ->prava ->priznak // zapis nove nasvtevy se nezobrazuje, pokud ma pravo jen na ocekavane navstevy
                ? "<li><a href='?menuid=navstevy&formaction=new&ocekavana=0'>" . TextManager::_('zapis_nove_navstevy') . "</a></li>"
                : ''
            ) .
            "<li><a id='najit_navstevu_pomoci_id_medium'>" . TextManager::_('najit_navstevu_pomoci_id_medium') . "</a></li>" .
            "</ul>";
    }


    /**
     * @param array $values
     * @return bool
     * @throws Exception
     */
    function enableLine($values)
    {
        //Zobrazi pokud ma na RC navstivene osoby pravo
        return $this->prava->lzeZobrazitRC($values[2]);
    }

    /**
     * @param Database $database
     * @return mixed
     * @throws DBException
     */
    private function rozdeleniVratnice($database)
    {
        $kod = RonPortalSession::getValue('rozdelenivratnice');
        if (!$kod) {
            return '';
        }

        $params = $database->createParameters();
        $sql =
            'SELECT Rozdeleni.Popis' .
            ' FROM Rozdeleni' .
            ' WHERE KodTypuRozdeleni = ' . $params ->addParameter(RonPortalSettings::getIntValue('typrozdelenivratnice')) .
            ' AND Rozdeleni.Kod = ' . $params->addParameter($kod);

        $cursor = $database->getSqlResult($sql, $params);
        $row = $cursor->fetch();
        $cursor ->close();
        return $row[0] ?? '';
    }


    /**
     * @return SqlStatement|string
     */
    function getSQLFilter()
    {
        $params = $this->database->createParameters();

        $result = '';

        if ($this ->filterForm ->omezeniVratnice  &&  RonPortalSession::getValue('rozdelenivratnice') != '') {
            $result =
                'Navsteva.Rozdeleni_Vratnice = ' . $params->addParameter(RonPortalSession::getValue('rozdelenivratnice'));
        }

//        if ($this ->filterForm ->jenMedium) {
//            $result .=
//                ($result ? ' AND ' : '') .
//                ' Karta.Kod = ' . $params->addParameter($this ->filterForm ->kodMedia) .
//                ' AND Prichod IS NOT NULL AND Odchod IS NULL';
        if ($medium = $this ->filterForm ->getFilterValue('medium')) {
            $result .=
                ($result ? ' AND ' : '') .
                ' Karta.Kod = ' . $params->addParameter($medium) .
                ' AND Prichod IS NOT NULL AND Odchod IS NULL';
        } else {
            $formFilter = $this ->filterForm ->buildSqlFilter($params);
            if ($formFilter) {
                $result .=
                    ($result ? ' AND ' : '') .
                    $formFilter;
            }

            //timestamp -> string
            if ($this ->filterForm ->omezeniNavstevFilter ->value == 1) {
                // ocekavane prichody
                $result .=
                    ($result ? ' AND ' : '') .
                    '(Navsteva.OcekavanyPrichod >= ' . $params->addDateParameter(time()) .
                    ' AND Navsteva.Prichod IS NULL AND Navsteva.Odchod IS NULL)';
            } else if ($this ->filterForm ->omezeniNavstevFilter ->value == 2) {
                // pritomni
                $result .=
                    ($result ? ' AND ' : '') .
                    ' (Navsteva.Prichod <= ' . $params->addTimestampParameter(time()) . // uz prisla a jeste neodesla
                    ' AND Navsteva.Odchod IS NULL';
                $result .= ")";

            } else {
                $result .=
                    ($result ? ' AND ' : '') .
                    '((Navsteva.Prichod IS NULL AND Navsteva.OcekavanyPrichod >= ' . $params->addDateParameter($this ->filterForm ->datumOd) .
                    ($this ->filterForm ->datumDo ? ' AND Navsteva.OcekavanyPrichod < ' . $params->addDateParameter(DateUtils::incDayStart($this ->filterForm ->datumDo)) : '') .
                    ') OR (' .
                    'Navsteva.Prichod >= ' . $params->addDateParameter($this ->filterForm ->datumOd) .
                    ($this ->filterForm ->datumDo ? ' AND Navsteva.Prichod < ' . $params->addDateParameter(DateUtils::incDayStart($this ->filterForm ->datumDo)) : '') .
                    '))';
            }

//            if ($this ->filterForm ->jenTypNavstevy) {
//                $result .=
//                    ($result ? ' AND ' : '') .
//                    " Navsteva.TypNavstevy = " . $params->addParameter($this ->filterForm ->typNavstevy);
//            }

//            if ($this ->filterForm ->jenNavstivenaOsoba) {
//                $result .=
//                    ($result ? ' AND ' : '') .
//                    " Navsteva.JmenoNavstiveneOsoby LIKE " . $params->addParameter("%{$this ->filterForm ->navstivenaOsoba}%");
//            }

//            if ($this ->filterForm ->jenFirma) {
//                $result .=
//                    ($result ? ' AND  ' : '') .
//                    'Navsteva.Firma LIKE ' . $params->addParameter("%{$this ->filterForm ->firma}%");
//            }

//            if ($this ->filterForm ->jenPrijmeniNavstevy && $this ->filterForm ->prijmeniNavstevy) {
//                $result .=
//                    ($result ? ' AND ' : '') .
//                    ' Navsteva.Prijmeni LIKE ' . $params->addParameter("%{$this ->filterForm ->prijmeniNavstevy}%");
//            }

//            if (RonPortalSession::getValue('jenKod')) {
//                $result .=
//                    ($result ? ' AND ' : '') .
//                    " Navsteva.kod = " . $params->addParameter($_SESSION['kod']);
//            }

//            if ($this ->filterForm ->jenDuvod  &&  $this ->filterForm ->duvod) {
//                $result .=
//                    ($result ? ' AND ' : '') .
//                    ' Navsteva.duvodNavstevy2 = ' . $params->addParameter($this ->filterForm ->duvod);
//            }


//            if ($this ->filterForm ->jenVozidlo) {
//                $result .=
//                    ($result ? ' AND ' : '') .
//                    ($this ->filterForm ->sVozidlem
//                        ? 'Navsteva.vozidlo IS NOT NULL'
//                        : 'Navsteva.vozidlo IS NULL'
//                    );
//            }

//            if ($this ->filterForm ->filtrovatSchvaleni) {
//                $result .=
//                    ($result ? ' AND ' : '') .
//                    ($this ->filterForm ->schvalene
//                        ? 'Navsteva.Schvalil IS NOT NULL'
//                        : 'Navsteva.Schvalil IS NULL'
//                    );
//            }
        }

        return new SqlStatement($result, $params);
    }
}
