<?php
/**
 * stanislav.elbl, 01.11.2017
 */

namespace RonPortal\Common;

use Ron\Controls\BrowserField;
use Ron\DateUtils;
use Ron\DB\ArrayCursor;
use Ron\DB\SqlStatement;
use Ron\Http\Csrf;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\JmenoPrijmeniBrowserField;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Prava;
use RonPortal\User;


class LoginAsLinkField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($rowData[3] < 2) { // nezobrazuje link pro zrusene a zamitnute
            if ($this->database->stringToTimestamp($rowData[1]) <= time() && (!$rowData[2] || $this->database->stringToTimestamp($rowData[2]) >= DateUtils::startOfTheDay(time()))) {
                return
                    '<a class="confirmQuestion" href="?menuid=zastupuzivatele&action=loginas&id=' . $rowData[0] .
                    Csrf::csrfUrlParameter() .
                    '" data-ron-confirm-id="Opravdu_prihlasit_jako_zastup">' .
                    TextManager::_('Prihlasit_se_jako_zastup') .
                    '</a>';
            }
        }

        return '';
    }
}


class ZastupUzivateleBrowser extends RonPortalBrowser
{
    const JA_ZASTUPUJI = 1;
    const JSEM_ZASTUPOVAN = 2;
    const VSECHNY_ZASTUPY = 3;

    public $coZobrazit;

    /**
     * @param \Ron\DB\Database $database
     * @param int $coZobrazit  konstanty JA_ZASTUPUJI/JSEM_ZASTUPOVAN/VSECHNY_ZASTUPY
     */
    function __construct($database, $coZobrazit)
    {
        parent::__construct($database, 'zastupuzivatele');
        $this ->coZobrazit = $coZobrazit;

        $this->rowClickLink = '?menuid=zastupuzivatele';

        if ($coZobrazit == self::JA_ZASTUPUJI) {
            $this->title = TextManager::_('Vy_zastupujete_uzivatele');
        } else if ($coZobrazit == self::JSEM_ZASTUPOVAN) {
            $this->title = TextManager::_('Zastupuje_vas_uzivatel');
        } else {
            $this->title = '';
        }

        $this->sqlFromPart =
            'ZastupUzivatele' .
            //' INNER JOIN RoleProZastupUzivatele ON RoleProZastupUzivatele.Kod = ZastupUzivatele.Role' .
            ' INNER JOIN Osoba Zastupovany ON Zastupovany.RC = ZastupUzivatele.Zastupovany' .
            ' INNER JOIN Osoba Zastupujici ON Zastupujici.RC = ZastupUzivatele.Zastupujici';

        $this ->sqlSelectPart =
            'ZastupUzivatele.Id, ' . //  ZastupUzivatele.Role, RoleProZastupUzivatele.Popis, ' . // 0
            'ZastupUzivatele.PlatnostOd, ZastupUzivatele.PlatnostDo, ZastupUzivatele.Potvrzeni, ZastupUzivatele.CasPotvrzeni, ' . // 1..4
            'Zastupovany.RC, Zastupovany.Jmeno, Zastupovany.Prijmeni, ' . // 5..7
            'Zastupujici.RC, Zastupujici.Jmeno, Zastupujici.Prijmeni, ' . // 8..10
            $database ->castWideMemo('ZastupUzivatele.Poznamka') . ', ' . $database ->castWideMemo('ZastupUzivatele.VyjadreniZastupujiciho'); //11,12

        $this ->primaryField = 0;

        $this->sqlOrderPart = 'PlatnostOd DESC';

        $params = $database ->createParameters();
        $where = '(ZastupUzivatele.PlatnostDo >= ' . $params ->addParameter($database ->dateToParam(time())) . ' OR ZastupUzivatele.PlatnostDo IS NULL)';

        if ($coZobrazit == self::JA_ZASTUPUJI) {
            $where .= ' AND ZastupUzivatele.Zastupujici = ' . $params ->addParameter(User::getRC());
        } else if ($coZobrazit == self::JSEM_ZASTUPOVAN) {
            $where .= ' AND ZastupUzivatele.Zastupovany = ' . $params ->addParameter(User::getRC());
        }

        $this ->sqlWherePart = new SqlStatement($where, $params);

        $this ->addField()
            ->setSqlFieldName('ZastupUzivatele.PlatnostOd', 1) ->setFormat('date') ->setHeaderText('Platnost_od');

        $this ->addField()
            ->setSqlFieldName('ZastupUzivatele.PlatnostDo',  2) ->setFormat('date') ->setHeaderText('Platnost_do');
        //$this ->addField( [BROWSER_FIELD_SQL_NAME => 'RoleProZastupUzivatele.Popis',  BROWSER_FIELD_SQL_INDEX => 2, BROWSER_FIELD_HEADER_TEXT => 'Role'] );

        if ($coZobrazit == self::JA_ZASTUPUJI  ||  $coZobrazit == self::VSECHNY_ZASTUPY) {
            $this->addField( new JmenoPrijmeniBrowserField($this ->database, 7, 6))
                ->setHeaderText('Zastupovany_uzivatel');
        }

        if ($coZobrazit == self::JSEM_ZASTUPOVAN  ||  $coZobrazit == self::VSECHNY_ZASTUPY) {
            $this->addField( new JmenoPrijmeniBrowserField($this ->database, 10, 9) )
                ->setHeaderText('Zastupujici_uzivatel');
        }

        $this ->addField(BrowserField::FIELD_TYPE_SELECT)
            ->setSqlFieldName('ZastupUzivatele.Potvrzeno',  3)
            ->setOptions([
                0 => TextManager::_('Zastup_potvrzeni_0'),
                1 => TextManager::_('Zastup_potvrzeni_1'),
                2 => TextManager::_('Zastup_potvrzeni_2'),
                3 => TextManager::_('Zastup_potvrzeni_3')
            ]);

        $this->addField(BrowserField::FIELD_TYPE_TEXT)
            ->setText(TextManager::_('Podrobnosti'))
            ->setLink('?menuid=zastupuzivatele');

        if ($coZobrazit == self::JA_ZASTUPUJI) {
            $this->addField( new LoginAsLinkField($this ->database));
        } else {
            $this->addField(BrowserField::FIELD_TYPE_TEXT)
                ->setText(TextManager::_('Upravit'))
                ->setLink('?menuid=zastupuzivatele&formaction=edit');
        }
    }

    function setFilterForm()
    {
        // parent::setFilterForm(); -- nedela nic, filtr se nezobrazuje
    }


    function load($sqlFilter = '')
    {
        parent::load($sqlFilter);
        $this ->cursor = new ArrayCursor($this ->cursor);
    }


    function enableLine($values)
    {
        return
            $this ->coZobrazit == self::JA_ZASTUPUJI  ||
            $this ->coZobrazit == self::JSEM_ZASTUPOVAN ||
            Prava::vratPrava('zast_uz_komu') ->lzeZobrazitRC($values[5], time(), time());
    }


    function renderLine($values, $className, $rowNumber, $edit = false)
    {
        parent::renderLine($values, $className, $rowNumber, $edit);

        if ($values[11]) {
            // zobrazi poznamku
            $colSpan = $this->columnCount - 1;

            echo "<tr class='$className'><td></td><td colspan='$colSpan'><div class='poznamka browserpoznamka'>" . StringUtils::htmlTextConvert($values[11]) . '</div></td>';
        }
    }


    function render()
    {
        if ($this ->cursor ->numRows()) {
            parent::render();
        } else {
            if ($this->coZobrazit == self::JSEM_ZASTUPOVAN) {
                echo '<div class="infomsg">' . TextManager::_('Nemate_naplanovan_zadny_zastup') . '</div>';
            } else if ($this->coZobrazit == self::JA_ZASTUPUJI) {
                echo '<div class="infomsg">' . TextManager::_('Nikoho_nezastupujete') . '</div>';
            }
        }

        if ($this->coZobrazit == self::JSEM_ZASTUPOVAN) {
            echo '<div><a href="?menuid=zastupuzivatele&action=novazadost">' . TextManager::_('Pozadat_o_zastup') . '</a></div>';
        }

        if ($this->coZobrazit == self::VSECHNY_ZASTUPY) {
            echo '<div><a href="?menuid=zastupuzivatele&action=novyzastup">' . TextManager::_('Naplanovat_zastup_uzivatele') . '</a></div>';
        }
    }


}
