<?php

namespace RonPortal\Controls;

use Ron\ArrayUtils;
use Ron\Controls\ComboBox;
use Ron\Controls\ConfigurableObject;
use Ron\Controls\LinksList;
use Ron\Controls\LinksListItem;
use Ron\Controls\ObjectConfiguration;
use Ron\Controls\Option;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\Parameters;
use Ron\DB\SqlStatement;
use Ron\GuiUtils;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\NastaveniUzivatele;
use RonPortal\User;
use Tracy\Debugger;


class FilterFormFilterItem extends LinksListItem implements ConfigurableObject
{
    /** @var string|SqlStatement|null   SQL filtr, ktery se pouzije pro filtrovani v seznamu */
    public $sqlFilter = null;

    /** @var bool       Urcuje, jestli se tato hodnota filtru zobrazi a jestli je mozne ji pouzivat */
    public $enabled = true;


    public function __construct($key, $caption)
    {
        parent::__construct($key, $caption);
        $this ->active = false;
    }


    /**
     * @param SqlStatement|string|null $sqlFilter
     * @return FilterFormFilterItem
     */
    public function setSqlFilter($sqlFilter)
    {
        $this->sqlFilter = $sqlFilter;
        return $this;
    }

    /**
     * @param bool $enabled
     * @return FilterFormFilterItem
     */
    public function setEnabled(bool $enabled): FilterFormFilterItem
    {
        $this->enabled = $enabled;
        return $this;
    }

    public function loadConfigurationItem($key, $value, $valueText)
    {
        switch ($key) {
            case 'key' :
            case 'sqlFilter' :
            case 'cssClass' :
            case 'group' :
                $this ->{$key} = $valueText;
                break;

            case 'caption' :
            case 'title' :
                $this ->{$key} = TextManager::_($valueText);
                break;

            case 'enabled' :
                $this ->{$key} = (int)$valueText;
                break;
        }
    }
}


/**
 * Reprezentuje jeden konkretni filtr (obvykle jeden radek) v obecnem filtrovacim formulari @link RonPortalFilterForm.
 */
class FilterFormFilter implements ConfigurableObject
{
    /** @var string     Filtr se zobrazi jako odkazy */
    const TYPE_LINKS = 'links';
    /** @var string     Filtr se zobrazi jako combobox */
    const TYPE_OPTIONS = 'options';
    /** @var string     Filtr se zobrazi jako editovaci pole */
    const TYPE_EDIT = 'edit';
    /** @var string     Zobrazi se jen text. Muze obsahovat HTML znacky. */
    const TYPE_TEXT = 'text';
    /** @var string     Filtr se zobrazi jako 2 comboboxy (napr. kod - popis) */
    const TYPE_COMBOBOXES = 'comboboxes';
    /** @var string     Filtr se zobrazi jako checkbox */
    const TYPE_CHECKBOX = 'checkbox';
    /** @var string     Filtr se zobrazi jako checkbox */
    const TYPE_HIDDEN = 'hidden';



    /** @var string     Typ filtru - tj. zpusob, jakym se filtr zobrazi (napr. seznam odkazu, select) */
    public $type = self::TYPE_LINKS;

    /** @var string     Jmeno filtru. Pouzije se predevsim pro jmeno GET parametru v URL. */
    public $name;

    /** @var string|bool    Jmeno pro ulozeni hodnoty filtru do SESSION. Pokud je true, tak ze pouzije "filterform_<$name>". Pokud se vyhodnoti jako false, tak se do session neuklada. */
    public $sessionName = true;

    /** @var string|bool    Jmeno pro ulozeni hodnoty filtru do uzivatelskeho nastaveni. Pokud je true, tak se pouzije "filterform_<$name>". Pokud se vyhodnoti jako false, tak se do uzivatelskeho nastaveni neuklada. */
    public $userSettingsName = false;

    //public $class;  // TODO ! ! !
    /** @var string         Css trida, ktera se prida ke standardni css tride prvku */
    public $addClass;

    /** @var string|bool     Nadpis filtru. Pokud je false, tak se nadpis vubec nezobrazi */
    public $caption;
    /** @var string|bool     Druhy nadpis filtru napriklad pro checkbox (zobrazi se za checkboxem). */
    public $caption2;

    /** @var string[]|null  HTML atributy inputu */
    public $htmlAttributes = null;
    /** @var string[]|null  HTML atributy druheho selectu pro "comboboxes" */
    public $htmlAttributes2 = null;
    /** @var string[]|null  HTML atributy nadpisu */
    public $captionHtmlAttributes;

    /** @var FilterFormFilterItem[]     Jednotlive polozky pro zobrazeni odkazu nebo vyber pomoci <select> */
    public $items = [];
    /** @var FilterFormFilterItem[]     Jednotlive polozky pro zobrazeni comboboxu 2 */
    public $items2 = [];

    /** @var bool   U typu TYPE_LINKS umoznuje zabraznit zobrazeni aktivni polozky */
    public $showSelectedItem = true;

    /** @var string|SqlStatement    SQL vyraz pro nacteni polozek ($items) z databaze. */
    public $optionsSql;
    /** @var string|null            SQL filtr pro options. Pokud je zadan, tak nahradi ve vyrazu optionsSql retezec %FILTER% */
    public $optionsSqlFilter = null;
    /** @var string|null            SQL razeni pro options. Pokud je zadano, tak nahradi ve vyrazu optionsSql retezec %ORDER% */
    public $optionsSqlOrder = null;
    /** @var string|null            Prefix pro nalezeni prekladu dat z databaze */
    public $optionsTranslationPrefix = null;
    /** @var string|null            SQL razeni pro options v comboboxu 2 (typ "comboboxes". Pokud je zadano, tak nahradi ve vyrazu optionsSql retezec %ORDER% */
    public $optionsSqlOrder2 = null;

    /** @var bool                   Urcuje, jestli se ma zobrazit kod v textu jednotlivych voleb (options) */
    public $showCode = false;


    /** @var mixed      Vychozi hodnota */
    public $defaultValue;

    /** @var bool       Urcuje, jestli je filtr povolen. Pokud neni, tak se vubec nezobrazi a nefiltruje se podle nej. */
    public $enabled = true;

    /** @var string|bool    HTML class pro vygenerovani radku filtru. Pokud je false, tak se radek vubec negeneruje */
    public $rowClass = 'filterrow';
    /** @var string|bool    HTML id radku filtru. Pokud je true, tak se pouzije "filterrow_<$name>" */
    public $rowId = true;

    /** @var mixed      Skutecna aktualni hodnota zvolena uzivatelem */
    public $value = null;

    /** @var string|null   SQL filtr - prida se k nemu parametr s vybranou hodnotou - takze by mel mit tvar napriklad 'tabulka.pole = ' */
    public $sqlFilter;

    /**
     * @param string $name
     * @param string $type
     */
    public function __construct(string $name, string $type = self::TYPE_LINKS)
    {
        $this->name = $name;
        $this->type = $type;
        $this->sessionName = 'filterform_' . $name;

        if ($this ->type === self::TYPE_HIDDEN) {
            $this ->rowClass = false;
            $this ->caption = false;
        }
    }


    /**
     * @param bool|string $caption
     * @return FilterFormFilter
     */
    public function setCaption($caption): FilterFormFilter
    {
        $this->caption = $caption;
        return $this;
    }


    /**
     * @param bool|string $caption2
     * @return FilterFormFilter
     */
    public function setCaption2($caption2): FilterFormFilter
    {
        $this->caption2 = $caption2;
        return $this;
    }


    /**
     * @param string[]|null $htmlAttributes
     * @return FilterFormFilter
     */
    public function setHtmlAttributes(?array $htmlAttributes): FilterFormFilter
    {
        $this->htmlAttributes = $htmlAttributes;
        return $this;
    }

    /**
     * @param string[]|null $htmlAttributes2
     * @return FilterFormFilter
     */
    public function setHtmlAttributes2(?array $htmlAttributes2): FilterFormFilter
    {
        $this->htmlAttributes2 = $htmlAttributes2;
        return $this;
    }



    /**
     * @param string[]|null $captionHtmlAttributes
     * @return FilterFormFilter
     */
    public function setCaptionHtmlAttributes($captionHtmlAttributes)
    {
        $this->captionHtmlAttributes = $captionHtmlAttributes;
        return $this;
    }



    /**
     * @param FilterFormFilterItem[] $items
     * @return FilterFormFilter
     */
    public function setItems(array $items): FilterFormFilter
    {
        $this->items = $items;
        return $this;
    }


    /**
     * @param FilterFormFilterItem|string $item
     * @param string|null $caption
     * @return FilterFormFilter
     */
    public function addItem($item, ?string $caption = null): FilterFormFilter
    {
        if ($item instanceof FilterFormFilterItem) {
            $this ->items[$item ->key] = $item;
        } else {
            $this ->items[$item] = new FilterFormFilterItem($item, $caption);
        }
        return $this;
    }

    /**
     * @param FilterFormFilterItem|string $item
     * @param string|null $caption
     * @return FilterFormFilter
     */
    public function addItem2($item, ?string $caption = null): FilterFormFilter
    {
        if ($item instanceof FilterFormFilterItem) {
            $this ->items2[$item ->key] = $item;
        } else {
            $this ->items2[$item] = new FilterFormFilterItem($item, $caption);
        }
        return $this;
    }


    /**
     * @param bool $showSelectedItem
     * @return FilterFormFilter
     */
    public function setShowSelectedItem(bool $showSelectedItem): FilterFormFilter
    {
        $this->showSelectedItem = $showSelectedItem;
        return $this;
    }


    /**
     * @param SqlStatement|string $optionsSql
     * @param string|null $optionsSqlFilter
     * @param string|null $optionsSqlOrder
     * @return FilterFormFilter
     */
    public function setOptionsSql($optionsSql, $optionsSqlFilter = null, $optionsSqlOrder = null): FilterFormFilter
    {
        $this->optionsSql = $optionsSql;
        $this->optionsSqlFilter = $optionsSqlFilter;
        $this->optionsSqlOrder = $optionsSqlOrder;

        return $this;
    }

    /**
     * @param string|null $optionsTranslationPrefix
     * @return FilterFormFilter
     */
    public function setOptionsTranslationPrefix(?string $optionsTranslationPrefix): FilterFormFilter
    {
        $this->optionsTranslationPrefix = $optionsTranslationPrefix;
        return $this;
    }


    /**
     * @param bool $showCode
     * @return FilterFormFilter
     */
    public function setShowCode(bool $showCode): FilterFormFilter
    {
        $this->showCode = $showCode;
        return $this;
    }


    /**
     * @param string|null $optionsSqlFilter
     * @return FilterFormFilter
     */
    public function setOptionsSqlFilter(?string $optionsSqlFilter): FilterFormFilter
    {
        $this->optionsSqlFilter = $optionsSqlFilter;
        return $this;
    }

    /**
     * @param string|null $optionsSqlOrder
     * @return FilterFormFilter
     */
    public function setOptionsSqlOrder(?string $optionsSqlOrder): FilterFormFilter
    {
        $this->optionsSqlOrder = $optionsSqlOrder;
        return $this;
    }

    /**
     * @param string|null $optionsSqlOrder2
     * @return FilterFormFilter
     */
    public function setOptionsSqlOrder2(?string $optionsSqlOrder2): FilterFormFilter
    {
        $this->optionsSqlOrder2 = $optionsSqlOrder2;
        return $this;
    }


    /**
     * @param mixed $defaultValue
     * @return FilterFormFilter
     */
    public function setDefaultValue($defaultValue): FilterFormFilter
    {
        $this->defaultValue = $defaultValue;
        return $this;
    }

    /**
     * @param bool $enabled
     * @return FilterFormFilter
     */
    public function setEnabled(bool $enabled): FilterFormFilter
    {
        $this->enabled = $enabled;
        return $this;
    }


    /**
     * @param bool|string $sessionName
     * @return FilterFormFilter
     */
    public function setSessionName($sessionName): FilterFormFilter
    {
        $this->sessionName = $sessionName;
        return $this;
    }

    /**
     * @param bool|string $userSettingsName
     * @return FilterFormFilter
     */
    public function setUserSettingsName($userSettingsName): FilterFormFilter
    {
        $this->userSettingsName = $userSettingsName;
        return $this;
    }


    /**
     * @param string $type
     * @return FilterFormFilter
     */
    public function setType(string $type): FilterFormFilter
    {
        $this->type = $type;
        return $this;
    }


    /**
     * @param string $addClass
     * @return FilterFormFilter
     */
    public function setAddClass($addClass): FilterFormFilter
    {
        $this->addClass = $addClass;
        return $this;
    }


    /**
     * @param bool|string $rowClass
     * @return FilterFormFilter
     */
    public function setRowClass($rowClass): FilterFormFilter
    {
        $this->rowClass = $rowClass;
        return $this;
    }

    /**
     * @param bool|string $rowId
     * @return FilterFormFilter
     */
    public function setRowId($rowId): FilterFormFilter
    {
        $this->rowId = $rowId;
        return $this;
    }

    /**
     * @param mixed|null $value
     * @return FilterFormFilter
     */
    public function setValue($value): FilterFormFilter
    {
        $this->value = $value;
        return $this;
    }


    /**
     * @param SqlStatement|string|null $sqlFilter
     */
    public function setSqlFilter($sqlFilter): FilterFormFilter
    {
        $this->sqlFilter = $sqlFilter;
        return $this;
    }


    protected function loadOptionsConfiguration($optionsConfiguration)
    {
        foreach ($optionsConfiguration as $optionConfiguration) {
            $key = $optionConfiguration['key'];
            $item = $this->items[$key] ?? null;
            if (!$item) {
                $item = new FilterFormFilterItem($key, $optionConfiguration['#text']);
                $this ->items[$key] = $item;
            } else if (is_string($item)) {
                $item = new FilterFormFilterItem($key, $optionConfiguration['#text'] ?? $item);
                $this ->items[$key] = $item;
            }

            $objectConfiguration = new ObjectConfiguration($item);
            $objectConfiguration->loadConfigurationArray($optionConfiguration);
        }
    }

    public function loadConfigurationItem($key, $value, $valueText)
    {
        switch ($key) {
            case 'type' :
            case 'sqlFilter' :
            case 'name' :
            case 'sessionName' :
            case 'userSettingsName' :
            case 'optionsSql' :
            case 'optionsSqlFilter' :
            case 'optionsSqlOrder' :
            case 'optionsTranslationPrefix' :
            case 'optionsSqlOrder2' :
            case 'defaultValue' :
            case 'rowClass' :
            case 'addClass' :
            case 'rowId' :
                $this ->{$key} = $valueText;
                break;

            case 'addRowClass' :
                $this ->rowClass .= $valueText;
                break;

            case 'caption' :
            case 'caption2' :
                $this ->{$key} = TextManager::_($valueText);
                break;

            case 'enabled' :
            case 'showCode' :
            case 'showSelectedItem' :
                $this ->{$key} = (int)$valueText;
                break;

            case 'option' :
                $this ->loadOptionsConfiguration($value);
                break;
        }
    }


    /**
     * Nacte volby z SQL dotazu
     * @param Database $database
     * @return $this
     * @throws DBException
     */
    public function loadOptions(Database $database): FilterFormFilter
    {
        $params = null;

        if ($this ->optionsSql) {
            if ($this ->optionsSql instanceof SqlStatement) {
                $optionsSql = $this ->optionsSql ->sqlText;
                $params = $this ->optionsSql ->parameters;
            } else {
                $optionsSql = $this ->optionsSql;
            }

            $sql = str_replace(
                ['%FILTER%', '%ORDER%'],
                [$this->optionsSqlFilter ?? '1 = 1', $this->optionsSqlOrder ?? ''],
                $optionsSql
            );

            $cursor = $database ->getSqlResult($sql, $params);
            while ($row = $cursor ->fetch()) {
                $text = $this ->optionsTranslationPrefix
                    ? TextManager::_($this ->optionsTranslationPrefix . $row[0], $row[1])
                    : $row[1];
                $item = new FilterFormFilterItem($row[0], $text);
                $this ->items[$row[0]] = $item;
            }
            $cursor ->close();

            if ($this ->type === self::TYPE_COMBOBOXES) {
                $sql = str_replace(
                    ['%FILTER%', '%ORDER%'],
                    [$this->optionsSqlFilter ?? '1 = 0', $this->optionsSqlOrder2 ?? ''],
                    $optionsSql
                );

                $cursor = $database ->getSqlResult($sql, $params);
                while ($row = $cursor ->fetch()) {
                    $text = $this ->optionsTranslationPrefix
                        ? TextManager::_($this ->optionsTranslationPrefix . $row[0], $row[1])
                        : $row[1];

                    $item = new FilterFormFilterItem($row[0], $row[0]);
                    $item ->setTitle($text);
                    $this ->items2[$row[0]] = $item;
                }
                $cursor ->close();
            }
        }

        return $this;
    }

    /**
     * Nacte hodnotu zadanou uzivatelem (podle GET) a pripadne seznam polozek z databaze
     * @param Database $database
     * @return void
     */
    public function load(Database $database)
    {
        if (!$this ->enabled) {
            return;
        }

        $sessionName = $this ->sessionName === true
            ? "filterform_{$this ->name}"
            : $this ->sessionName;

        $userSettingsName = $this ->userSettingsName === true
            ? "filterform_{$this ->name}"
            : $this ->userSettingsName;

        $this ->loadOptions($database);

        if (isset($_GET[$this ->name])) {
            if ($this ->type === self::TYPE_CHECKBOX) {
                $this ->value = (int)$_GET[$this ->name];
            } else {
                $this->value = $_GET[$this->name];
                if ($this->value === '') {
                    $this->value = null;
                }
            }

            if ($sessionName) {
                $_SESSION[$sessionName] = $this ->value;
            }

            if ($userSettingsName  &&  User::isAuthenticated()) {
                NastaveniUzivatele::setUserSettingsItem($database, $userSettingsName, $this ->value);
            }
        } else {
            if ($sessionName  &&  isset($_SESSION[$sessionName])) {
                $this ->value = $_SESSION[$sessionName];
            } else if ($userSettingsName  &&  User::isAuthenticated()) {
                $this ->value = NastaveniUzivatele::getUserSettingsItem($database, $userSettingsName);
            } else {
                $this ->value = $this ->defaultValue;
            }
        }

        // zkontroluje, jestli existuje zadana volba. Pokud ne, tak vybere prvni ze seznamu
        if ($this ->type !== self::TYPE_CHECKBOX  &&  $this ->type !== self::TYPE_HIDDEN  &&  isset($this ->value)  &&  !isset($this ->items[$this ->value])) {
            $this ->value = ArrayUtils::firstKey($this ->items);
            if ($this ->value === '') {
                $this ->value = null;
            }
        }
    }


    protected function renderComboBoxes()
    {
        $combo = new ComboBox($this ->name, $this ->items);
        $combo->showCode = true;
        //$combo->loadData($this->database, $sql . ' ORDER BY Absence DESC, Popis, Kod', $this ->value);
//        $combo->htmlParams = ['id' => 'filtr_operace', 'data-ron-select2' => '#filtr_operace_kod', 'data-ron-autosubmit' => '1'];
        $combo->selectedOption = $combo ->findOption($this ->value);
        $combo->titleFieldName = 'title';
        $combo
            ->setClassName('adsfilterforminput_popis ' . $this ->addClass)
            ->setHtmlParams($this ->htmlAttributes);
        $combo->render();

        $combo = new ComboBox('', $this->items2);
//        $combo->keyIndex = 0;
//        $combo->captionIndex = 0;
//        $combo->emptyRow = TextManager::_('_vse_');
//        $combo->loadData($this->database, $sql . ' ORDER BY Absence DESC, Kod', $this ->value);
//        $combo->htmlParams = 'id="filtr_operace_kod" data-ron-select2="#filtr_operace" data-ron-autosubmit="1"';
        $combo->selectedOption = $combo ->findOption($this ->value);
        $combo->titleFieldName = 'title';
        $combo
            ->setClassName('adsfilterforminput_kod desktop-only ' . $this ->addClass)  // na mobilu se zobrazi jen popis
            ->setHtmlParams($this ->htmlAttributes2);
        $combo->render();
    }

    public function render()
    {
        switch ($this ->type) {
            case self::TYPE_LINKS :
                LinksList::renderLinks($this ->items, $this ->value, $this ->name, '', false,
                    ['hiddenFieldName' => $this ->name, 'showSelectedItem' => $this ->showSelectedItem]);
                break;

            case self::TYPE_OPTIONS :
                ComboBox::renderComboBox($this ->name, $this ->items, $this ->value,
                    [
                        'className' => 'form-control adsfilterforminput_full ' . $this ->addClass, 'htmlParams' => array_merge(['data-ron-autosubmit=' => '1'], $this ->htmlAttributes ?? []),
                        'showCode' => $this->showCode,
                        'dataTypeComparison' => false
                    ]); // PHP automaticky konvertuje indexy na integer! (napr ['' => 'vse', '0' => 'bez', '1' => 's']
                break;

            case self::TYPE_EDIT :
                echo "<input name='{$this ->name}' class='adsfilterforminput_full form-control {$this ->addClass}' value='" . StringUtils::htmlTextConvert($this->value ?? '') . "'/>";
                break;

            case self::TYPE_CHECKBOX :
                echo
                    "<input type='hidden' id='filtr_checkbox_{$this ->name}' name='{$this ->name}' value='" . ($this ->value ? 1 : 0) . "'/>" .
                    "<label>" .
                    "<input type='checkbox' class='clickAutoSubmit setValue {$this ->addClass}'" .
                    " data-selector='#filtr_checkbox_{$this ->name}' data-submit='1' data-value='" . ($this ->value ? 0 : 1) . "'" . ($this ->value ? " checked='checked'" : '') .
                    '/>' .
                    ($this ->caption2 ?? '') .
                    '</label>';
                break;

            case self::TYPE_TEXT :
                echo $this ->value;
                break;

            case self::TYPE_COMBOBOXES :
                $this ->renderComboBoxes();
                break;

            case self::TYPE_HIDDEN :
                echo
                    '<input ' . GuiUtils::getAttributesHtml(array_merge(
                        ['type' => 'hidden', 'name' => $this->name, 'value' => StringUtils::htmlTextConvert($this ->value ?? '')],
                        $this ->htmlAttributes ?: [])) . "/>";
        }
    }


    /**
     * Sestavi SQL filtr podle aktualne zvolene hodnoty
     * @param Database $database
     * @param Parameters $params
     * @return string|null
     */
    public function buildSQLFilter(Database $database, Parameters $params)
    {
//        Debugger::dump($this ->items);
//        Debugger::dump($this ->value);
//        Debugger::dump($this ->items[$this ->value]);
        if (!$this ->enabled) {
            return null;
        }

        if ($this ->type === self::TYPE_CHECKBOX) {
            return $this ->value
                ? $this ->sqlFilter
                : null;
        }

        $item = $this ->items[$this ->value] ?? null;

        if (($item instanceof FilterFormFilterItem)  &&  $item ->sqlFilter) {
            if ($item->sqlFilter instanceof SqlStatement) {
                $params->appendParameters($item->sqlFilter->parameters);
                return $item->sqlFilter->sqlText;
            }
            return $item ->sqlFilter;
        }

        if (!isset($this ->value)) {
            return null;
        }

        if ($this ->sqlFilter) {
            return strpos($this ->sqlFilter, ' LIKE ')
                ? $this ->sqlFilter . $params ->addParameter("%{$this ->value}%") // sql filtr obsahuje LIKE
                : $this ->sqlFilter . $params ->addParameter($this ->value);
        }

        return null;
    }
}
