<?php

/**
 * Seznam eNeschopenek
 */

namespace RonPortal\Dochazka;


use Ron\Controls\BrowserField;
use Ron\Controls\LinksList;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\SqlStatement;
use Ron\TextManager;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\RonPortalSettings;
use RonPortal\RonPortalUtils;
use RonPortal\Rozdeleni;
use RonPortal\RozdeleniUtils;

class ENeschopenkaFilterForm extends RonPortalFilterForm
{
    /** @var int 0 = vse, 1 = ukoncene, 2 = neukoncene */
    public $ukoncene = 0;

    private function loadUkoncene()
    {
        if (isset($_GET['ukoncene'])) {
            $_SESSION['ukoncene'] = $_GET['ukoncene'];
        }

        $this->ukoncene = isset($_SESSION['ukoncene']) ? $_SESSION['ukoncene'] : 0;
    }

    function loadData()
    {
        parent::loadData();
        $this->loadUkoncene();
    }

    private function renderJenUkoncene()
    {
        echo '<div class="filterrow wrap">';
        echo '<span class="filtercaption"></span>';

        LinksList::renderLinks(
            [
                0 => TextManager::_('Vse'),
                1 => TextManager::_('E_neschopenka_ukoncene'),
                2 => TextManager::_('E_neschopenka_neukoncene'),
            ],
            $this->ukoncene,
            '?menuid=eneschopenka&ukoncene='
        );

        echo "</div>\n";
    }

    function renderFilters()
    {
        parent::renderFilters();
        $this->renderJenUkoncene();
    }
}


class PricinaNemociBrowserField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        return TextManager::_('E_neschopenka_pricina_' . (int)$value);
    }
}


class ENeschopenkaBrowser extends RonPortalBrowser
{
    /** @var PravaNaOsoby */
    private $prava;
    
    /** @var ENeschopenkaFilterForm */
    public $filterForm;

    /**
     * ENeschopenkaBrowser constructor.
     * @param Database $database
     * @param PravaNaOsoby $prava
     * @throws \Exception
     */
    function __construct($database, $prava)
    {
        parent::__construct($database, 'eneschopenka');

        $this->prava = $prava;

        $this->code = 'browsereneschopenka';

        $this->title = TextManager::_('E_neschopenka');
        $this->cssClass = 'browser';

        //$this->sqlSelectPart = "Osoba.Jmeno, Osoba.Prijmeni, DruhDavkyNZ.Popis, ENeschopenka.DatumPocatkuDavky, ENeschopenka.DatumUkonceniDavky, ENeschopenka.UkoncenaDavka"; // 0 - 5

        $novaVerze = false;
        if (RonPortalUtils::jeNainstalovanaAplikace($this->database, 'ADS') > DateUtils::dateStringToTimestamp('25.11.2020') ||
            RonPortalUtils::jeNainstalovanaAplikace($this->database, 'MZDY') > DateUtils::dateStringToTimestamp('25.11.2020')) {
            $novaVerze = true;
            //$this->sqlSelectPart .= ', ENeschopenka.PricinaNemoci, {fn_CASTWIDEMEMO(ENeschopenka.Poznamka)}'; // 6,7
        }

        $this->sqlFromPart =
            'ENeschopenka JOIN Osoba ON ENeschopenka.RC = Osoba.RC' .
            ' JOIN DruhDavkyNZ ON DruhDavkyNZ.Kod = ENeschopenka.DruhDavky';

        $this ->filterFormAppCode = 'ADS';
        $this ->filterFormRights = 'eneschopenka_prohl';
        $this ->filterFormVcetneExternich = true;

        $this->setFilterForm();
        $this->filterForm->loadData();

        $this ->addField()
            ->setSqlFieldName('Osoba.Prijmeni') ->setHeaderText('Prijmeni') ->setOrdering(1) ->setClassName('font-weight-bold')
            ->setCode('col-prijmeni')       // ->setWrap(1)
            ->setVisible($this->filterForm->vsechnyOsoby);

        $this ->addField()
            ->setSqlFieldName('Osoba.Jmeno') ->setHeaderText('Jmeno') ->setOrdering(1) ->setClassName('font-weight-bold')
            ->setCode('col-prijmeni')       // ->setWrap(1)
            ->setVisible($this->filterForm->vsechnyOsoby);

        $this->addField()
            ->setSqlFieldName('DruhDavkyNZ.Popis') ->setHeaderText('E_neschopenka_druh') ->setOrdering(1) ->setAlign('center')
            ->setCode('col-druhdavky');

        $this->addField()
            ->setSqlFieldName('ENeschopenka.DatumPocatkuDavky') ->setHeaderText('E_neschopenka_datum_pocatku') ->setOrdering(1) ->setAlign('center') ->setFormat('date')
            ->setCode('col-datumpocatkudavky');

        //if (!$this->filterForm->vsechnyOsoby) {
            $this->orderFieldIndex = count($this->fields);
        //}

        $this->addField()
            ->setSqlFieldName('ENeschopenka.DatumUkonceniDavky') ->setHeaderText('E_neschopenka_datum_ukonceni') ->setOrdering(1) ->setAlign('center') ->setFormat('date')
            ->setCode('col-datumukoncenidavky');

        $this->addField()
            ->setSqlFieldName('ENeschopenka.UkoncenaDavka') ->setHeaderText('E_neschopenka_ukoncena_davka') ->setOrdering(1) ->setAlign('center') ->setFormat('anone')
            ->setCode('col-ukoncenadavka');

        if ($novaVerze) {
            $this->addField(new PricinaNemociBrowserField($this->database))
                ->setSqlFieldName('ENeschopenka.PricinaNemoci') ->setHeaderText('E_neschopenka_pricina_nemoci') ->setOrdering(1) ->setAlign('center')
                ->setCode('col-pricinanemoci');

            $this->addField()
                ->setSqlFieldName('ENeschopenka.Poznamka') ->setHeaderText('Poznamka') ->setOrdering(1)
                ->setCode('col-poznamka');
        }
    }


    function setFilterForm()
    {
        $this->filterForm = new ENeschopenkaFilterForm($this ->database, $this ->action);

        if ($this ->filterFormAppCode) {
            $this->filterForm->nastavObdobi(true, $this->filterFormAppCode, false); // jenotevrene = false
        }

        $prava = Prava::vratPrava($this ->filterFormRights);

        if ($prava ->zobrazitVyberRozdeleni()) { // napr. pro "pouze vlastni" se vyber nezobrazuje
            $this ->filterForm ->nastavRozdeleni(true);
            $this ->filterForm ->f_pravarozdeleni = $prava;
            $this->filterForm->vcetnePodrizenychRozdeleni = true;

            if ( RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP)) {
                $this ->filterForm ->nastavRozdeleni2(true);
                $this->filterForm->vcetnePodrizenychRozdeleni2 = true;
            }
        }

        $this->filterForm ->nastavOsoby(true, $prava, 0, 0, $this ->filterFormVcetneExternich); // datum od-do podle obdobi
        $this->filterForm ->nastavAkciGet('eneschopenka');
        $this->filterForm->povolitVsechyOsoby = true;
    }

    function getSQLFilter()
    {
        $params = $this->database->createParameters();

        $sql =
            'ENeschopenka.DatumPocatkuDavky <= ' . $params->addParameter($this->database->dateToParam($this->filterForm->vybraneObdobiDo)) . ' AND ' .
            '(ENeschopenka.DatumUkonceniDavky >= ' . $params->addParameter($this->database->dateToParam($this->filterForm->vybraneObdobiOd)) . ' OR ENeschopenka.DatumUkonceniDavky IS NULL OR ENeschopenka.UkoncenaDavka = 0)';

        if ($this->filterForm->vsechnyOsoby) {
            $sql .=
                ' AND EXISTS(SELECT ID FROM PracovniPomer WHERE PracovniPomer.RC = Osoba.RC AND PracovniPomer.DatumNastupu <= ' . $params->addParameter($this->database->dateToParam($this->filterForm->vybraneObdobiDo)) .
                '           AND (PracovniPomer.DatumUkonceni >= ' . $params->addParameter($this->database->dateToParam($this->filterForm->vybraneObdobiOd)) . ' OR PracovniPomer.DatumUkonceni IS NULL)' .
                ' AND ' . $this->prava->sqlFiltr('PracovniPomer', $this->filterForm->vybraneObdobiOd, $this->filterForm->vybraneObdobiDo, $params);

            if (isset($this->filterForm->kodRozdeleni)) {
                $sql .=
                    ' AND ' .
                    RozdeleniUtils::sqlFilterRozdeleniPP($this->database,
                        'PracovniPomer',
                        RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP), $this->filterForm->kodRozdeleni,
                        $this->filterForm->vybraneObdobiOd, $this->filterForm->vybraneObdobiDo,
                        $this->filterForm->vcetnePodrizenychRozdeleni,
                        $params);
            }

            if (isset($this->filterForm->kodRozdeleni2)) {
                $sql .=
                    ' AND ' .
                    RozdeleniUtils::sqlFilterRozdeleniPP($this->database,
                        'PracovniPomer',
                        RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP), $this->filterForm->kodRozdeleni2,
                        $this->filterForm->vybraneObdobiOd, $this->filterForm->vybraneObdobiDo,
                        $this->filterForm->vcetnePodrizenychRozdeleni2,
                        $params);
            }

            $sql .=
                ')';
        } else {
                $sql .= ' AND ENeschopenka.RC = ' . $params->addParameter($this->filterForm->vybranaOsobaRC);
        }

        switch ($this->filterForm->ukoncene) {
            case 0 :
            default :
                break;
            case 1 :
                $sql .= ' AND ENeschopenka.UkoncenaDavka = 1';
                break;
            case 2 :
                $sql .= ' AND ENeschopenka.UkoncenaDavka = 0';
                break;
        }

        return new SqlStatement($sql, $params);
    }
}
