<?php
/**
 * roman.Szablatura 22.11.2016
 */

namespace RonPortal\Napady;

use Exception;
use Ron\Controls\BrowserField;
use Ron\Controls\ComboBox;
use Ron\Controls\Option;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\SqlStatement;
use Ron\GuiUtils;
use Ron\TextManager;
use RonPortal\Controls\ComboBoxOsoba;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\User;


/**
 * Class NapadyFilterForm
 * @package RonPortal\Napady
 */
class NapadyFilterForm extends RonPortalFilterForm
{
    /** @var  PravaNaOsoby */
    private $prava;

    /** @var int|null   Hodnota filtru stavu napadu */
    public $stavNapadu;

    /** @var int|null   Hodnota filtru autora napadu */
    public $autorNapadu;
    /** @var int|null   Hodnota filtru autora napadu */
    public $pilotNapadu;

    public $oblastNapadu;

    public $prioritaNapadu;


    /**
     * NapadyFilterForm constructor.
     * @param Database $database
     */
    function __construct($database)
    {
        parent::__construct($database, 'napady', true);

        $this->prava = Prava::vratPrava('napady');

//        $this->f_formaction = '?menuid=napady';
//        $this->f_classname = 'filtercontainer leftcaptions';
        $this ->nastavLink('<a href="?menuid=napady&formaction=new">' . TextManager::_('zapis_noveho_napadu') . '</a>');
    }

    /**
     * @throws DBException
     */
    function renderStavNapadu()
    {
        echo '<div class="filterrow">';
        echo
            '<span class="filtercaption">' .
            TextManager::_('stav_napadu') .
            '</span>';

        $combo = new ComboBox('stav');
        $combo
            ->setClassName('adsfilterforminput_full')
            ->addHtmlParam('data-ron-autosubmit', '1')
            ->setEmptyRow(TextManager::_('_vse_'))
            ->setDataTypeComparison(false);

        $combo->loadData($this->database, 'SELECT DISTINCT Kod, Popis FROM StavNapadu ORDER BY Kod', $this ->stavNapadu);
        $combo->render();

        echo '</div>';
    }


    /**
     * @throws DBException
     */
    function renderOblastNapadu()
    {
        echo '<div class="filterrow">';
        echo
            '<span class="filtercaption">' .
            TextManager::_('oblast_napadu') .
            '</span>';

        $combo = new ComboBox('oblast');
        $combo
            ->setClassName('adsfilterforminput_full')
            ->addHtmlParam('data-ron-autosubmit', '1')
            ->setEmptyRow(TextManager::_('_vse_'))
            ->setShowCode(true);

        $combo->loadData($this->database,
            'SELECT Kod, Popis FROM OblastNapadu ORDER BY Popis',
            $this ->oblastNapadu);
        $combo->render();

        echo '</div>';
    }


    function renderAutorNapadu()
    {
        echo '<div class="filterrow">';
        echo
            '<span class="filtercaption">' .
            TextManager::_('autor') .
            '</span>';

        $datumOd = $datumDo = DateUtils::today();

        $combo = new ComboBoxOsoba('autor');
        $combo
            ->setClassName('adsfilterforminput_full')
            ->addHtmlParam('data-ron-autosubmit', '1')
            ->setDataTypeComparison(false);

        $combo->filtr = ' EXISTS(SELECT * FROM Napad WHERE Napad.Autor = Osoba.RC)';

        $combo->loadDataPP($this ->database,
            $datumOd, $datumDo, $this->prava,
            TextManager::_('_vsichni_'),
            $this ->autorNapadu ?: '',
            false);
        $combo->render();
        echo '</div>';
    }

    /**
     * @throws DBException
     */
    function renderFilterPilot()
    {
        echo '<div class="filterrow">';
        echo
            '<span class="filtercaption">' .
            TextManager::_('pilot') .
            '</span>';

        $datumOd = $datumDo = DateUtils::today();

        $combo = new ComboBoxOsoba('pilot');
        $combo
            ->setClassName('adsfilterforminput_full')
            ->addHtmlParam('data-ron-autosubmit', '1')
            ->setDefaultRows(['' => TextManager::_('_vsichni_'), '-' => TextManager::_('pilot_neni_urcen')])
            ->setDataTypeComparison(false);

        $combo->filtr = ' EXISTS(SELECT * FROM Napad WHERE Napad.Pilot = Osoba.RC)';

        $combo->addOption(new Option('cosi', 'neco'));

        $combo->loadDataPP($this ->database,
            $datumOd, $datumDo, $this->prava,
            null,
            //['' => TextManager::_('_vsichni_'), '-' => TextManager::_('pilot_neni_urcen')],
            $this ->pilotNapadu ?: '',
            false);
        $combo->render();
        echo '</div>';
    }


    /**
     * @throws DBException
     */
    function renderFilterPriorita()
    {
        echo '<div class="filterrow">';
        echo
            '<span class="filtercaption">' .
            TextManager::_('priorita') .
            '</span>';

        $sql =
            "SELECT DISTINCT Napad.Priorita, Napad.Priorita " .
            "FROM Napad " .
            "WHERE Napad.Priorita IS NOT NULL";

        $combo = new ComboBox('priorita');
        $combo
            ->setClassName('adsfilterforminput_full')
            ->addHtmlParam('data-ron-autosubmit', '1')
            ->setCaptionIndex(0)
            ->setEmptyRow(TextManager::_('_vse_'));

        $combo->loadData($this->database,
            'SELECT DISTINCT Priorita FROM Napad ' .
            ' WHERE Priorita IS NOT NULL' .
            ' ORDER BY Priorita', $this ->prioritaNapadu);
        $combo->render();
        echo '</div>';
    }

    function loadStavNapadu()
    {
        if (isset($_GET['stav'])) {
            $this ->stavNapadu = ($_GET['stav'] === '') ? null : (int)$_GET['stav'];
            $_SESSION['StavNapadu'] = $this ->stavNapadu;
        } else {
            $this ->stavNapadu = $_SESSION['StavNapadu'] ?? null;
        }
    }

    function loadAutorNapadu()
    {
        if (isset($_GET['autor'])) {
            $this ->autorNapadu = ($_GET['autor'] === '') ? null : (int)$_GET['autor'];
            $_SESSION['AutorNapadu'] = $this ->autorNapadu;
        } else {
            $this ->autorNapadu = $_SESSION['AutorNapadu'] ?? null;
        }
    }

    function loadOblastNapadu()
    {
        if (isset($_GET['oblast'])) {
            $this ->oblastNapadu = $_GET['oblast'] === '' ? null : $_GET['oblast'];
            $_SESSION['OblastNapadu'] = $this ->oblastNapadu;
        }
    }

    function loadFilterPilot()
    {
        if (isset($_GET['pilot'])) {
            $this ->pilotNapadu = ($_GET['pilot'] === '-')
                ? '-'
                : (($_GET['pilot'] === '') ? null : (int)$_GET['pilot']);
            $_SESSION['PilotNapadu'] = $this ->pilotNapadu;
        } else {
            $this ->pilotNapadu = $_SESSION['PilotNapadu'] ?? null;
        }
    }

    function loadFilterPriorita()
    {
        if (isset($_GET['priorita'])) {
            $this ->prioritaNapadu = $_GET['priorita'] === '' ? null : $_GET['priorita'];
            $_SESSION['PrioritaNapadu'] = $this ->prioritaNapadu;
        }
    }

    function loadData()
    {
        $this->loadStavNapadu();
        $this->loadAutorNapadu();
        $this->loadOblastNapadu();
        $this->loadFilterPilot();
        $this->loadFilterPriorita();
        parent::loadData();
    }

    /**
     * @throws DBException
     */
    function renderFilters()
    {
        $this->renderStavNapadu();
        $this->renderAutorNapadu();
        $this->renderOblastNapadu();
        $this->renderFilterPilot();
        $this->renderFilterPriorita();

        parent::renderFilters();
    }
}


class PrirazeniPilotaKNapadu extends BrowserField
{
    /** @var  PravaNaOsoby */
    var $prava_admin;

    /**
     * PrirazeniPilotaKNapadu constructor.
     * @param null|Database $database
     * @param PravaNaOsoby $prava_admin
     */
    function __construct($database, $prava_admin)
    {
        parent::__construct($database);
        $this->prava_admin = $prava_admin;
    }

    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if (trim($value)) {
            return parent::getViewText($value, $rowNumber, $recordId, $rowData);
        }

        if ($this->prava_admin->jsouNejakePrava()) {
            return '<a id="odkaz_pilot" href="?menuid=napady&formaction=edit&record=' . $rowData[0] . '">' . TextManager::_('prirazeni_pilota') . '</a>';
        }

        return '';
    }
}


/**
 * Class BarvaNapadu
 * @package RonPortal\Napady
 */
class BarvaNapadu extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        return
            "<i class='inline-glyph fa glyph-napad glyph-napad-{$rowData[3]}'></i>";
    }
}


/**
 * Class PocetDokumentuNapaduField
 * @package RonPortal\Napady
 */
class PocetDokumentuNapaduField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($value) {
            return '<i class="fa inline-glyph glyph-attachment" title="' . TextManager::_('Pocet_priloh') . ': ' . $value . '"></i>';
        }
        return '';
    }
}


/**
 * Class NapadyBrowser
 * @package RonPortal\Napady
 */
class NapadyBrowser extends RonPortalBrowser
{
    /** @var  PravaNaOsoby */
    var $prava;

    /** @var  PravaNaOsoby */
    var $prava_admin;

    /** @var string[] */
    private $colorsCSS = [];


    /**
     * NapadyBrowser constructor.
     * @param Database $database
     * @throws Exception
     */
    function __construct( $database)
    {
        parent::__construct($database, 'napady');

        $this->prava = Prava::vratPrava('napady');
        $this->prava_admin = Prava::vratPrava('napady_admin');

        Prava::nastavObdobiProPrava(time(),time());

        $this->title = TextManager::_('evidence_napadu');
        $this->code = 'browsernapady';
        $this->tableId = 'browsernapady';
        $this->browserUrl = '?menuid=napady';

        $this->sqlSelectPart =
            'Napad.ID, Autor.RC, Pilot.RC,' .       // 0 - 2 (primarni klic pro odkazy, rc pro kontrolu prav)
            ' StavNapadu.Kod, StavNapadu.Barva';    // 3,4   (pro zobrazeni barevnych ikon)

        $this->sqlFromPart =
            'Napad LEFT JOIN Osoba Autor ON (Napad.Autor = Autor.RC)' .
            ' LEFT JOIN Osoba Pilot ON (Napad.Pilot = Pilot.RC)' .
            ' LEFT JOIN StavNapadu ON (Napad.Stav = StavNapadu.Kod)' .
            ' LEFT JOIN OblastNapadu ON (Napad.Oblast = OblastNapadu.Kod)';

        $this->primaryField = 0;

        $this->filterForm = new NapadyFilterForm($this->database);
        $this->filterForm->loadData();

        $this->addField(new BarvaNapadu($this->database))
            ->setCode('col-icon');

        $this->addField()
            ->setSqlFieldName('Napad.Datum') ->setHeaderText('datum_vlozeni')
            ->setCode('col-datum')
            ->setFormat('finedate') ->setOrdering(true) ->setAlign('center');
        // Seradit dle data
        $this->defaultSort = 'col-datum';

        $this->addField()
            ->setSqlFieldName('StavNapadu.Popis') ->setHeaderText('stav_napadu')
            ->setCode('col-stav')
            ->setOrdering(true) ->setLink('?menuid=napady&action=zaznam');

        $this->addField()
            ->setSqlFieldName("Coalesce (Autor.Prijmeni, '') {fn_concat()} ' ' {fn_concat()} Coalesce (Autor.Jmeno, '') {fn_concat()} Coalesce (', ' {fn_concat()}  Autor.Titul,'') Autor")
            ->setHeaderText('autor') ->setOrdering(true)
            ->setSortSQL('Autor.Prijmeni, Autor.Jmeno') ->setSortSQLDesc('Autor.Prijmeni DESC, Autor.Jmeno DESC')
            ->setCode('col-autor');

        // Oblast napadu s kodem
        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Napad.Oblast')
            ->setCode('col-oblast');

        $this->addField()
            ->setSqlFieldName('OblastNapadu.Popis') ->setSqlFieldIndex2('col-oblast')
            ->setCode('col-popisoblasti')
            ->setOrdering(true) ->setHeaderText('oblast_napadu');

        $this->addField()
            ->setSqlFieldName('Nazev') ->setHeaderText('nazev_napadu')
            ->setCode('col-nazev')
            ->setOrdering(true) ->setLink('?menuid=napady&action=zaznam');

        // Pilot
        $this->addField(new PrirazeniPilotaKNapadu($this->database, $this->prava_admin))
            ->setSqlFieldName("Coalesce (Pilot.Prijmeni, '') {fn_concat()} ' ' {fn_concat()} Coalesce (Pilot.Jmeno, '') {fn_concat()} Coalesce (', ' {fn_concat()}  Pilot.Titul,'') Pilot")
            ->setSortSQL('Pilot.Prijmeni, Pilot.Jmeno') ->setSortSQLDesc('Pilot.Prijmeni DESC, Pilot.Jmeno DESC')
            ->setOrdering(true) ->setHeaderText('pilot')
            ->setCode('col-pilot');

        // Priorita
        $this->addField()
            ->setSqlFieldName('Priorita') ->setHeaderText('priorita') ->setOrdering(true)
            ->setCode('col-priorita');

        $this->addField()
            ->setSqlFieldName('Napad.Rozpocet') ->setHeaderText('rozpocet')
            ->setCode('col-rozpocet')
            ->setFormat('number_2') ->setOrdering(true) ->setAlign('right');

        $this->addField()
            ->setSqlFieldName('OdmenaCastka') ->setHeaderText('odmena')
            ->setCode('col-odmena')
            ->setFormat('number_2') ->setOrdering(true) ->setAlign('right');

        $this->addField()
            ->setSqlFieldName('OdmenaVyplacenaVObdobi') ->setHeaderText('obdobi_vyplaceni_odmeny')
            ->setCode('col-obdobiodmeny')
            ->setFormat('finedate') ->setOrdering(true) ->setAlign('center');

        $this->addField(new PocetDokumentuNapaduField())
            ->setSqlFieldName('(SELECT COUNT(*) FROM DokumentNapadu WHERE DokumentNapadu.IDNapadu = Napad.ID) AS PocetDokumentu')
            ->setCode('col-dokumenty')
            ->setSortSQL('PocetDokumentu')
            ->setAlign('center');

        $this->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Napad.Update_UserName')
            ->setCode('col-update_username');

        $this->addField() //new DatumPosledniAktualizaceField($this->database));
            ->setSqlFieldName('Napad.Update_Timestamp') ->setSqlFieldIndex2('col-update_username') ->setHeaderText('datum_editace')
            ->setCode('col-aktualizace')
            ->setFormat('date') ->setOrdering(true) ->setAlign('center');

//        //Pokud vchazim do Browseru mazu seznam ID napadu v session aby se vzdy vytvoril aktualni seznam pri kazde zmene filtru
//        unset($_SESSION['pocetnapadufilru']);
//        unset($_SESSION['seznamnapadu']);
    }


    function setFilterForm()
    {
        // nedela nic - filter form nastaven v konstuktoru
    }


    /**
     * @param array $values
     * @return bool
     * @throws Exception
     */
    function enableLine($values)
    {
        //Zobrazi pokud ma na autora pravo nebo je pilotem napadu
        if ($this->prava->lzeZobrazitRC($values[1]) || $this->prava_admin->lzeZobrazitRC($values[1]) || User::hasRC($values[2])) {
//            //Vytvarim seznam nazvu napadu oddelenych ;
//            $_SESSION['seznamnapadu'] .=  $values[6] . ';;';
//
//            if ($_SESSION['pocetnapadufilru'] != ''){
//                $_SESSION['pocetnapadufilru'] .= ',';
//            }
//            $_SESSION['pocetnapadufilru'] .= $values[0];
            return true;
        }

        return false;
    }


    /**
     * @return SqlStatement|string
     * @throws DBException
     */
    public function getSQLFilter()
    {
        $result = '';

        $params = $this->database->createParameters();

        if (isset($this ->filterForm ->stavNapadu)){
            $result .= 'StavNapadu.Kod = ' . $params->addParameter($this ->filterForm ->stavNapadu);
        }

        if ($this ->filterForm ->autorNapadu){
            $pracovniPomerAutora = PracovniPomer::nactiPP($this->database, $this ->filterForm ->autorNapadu);
            $result .=
                ($result ? ' AND ' : '') .
                ' Napad.Autor = ' . $params->addParameter($pracovniPomerAutora->rc);
        }

        if ($this ->filterForm ->oblastNapadu){
            $result .=
                ($result ? ' AND ' : '') .
                ' Napad.Oblast = ' . $params->addParameter($this ->filterForm ->oblastNapadu);
        }

        if ($this ->filterForm ->pilotNapadu){
            if ($this ->filterForm ->pilotNapadu === '-') {
                $result .=
                    ($result ? ' AND ' : '') .
                    ' Napad.Pilot IS NULL';
            } else {
                $pracovniPomerPilota = PracovniPomer::nactiPP($this->database, $this->filterForm->pilotNapadu);
                $result .=
                    ($result ? ' AND ' : '') .
                    ' Napad.Pilot = ' . $params->addParameter($pracovniPomerPilota->rc);
            }
        }

        if ($this ->filterForm ->prioritaNapadu){
            $result .=
                ($result ? ' AND ' : '') .
                ' Napad.Priorita = ' . $params->addParameter($this ->filterForm ->prioritaNapadu);
//            if ($_SESSION['priorita'] != ''){
//                $result .= " Napad.Priorita = " . $params->addParameter($_SESSION['priorita']);
//            } else {
//                $result .= ' Napad.Priorita IS NULL';
//            }
        }

        return $result
            ? new SqlStatement($result, $params)
            : null;
    }

    public function renderLine($values, $className, $rowNumber, $edit = false)
    {
        parent::renderLine($values, $className, $rowNumber, $edit);

        if (!isset($this ->colorsCSS[$values[3]])  &&  isset($values[4])) {
            $this ->colorsCSS[$values[3]] = $values[4];
        }
    }


    public function render()
    {
        $this ->colorsCSS = [];

        parent::render();

        // styl pro obarveni ikon
        if ($this ->colorsCSS) {
            echo "\n<style>";

            foreach ($this ->colorsCSS as $key => $value) {
                echo " .glyph-napad-$key {color:" . GuiUtils::getHtmlColor($value) . ';}';
            }

            echo "</style>\n";
        }
    }


}