<?php
/**
 * UTF-8
 * Created by PhpStorm.
 * User: pavel.rudol
 * Date: 15.3.2016
 * Time: 10:12
 */

namespace RonPortal\PokladniKniha;

use Exception;
use Ron\Controls\Browser;
use Ron\Controls\BrowserField;
use Ron\Controls\ComboBox;
use Ron\Controls\LinksList;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\DB\SqlStatement;
use Ron\DB\SQLUtils;
use Ron\GuiUtils;
use Ron\Http\Csrf;
use Ron\TextManager;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Controls\RozdeleniBrowserField;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\RonPortalSettings;
use RonPortal\RozdeleniUtils;
use RonPortal\TypRozdeleni;
use function Ron\ronText;

class PokladniKnihaFilter extends RonPortalFilterForm
{
    private $pravaProhlizet;
    private $pravaVlozit;
    private $pravaSchvalit1;
    private $pravaSchvalit2;
    private $pravaSchvalit3;
    private $pravaSchvalit4;

    /** @var  SeznamPolozekPokladniKnihy */
    private $seznamPolozekPokladniKnihy;

    /**
     * PokladniKnihaFilter constructor.
     * @param Database $database
     * @param string $action
     * @param SeznamPolozekPokladniKnihy $seznamPolozekPokladniKnihy
     */
    public function __construct($database, $action, $seznamPolozekPokladniKnihy)
    {
        $this->seznamPolozekPokladniKnihy = $seznamPolozekPokladniKnihy;

        $this->pravaProhlizet = Prava::vratPrava('pkniha_prohl');
        $this->pravaVlozit = Prava::vratPrava('pkniha_vloz');
        $this->pravaSchvalit1 = Prava::vratPrava('pkniha_schval1');
        $this->pravaSchvalit2 = Prava::vratPrava('pkniha_schval2');
        $this->pravaSchvalit3 = Prava::vratPrava('pkniha_schval3');
        $this->pravaSchvalit4 = Prava::vratPrava('pkniha_schval4');

        parent::__construct($database, $action);

        $this->f_formaction = '?menuid=' . $action . '&browser=' . $this->seznamPolozekPokladniKnihy->cislo;

        $this->nastavOsoby(true, [$this->pravaProhlizet, $this->pravaVlozit, $this->pravaSchvalit1, $this->pravaSchvalit2, $this->pravaSchvalit3, $this->pravaSchvalit4], time(),time());
        $this->povolitVsechyOsoby = true;

        $this->nastavDatumOdDo(true,false);

        $this->nastavRozdeleni(true);
        $this->nastavRozdeleni2(true);
    }

    /**
     * Filtr na typ polozky pokladni knihy
     * @throws DBException
     */
    private function renderTypPolozky()
    {
        echo '<DIV class="filterrow">';

        echo '<SPAN class="filtercaption">';
        GuiUtils::renderCheckbox('jenTypPolozkyPK', ronText('jen_typ') . '&nbsp;&nbsp;', isset($_SESSION['jenTypPolozkyPK']) && $_SESSION['jenTypPolozkyPK'], 'onClick="submit()"');

        echo '</SPAN>';

        // combobox pro vyber
        $params = $this->database->createParameters();

        $sql =
            'SELECT Kod, Popis FROM TypPolozkyPokladniKnihy ';

        if (!empty($this->seznamPolozekPokladniKnihy->seznamTypuSQL))
            $sql .=
                'WHERE ' . SQLUtils::getINCondition($this->database,'Kod',$this->seznamPolozekPokladniKnihy->seznamTypuSQL, $params);

        $sql .=
            ' ORDER BY Popis';

        // vyber typu udalosti podle popisu
        $combo = new ComboBox('typPolozkyPK');
        $combo->className = 'app_combo adsfilterforminput_full ronselect';
        $combo->setTranslationPrefix('db_typpolozkypokladniknihy_popis_');
        $combo->loadData($this ->database, $sql, isset($_SESSION['typPolozkyPK']) ? $_SESSION['typPolozkyPK'] : '', $params);
        $combo ->htmlParams = 'onChange="if (jenTypPolozkyPK.checked) submit();"';
        $combo->render();

        echo "</DIV>\n";
    }

    private function renderFilterStav()
    {
        echo '<div class="filterrow" id="filter_urovenschvaleni">';
        echo '<span class="filtercaption">' . ronText('PoklK_uroven_schvaleni') . '</span>';

        $data = array(
            0 => ronText('Vse'),
            -1 => ronText('PoklK_ke_schvaleni'),
            -2 => ronText('PoklK_bez_schvaleni'),
            1 => ronText('PoklK_schvaleni_uroven1'),
            2 => ronText('PoklK_schvaleni_uroven2'),
            3 => ronText('PoklK_schvaleni_uroven3'),
            4 => ronText('PoklK_schvaleni_uroven4')
        );

        LinksList::renderLinks($data, isset($_SESSION['urovenschvaleniPoklK']) ? $_SESSION['urovenschvaleniPoklK'] : 0, $this ->f_formaction .  '&urovenschvaleni=', '', false);

        echo "</div>\n";
    }

    /**
     * Vypise vsechny filtry
     * @throws DBException
     */
    function renderFilters()
    {
        parent::renderFilters();

        $this->renderTypPolozky();
        $this->renderFilterStav();

    }

    private function loadTypPolozky()
    {
        if (isset($_POST['adsfilterform'])) {
            $_SESSION['jenTypPolozkyPK'] = isset($_POST['jenTypPolozkyPK']);

            $_SESSION['typPolozkyPK'] = $_POST['typPolozkyPK'];
        }
    }

    private function loadUrovenSchvaleni()
    {
        if (isset($_GET['urovenschvaleni'])) {
            $_SESSION['urovenschvaleniPoklK'] = $_GET['urovenschvaleni'];
        }
    }

    function loadData()
    {
        parent::loadData();

        $this->loadTypPolozky();
        $this->loadUrovenSchvaleni();
    }

}

class SchvaleniZaznamuPKField extends BrowserField
{
    /**
     * @param Database $database
     */
    function __construct($database)
    {
        parent::__construct($database);

        $this->align = 'left';

        $this->header = ronText('PoklK_schvaleni');
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $hint = '';

        if ($rowData[39]) {
            $date = $rowData[39];
            $name = $rowData[40];
            $class = 'textschvaleni4';
        } elseif ($rowData[15]) {
            $date = $rowData[15];
            $name = $rowData[16];
            $class = 'textschvaleni3';
        } else if ($rowData[13]) {
            $date = $rowData[13];
            $name = $rowData[14];
            $class = 'textschvaleni2';
        } else if ($rowData[11]) {
            $date = $rowData[11];
            $name = $rowData[12];
            $class = 'textschvaleni1';
        }  else {
            return '';
        }

        $result = "<SPAN class='$class' title='$hint'>";
        $result .= DBUtils::getFormatedValue($this ->database, 'date', $date);
        if ($name) $result .= " ($name)";
        $result .= "</SPAN>\n";

        return $result;
    }
}

class SchvaleniZaznamuPKEditField extends BrowserField
{
    /** @var  PravaNaOsoby */
    public $pravaStr;
    public $indexDatum;
    public $indexUzivatel;
    public $indexTypPolozky;
    public $htmlIndex;

    /** @var  PokladniKnihaBrowser */
    public $browser;

    public $existsEdit;


    /**
     * @param Browser $browser
     * @param string $pravaStr
     * @param int $indexDatum
     * @param int $indexUzivatel
     * @param int $htmlIndex
     * @param string $caption
     */
    function __construct($browser, $pravaStr, $indexDatum, $indexUzivatel, $indexTypPolozky, $htmlIndex, $caption)
    {
        parent::__construct($browser ->database);
        $this ->align = "center";

        $this ->header =
            "<span id='schvaleniPKMenu_$htmlIndex''>$caption</span>";

        $this ->existsEdit = false;

        $this ->browser = $browser;

        $this ->htmlIndex = $htmlIndex;
        $this ->pravaStr = $pravaStr;
        $this ->indexDatum = $indexDatum;
        $this ->indexUzivatel = $indexUzivatel;
        $this ->indexTypPolozky = $indexTypPolozky;
    }


    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        /** @noinspection DuplicatedCode  */
        $schvaleno = $rowData[$this ->indexDatum];

        $prava = Prava::vratPrava($this->pravaStr, $this->pravaStr . '_' . $rowData[$this->indexTypPolozky]);

        $hint = '';
        if ($schvaleno) {
            $hint = $rowData[$this ->indexUzivatel];
            if ($this ->indexDatum  &&  $rowData[$this ->indexDatum]) $hint .= ' (' . DBUtils::getFormatedValue($this ->database, 'finedatetime', $rowData[$this ->indexDatum]) . ')';
        }

        if ($prava ->lzeZobrazitPP($rowData[27], $this->database->stringToTimestamp($rowData[1]), $this->database->stringToTimestamp($rowData[1]))) {
            // muze schvalovat
            $this ->existsEdit = true;

            return
                "<input type='hidden' name='NastavitSchvaleni_{$rowData[0]}_{$this ->htmlIndex}' value='1' />" .
                "<input type='checkbox' data-ronschvalenipoklk_index='{$this ->browser ->jsonRowIndex}' name='Schvaleni_{$rowData[0]}_{$this ->htmlIndex}' " .
                    "id='Schvaleni_{$rowData[0]}_{$this ->htmlIndex}' " .
                ($schvaleno ? " checked='checked'" : '') . ( $hint ? " title='$hint'" : '' ) . " />\n";
        } else {
            return '<span' . ( $hint ? " title='$hint'" : '' ) . '>' . ronText($schvaleno ? 'Str_Yes' : 'Str_No') . '</span>';
        }
    }


    /**
     * @param int $fieldIndex
     * @param string $url
     * @return string
     */
    function getFooterText($fieldIndex, $url = null)
    {
        if ($this ->existsEdit) {
            return "<input type='checkbox' name='schvaleni_ALL_{$this ->htmlIndex}' id='schvaleni_ALL_{$this ->htmlIndex}' title='' />\n";
        }
        else return '';
    }

}

class ZruseniPKField extends BrowserField
{
    /** @var  string */
    public $pravaStr;
    public $indexTypPolozky;

    /**
     * @param Database $database
     * @param string $pravaStr
     */
    function __construct($database, $pravaStr, $indexTypPolozky)
    {
        parent::__construct($database);
        $this->pravaStr = $pravaStr;
        $this->indexTypPolozky = $indexTypPolozky;
    }


    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws Exception
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($rowData[11] || $rowData[13] || $rowData[15] || $rowData[39] || $rowData[41]) return ''; // jiz je schvaleno, nejde smazat

        $prava = Prava::vratPrava($this->pravaStr, $this->pravaStr . '_' . $rowData[$this->indexTypPolozky]);

        // zkontroluje prava na editaci zastupu
        if ($prava->lzeZobrazitPP($rowData[27], $this ->database ->stringToTimestamp($rowData[1]), $this ->database ->stringToTimestamp($rowData[1]))) {
            return GuiUtils::changeLocationQuest('<i class="inline-glyph glyph-button fa glyph-delete" title="' . ronText('PoklK_zrusit_zaznam') . '"></i>', ronText('PoklK_opravdu_zrusit'),
                '?menuid=pokladnikniha&action=zrusitzaznam&record=' . $rowData[0] . Csrf::csrfUrlParameter());
        }

        return '';
    }
}

class DokumentyPKField extends BrowserField
{
    function __construct()
    {
        parent::__construct();
        $this->align = 'center';
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        if ($rowData[28]) {
            return
                '<i class="inline-glyph glyph-button fa glyph-attachment" title="' . ronText('Pocet_priloh') . ': ' . $rowData[28] . '"></i>';
        }

        return '';
    }
}

class PokladniKnihaBrowser extends RonPortalBrowser
{
    /** @var  PokladniKnihaFilter */
    public $filterForm;
    private $schvalovani;
    /** @var  SeznamPolozekPokladniKnihy */
    private $seznamPolozekPokladniKnihy;

    private $pravaProhlizet;
    private $pravaVlozit;
    private $pravaSchvalit1;
    private $pravaSchvalit2;
    private $pravaSchvalit3;
    private $pravaSchvalit4;

    public $jsonRowIndex;
    public $jsonObject;
    public $hiddenFields;

    /**
     * PokladniKnihaBrowser constructor.
     * @param $database
     * @param $cisloBrowseru
     * @param bool $schvalovani
     * @throws DBException
     */
    public function __construct($database, $cisloBrowseru, $schvalovani = false)
    {
        parent::__construct($database, 'pokladnikniha');

        $this->seznamPolozekPokladniKnihy = SeznamPolozekPokladniKnihy::vratBrowser($this->database, $cisloBrowseru);

        $this->pravaVlozit = Prava::vratPrava('pkniha_vloz');

        //nactou se obecna prava bez filtru na zpusob pohybu (aby fungoval filtr)
        $this->pravaSchvalit2 = Prava::vratPrava('pkniha_schval2');
        $this->pravaSchvalit3 = Prava::vratPrava('pkniha_schval3');
        $this->pravaSchvalit4 = Prava::vratPrava('pkniha_schval4');

        $this->title = ronText('pokladni_kniha' . ($cisloBrowseru != 1 ? "_$cisloBrowseru" : ''));

        $this->browserUrl = "?menuid=pokladnikniha&browser=$cisloBrowseru&";
        $this->code = 'browserpokladnikniha' . ($cisloBrowseru != 1 ? "_$cisloBrowseru" : '');
        $this->tableId = 'seznampokladniknihy';

        $this->schvalovani = $schvalovani;
        $this->showFooter = $schvalovani;

        $this->sqlSelectPart =
            'PokladniKniha.ID, PokladniKniha.DatumPohybu, PokladniKniha.RC, PokladniKniha.OsobniCislo, PokladniKniha.DatumNastupu, PokladniKniha.Castka, ' . //0 - 5
            'PokladniKniha.Mena, PokladniKniha.TypPolozky, PokladniKniha.Popis, PokladniKniha.Poznamka, PokladniKniha.IDDokumentu, ' . //6 - 10
            'PokladniKniha.Schvaleni1, PokladniKniha.Schvalil1, ' . //11, 12
            'PokladniKniha.Schvaleni2, PokladniKniha.Schvalil2, ' . //13, 14
            'PokladniKniha.Schvaleni3, PokladniKniha.Schvalil3, ' . //15, 16
            'PokladniKniha.PoznamkaKVraceni, PokladniKniha.HistorieSchvalovani, ' . //17, 18
            'PokladniKniha.ObdobiVyuctovani, PokladniKniha.Insert_UserName, PokladniKniha.Insert_Timestamp, PokladniKniha.Update_UserName, PokladniKniha.Update_Timestamp, ' . //19 - 23
            'Osoba.Jmeno, Osoba.Prijmeni, TypPolozkyPokladniKnihy.Popis, PracovniPomer.ID, ' . //24 - 27
            '(SELECT COUNT(*) FROM DokumentPokladniKnihy WHERE IDPokladniKnihy = PokladniKniha.ID), ' . //28
            'PokladniKniha.CastkaBezDPH, PokladniKniha.CastkaDPH, PokladniKniha.KodTypuRozdeleni, PokladniKniha.Rozdeleni, PokladniKniha.KodTypuRozdeleni2, PokladniKniha.Rozdeleni2, ' .  //29 - 34
            'Rozdeleni.Popis, R2.Popis, PokladniKniha.Rozdeleni3, R3.Popis, PokladniKniha.Schvaleni4, PokladniKniha.Schvalil4, ' . //35 - 40
            'PokladniKniha.Zauctovano, PokladniKniha.Zauctovano_Uzivatel, PokladniKniha.Zauctovano_DatumCas'; // 41 - 43

        if (PokladniKnihaController::zobrazitRozdeleni4($this->database)) {
            $this->sqlSelectPart .=
                ', PokladniKniha.Rozdeleni4, R4.Popis'; //44 - 45
        }

        $this->sqlFromPart =
            'PokladniKniha  ' .
            'JOIN Osoba ON (Osoba.RC = PokladniKniha.RC) ' .
            'JOIN TypPolozkyPokladniKnihy ON (TypPolozkyPokladniKnihy.Kod = PokladniKniha.TypPolozky) ' .
            'JOIN PracovniPomer ON (PracovniPomer.RC = PokladniKniha.RC AND PracovniPomer.OsobniCislo = PokladniKniha.OsobniCislo AND PracovniPomer.DatumNastupu = PokladniKniha.DatumNastupu) ' .
            'LEFT OUTER JOIN Rozdeleni ON Rozdeleni.KodTypuRozdeleni = PokladniKniha.KodTypuRozdeleni AND Rozdeleni.Kod = PokladniKniha.Rozdeleni '.
            'LEFT OUTER JOIN Rozdeleni R2 ON R2.KodTypuRozdeleni = PokladniKniha.KodTypuRozdeleni2 AND R2.Kod = PokladniKniha.Rozdeleni2 ' .
            'LEFT OUTER JOIN Rozdeleni R3 ON R3.KodTypuRozdeleni = PokladniKniha.KodTypuRozdeleni3 AND R3.Kod = PokladniKniha.Rozdeleni3 ';

        if (PokladniKnihaController::zobrazitRozdeleni4($this->database)) {
            $this->sqlFromPart .=
                'LEFT OUTER JOIN Rozdeleni R4 ON R4.KodTypuRozdeleni = PokladniKniha.KodTypuRozdeleni4 AND R4.Kod = PokladniKniha.Rozdeleni4';
        }

        $this->primaryField = 0;

        $f = $this->addField()
            ->setHeaderText('Prijmeni')->setSqlFieldIndex(25)->setSqlFieldName('Osoba.Prijmeni')
            ->setAlign('left')->setLink('?menuid=pokladnikniha&action=zaznam')->setOrdering(true)
            ->setCode('col-prijmeni');

        if ($this ->schvalovani) {
            $f ->setFooter(TextManager::_('Schvalit_hromadne'));
        }

        $this->addField()
            ->setHeaderText('Jmeno')->setSqlFieldIndex(24)->setSqlFieldName('Osoba.Jmeno')
            ->setAlign('left')->setLink('?menuid=pokladnikniha&action=zaznam')->setOrdering(true)
            ->setCode('col-jmeno');

        $this->addField()
            ->setHeaderText('datum_pohybu')->setSqlFieldIndex(1)->setSqlFieldName('PokladniKniha.DatumPohybu')
            ->setFormat('date')->setAlign('right')->setLink('?menuid=pokladnikniha&action=zaznam')->setOrdering(true)
            ->setCode('col-datum-pohybu');

        $this->defaultSort = 'col-datum-pohybu';

        $this->addField()
            ->setHeaderText('typ_polozky')->setSqlFieldIndex(26)->setSqlFieldName('TypPolozkyPokladniKnihy.Popis')->setAlign('left')
            ->setCode('col-typ-polozky')->setTranslationPrefix('db_typpolozkypokladniknihy_popis_', 7);

        $this->addField()
            ->setHeaderText('popis' . ($cisloBrowseru != 1 ? "_$cisloBrowseru" : ''))->setSqlFieldIndex(8)
            ->setSqlFieldName('PokladniKniha.Popis')->setAlign('left')
            ->setCode('col-popis' . ($cisloBrowseru != 1 ? "_$cisloBrowseru" : ''));

        if (RonPortalSettings::getIntValue('typrozdelenipoklk')) {
            $typRozdeleni = TypRozdeleni::getTypRozdeleni($database, RonPortalSettings::getIntValue('typrozdelenipoklk'));

            $this->addField(new RozdeleniBrowserField($this->database, 32, 35))->setHeaderText($typRozdeleni->popis)->setCode('col-rozdeleni');
        }
        if (RonPortalSettings::getIntValue('typrozdelenipoklk2')) {
            $typRozdeleni = TypRozdeleni::getTypRozdeleni($database, RonPortalSettings::getIntValue('typrozdelenipoklk2'));

            $this->addField(new RozdeleniBrowserField($this->database, 34, 36, 'col-rozdeleni2'))->setHeaderText($typRozdeleni->popis)->setCode('col-rozdeleni2')
                ->setSqlFieldName(RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_CO_ZOBRAZIT) == 1 ? 'R2.Kod' : 'R2.Popis');
        }
        if (RonPortalSettings::getIntValue('typrozdelenipoklk3')) {
            $typRozdeleni = TypRozdeleni::getTypRozdeleni($database, RonPortalSettings::getIntValue('typrozdelenipoklk3'));

            $this->addField(new RozdeleniBrowserField($this->database, 37, 38, 'col-rozdeleni3'))->setHeaderText($typRozdeleni->popis)->setCode('col-rozdeleni3')
                ->setSqlFieldName(RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_CO_ZOBRAZIT) == 1 ? 'R3.Kod' : 'R3.Popis');
        }
        if (PokladniKnihaController::zobrazitRozdeleni4($this->database) && RonPortalSettings::getIntValue('typrozdelenipoklk4')) {
            $typRozdeleni = TypRozdeleni::getTypRozdeleni($database, RonPortalSettings::getIntValue('typrozdelenipoklk4'));

            $this->addField(new RozdeleniBrowserField($this->database, 44, 45, 'col-rozdeleni4'))->setHeaderText($typRozdeleni->popis)->setCode('col-rozdeleni4')
                ->setSqlFieldName(RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_ZOBRAZENI_CO_ZOBRAZIT) == 1 ? 'R4.Kod' : 'R4.Popis');
        }

        if ($this->seznamPolozekPokladniKnihy->lzeZobrazitCastkuBezDPH()) {
            $this->addField()
                ->setHeaderText('castkaBezDPH')->setSqlFieldIndex(29)->setSqlFieldName('PokladniKniha.CastkaBezDPH')
                ->setAlign('right')->setOrdering(1)->setFormat('number2')->setCode('col-castka-bez-dph');
        }

        if ($this->seznamPolozekPokladniKnihy->lzeZobrazitCastkuDPH()) {
            $this->addField()
                ->setHeaderText('castkaDPH')->setSqlFieldIndex(30)->setSqlFieldName('PokladniKniha.CastkaDPH')
                ->setAlign('right')->setOrdering(1)->setFormat('number2')->setCode('col-castka-dph');
        }

        $this->addField()
            ->setHeaderText('castka')->setSqlFieldIndex(5)->setSqlFieldName('PokladniKniha.Castka')
            ->setAlign('right')->setOrdering(1)->setFormat('number2')->setCode('col-castka');

        $this->addField()
            ->setHeaderText('mena')->setSqlFieldIndex(6)->setSqlFieldName('PokladniKniha.Mena')->setAlign('left')->setCode('col-mena');

        if ($this->schvalovani) {
            $this->addField(new SchvaleniZaznamuPKEditField($this, 'pkniha_schval1', 11, 12,7,'1',
                ronText('PoklK_schvaleni_uroven1' . ($cisloBrowseru != 1 ? "_$cisloBrowseru" : ''))))
                ->setCode('col-schvaleni-uroven1');
            $this->addField(new SchvaleniZaznamuPKEditField($this, 'pkniha_schval2', 13, 14,7, '2',
                ronText('PoklK_schvaleni_uroven2' . ($cisloBrowseru != 1 ? "_$cisloBrowseru" : ''))))
                ->setCode('col-schvaleni-uroven2');
            $this->addField(new SchvaleniZaznamuPKEditField($this, 'pkniha_schval3', 15, 16, 7, '3',
                ronText('PoklK_schvaleni_uroven3' . ($cisloBrowseru != 1 ? "_$cisloBrowseru" : ''))))
                ->setCode('col-schvaleni-uroven3');
            $this->addField(new SchvaleniZaznamuPKEditField($this, 'pkniha_schval4', 39, 40, 7, '4',
                ronText('PoklK_schvaleni_uroven4' . ($cisloBrowseru != 1 ? "_$cisloBrowseru" : ''))))
                ->setCode('col-schvaleni-uroven4');
        } else {
            $this->addField(new SchvaleniZaznamuPKField($this ->database))->setCode('col-schvaleni');
        }

        $this->addField(new DokumentyPKField())->setCode('col-dokumenty');
        $this->addField(new ZruseniPKField($this->database, 'pkniha_vloz', 7))->setCode('col-zruseni');
    }

    public function setFilterForm()
    {
        $this->filterForm = new PokladniKnihaFilter($this->database,'pokladnikniha', $this->seznamPolozekPokladniKnihy);
    }

    function renderFilter()
    {
        $this->filterForm->render();

        // pokud muze editovat, tak zobrazi odkaz "novy zaznam"
        if (!$this->schvalovani) {
            if ($this->pravaVlozit->jsouNejakePrava())
               $this->renderNewRecord();

            echo "<div id='poklk_hromadneschvalovani'><input type='button' class='button' name='zobrazithromadne' value='" . ronText('PoklK_Schvalit_hromadne') . "'/></div>";
        }
    }

    function renderNewRecord()
    {
        echo "<div class='normaltext poklk_links'><a href='?menuid=pokladnikniha&browser=" . $this->seznamPolozekPokladniKnihy->cislo . "&formaction=new'>" . ronText('New_record') . "</a></div>\n";
    }

    function getSQLFilter()
    {
        $result = '(1 = 1 ) ';

        $params = $this->database->createParameters();

        if (!empty($this->seznamPolozekPokladniKnihy->seznamTypuSQL)) {
            $result .= ' AND ' . SQLUtils::getINCondition($this->database, 'PokladniKniha.TypPolozky', $this->seznamPolozekPokladniKnihy->seznamTypuSQL, $params);
        }
        $result .= ' AND ';

        $urovenSchvaleni = isset($_SESSION['urovenschvaleniPoklK']) ? $_SESSION['urovenschvaleniPoklK'] : 0;
        // uroven schvaleni (vse, ke schvaleni, ...)
        switch ($urovenSchvaleni) {
            case -2 : // bez schvaleni
                $result .= 'PokladniKniha.Schvaleni1 IS NULL  AND  PokladniKniha.Schvaleni2 IS NULL  AND  PokladniKniha.Schvaleni3 IS NULL  AND  PokladniKniha.Schvaleni4 IS NULL ';
                break;

            case 1 :
                $result .= 'PokladniKniha.Schvaleni1 IS NOT NULL  AND  PokladniKniha.Schvaleni2 IS NULL  AND  PokladniKniha.Schvaleni3 IS NULL AND PokladniKniha.Schvaleni4 IS NULL ';
                break;

            case 2 :
                $result .= 'PokladniKniha.Schvaleni2 IS NOT NULL  AND  PokladniKniha.Schvaleni3 IS NULL AND  PokladniKniha.Schvaleni4 IS NULL ';
                break;

            case 3 :
                $result .= 'PokladniKniha.Schvaleni3 IS NOT NULL AND PokladniKniha.Schvaleni4 IS NULL ';
                break;

            case 4 :
                $result .= 'PokladniKniha.Schvaleni4 IS NOT NULL ';
                break;

            case -1 : // ke schvaleni (dale se filtruje v enable line podle konkretnich prav)
                $result .= 'PokladniKniha.Schvaleni4 IS NULL ';
                // ty co jsou schvaleny posledni urovni se nezobrazuji (nejsou "ke schvaleni")

                if (!$this->pravaSchvalit4->jsouNejakePrava()) {
                    $result .= ' AND PokladniKniha.Schvaleni3 IS NULL ';
                    // pokud uz je schvalena urovni 3 a nemam pravo na 4, tak se nezobrazuje

                    if (!$this->pravaSchvalit3->jsouNejakePrava()) {
                        $result .= ' AND PokladniKniha.Schvaleni2 IS NULL ';
                        // pokud uz je schvalena urovni 2 a nemam pravo na 3, tak se nezobrazuje

                        if (!$this->pravaSchvalit2->jsouNejakePrava()) {
                            $result .= ' AND PokladniKniha.Schvaleni1 IS NULL ';
                            // pokud uz je schvalena urovni 1 a nemam pravo na 2 ani 3, tak se nezobrazuje
                        }
                    }
                }
                break;

            default : // vse
                $result .= ' 1 = 1 ';
        }

        $datumOd = DateUtils::today();
        $datumDo = $datumOd;

        if ($this->filterForm->filtrovatDatum) {
            $datumOd = $this->filterForm->datumOd;
            $datumDo = $this->filterForm->datumDo;

            $result .=
                ' AND PokladniKniha.DatumPohybu >= ' . $params->addParameter($this->database->timestampToParam($datumOd)) .
                ' AND PokladniKniha.DatumPohybu <= ' . $params->addParameter($this->database->timestampToParam($datumDo));
        }

        if (!$this->filterForm->vsechnyOsoby) {
            $result .=
                " AND PokladniKniha.RC = " . $params->addParameter($this->filterForm->vybranaOsobaRC) .
                " AND PokladniKniha.OsobniCislo = " . $params->addParameter($this->filterForm->vybranaOsobaOsobniCislo) .
                " AND PokladniKniha.DatumNastupu = " . $params->addParameter($this->database->timestampToParam($this->filterForm->vybranaOsobaDatumNastupu));
        }

        if ($this->filterForm->kodRozdeleni != '') {
            $result .= ' AND ' . RozdeleniUtils::sqlFilterRozdeleniPP($this->database, 'PokladniKniha', RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI_TYP), $this->filterForm->kodRozdeleni,
                   $datumOd, $datumDo, $this->filterForm->vcetnePodrizenychRozdeleni, $params);

        }

        if ($this->filterForm->kodRozdeleni2 != '') {
            $result .= ' AND ' . RozdeleniUtils::sqlFilterRozdeleniPP($this->database, 'PokladniKniha', RonPortalSettings::getIntValue(RonPortalSettings::ROZDELENI2_TYP), $this->filterForm->kodRozdeleni2,
                   $datumOd, $datumDo, $this->filterForm->vcetnePodrizenychRozdeleni2, $params);

        }

        if (isset($_SESSION['jenTypPolozkyPK']) && $_SESSION['jenTypPolozkyPK']) {
            $result .=
                " AND PokladniKniha.TypPolozky = " . $params->addParameter($_SESSION['typPolozkyPK']);
        }

        return new SqlStatement($result, $params);
    }

    public function nactiPrava($suffix)
    {
        $this->pravaProhlizet = Prava::vratPrava('pkniha_prohl', 'pkniha_prohl_' . $suffix);
        $this->pravaVlozit = Prava::vratPrava('pkniha_vloz', 'pkniha_vloz_' . $suffix);
        $this->pravaSchvalit1 = Prava::vratPrava('pkniha_schval1', 'pkniha_schval1_' . $suffix);
        $this->pravaSchvalit2 = Prava::vratPrava('pkniha_schval2', 'pkniha_schval2_' . $suffix);
        $this->pravaSchvalit3 = Prava::vratPrava('pkniha_schval3', 'pkniha_schval3_' . $suffix);
        $this->pravaSchvalit4 = Prava::vratPrava('pkniha_schval4', 'pkniha_schval4_' . $suffix);
    }
    
    /**
     * @param array $values
     * @param int $datumod
     * @param int $datumdo
     * @return bool
     * @throws Exception
     */
    function poklkEnableLine($values, $datumod, $datumdo)
    {
        $this ->jsonRowIndex = null;

        if (isset($_SESSION['urovenschvaleniPoklK']) && ($_SESSION['urovenschvaleniPoklK'] == -1)) {
            // filtr Ke schvaleni - zkontroluje, jesti to ceka na me

            if ($values[39]) return false;
            // jiz je schvaleno nejvyssi urovni = neceka na nikoho

            if ($values[15]) return $this->pravaSchvalit4->lzeZobrazitPP($values[27], $datumod, $datumdo);
            // je schvaleno 3, ceka na 4 => pokud mam pravo, ceka na me

            if ($values[13]) return $this->pravaSchvalit3->lzeZobrazitPP($values[27], $datumod, $datumdo);
            // je schvaleno 2, ceka na 3 => pokud mam pravo, ceka na me

            if ($values[11]) return $this->pravaSchvalit2->lzeZobrazitPP($values[27], $datumod, $datumdo);

            // je schvaleno 1, ceka na 2 => pokud mam pravo, ceka na me

            return $this->pravaSchvalit1->lzeZobrazitPP($values[27], $datumod, $datumdo);
            // neni schvaleno zadnou urovni => pokud mam pravo na 1, ceka na me
        }

        // Zobrazi zaznam, pokud mam kterekoliv pravo
        if (!$this->schvalovani) {
            if ($this->pravaVlozit->lzeZobrazitPP($values[27], $datumod, $datumdo)) return true;
            if ($this->pravaProhlizet->lzeZobrazitPP($values[27], $datumod, $datumdo)) return true;
        }

        if ($this->pravaSchvalit1->lzeZobrazitPP($values[27], $datumod, $datumdo)) return true;
        if ($this->pravaSchvalit2->lzeZobrazitPP($values[27], $datumod, $datumdo)) return true;
        if ($this->pravaSchvalit3->lzeZobrazitPP($values[27], $datumod, $datumdo)) return true;
        if ($this->pravaSchvalit4->lzeZobrazitPP($values[27], $datumod, $datumdo)) return true;

        return false;
    }


    /**
     * @param array $values
     * @return bool
     * @throws Exception
     */
    function enableLine($values)
    {
        $this->nactiPrava($values[7]);
        
        $datumod = $this ->database ->stringToTimestamp($values[1]);
        $datumdo = $this ->database ->stringToTimestamp($values[1]);

        $result = $this ->poklkEnableLine($values, $datumod, $datumdo);
        if (!$result) return false;

        if ($this ->schvalovani) {

            $this ->jsonRowIndex = count($this ->jsonObject['items']);

            $this ->hiddenFields .= "<input type='hidden' name='idPoklK_{$this ->jsonRowIndex}' value='{$values[0]}'/>";

            $this ->jsonObject['items'][] = [
                'id' => $values[0],
                'pravo_1' => $this ->pravaSchvalit1 ->lzeZobrazitPP($values[27], $datumod, $datumdo) ? 1 : 0,
                'pravo_2' => $this ->pravaSchvalit2 ->lzeZobrazitPP($values[27], $datumod, $datumdo) ? 1 : 0,
                'pravo_3' => $this ->pravaSchvalit3 ->lzeZobrazitPP($values[27], $datumod, $datumdo) ? 1 : 0,
                'pravo_4' => $this ->pravaSchvalit4 ->lzeZobrazitPP($values[27], $datumod, $datumdo) ? 1 : 0,
                'schvaleno_1' => $values[11] ? 1 : 0,
                'schvaleno_2' => $values[13] ? 1 : 0,
                'schvaleno_3' => $values[15] ? 1 : 0,
                'schvaleno_4' => $values[39] ? 1 : 0
            ];

        } else {
            $this ->jsonRowIndex = null;
        }

        return true;
    }

    /**
     * @param array  $values
     * @param string $className
     * @param int    $rowNumber
     * @return string   Vraci atributy pro <tr>
     */
    function getRowAttributes($values, $className, $rowNumber)
    {
        if ($values[39]) $className .= ' poklk_schvalena4';
        else if ($values[15]) $className .= ' poklk_schvalena3';
        else if ($values[13]) $className .= ' poklk_schvalena2';
        else if ($values[11]) $className .= ' poklk_schvalena1';

        $result = parent::getRowAttributes($values, $className, $rowNumber);

        $result .= ' id="poklk_row_' . $values[0] . '"';

        return $result;
    }

    function renderLine($values, $className, $rowNumber, $edit = false)
    {
        parent::renderLine($values, $className, $rowNumber, $edit);

        if ($values[17]) {
            // zobrazi poznamku k vraceni
            echo "<tr class='$className'>";
            echo    '<td></td><td><div class="browserpoznamka" style="text-align:right"><strong>' . ronText('PoklK_zaznam_vracen') . '</strong>: </div></td>'.
                    '<td colspan="' . ($this->columnCount - 2) . '"><div class="browserpoznamka">' . $values[17] . '</div></td>';
            echo '</tr>';
        }
    }

    function renderContent()
    {
        $this ->jsonObject = [];
        $this ->jsonObject['items'] = [];

        $this ->hiddenFields = '';

        if ($this ->schvalovani) {
            echo
                '<form method="post" name="hromadneSchvalovaniPoklK" action="?menuid=pokladnikniha&browser=' . $this->seznamPolozekPokladniKnihy->cislo . '">' .
                '<input type="hidden" name="hromadneSchvaleni" value="1"/>' .
                Csrf::csrfHiddenInput();
        }

        echo
            '<ul class="popupmenu"  id="schvaleniPoklKMenu" style="display:none">' .
            "<li data-ron-hodnotaschvaleni='1'>" . ronText('Oznacit_vse') . "</li>".
            "<li data-ron-hodnotaschvaleni='0'>" . ronText('Zrusit_oznaceni') . "</li>".
            "<li data-ron-hodnotaschvaleni='-1'>" . ronText('form.Original') . "</li>".
            '</ul>';


        parent::renderContent();

        if ($this ->hiddenFields) echo $this ->hiddenFields;
        echo '<input type="hidden" name="pocet" value="' . count($this ->jsonObject['items']) . '" />';

        if ($this->schvalovani) {
            echo
                "<div class='buttons'>" .
                "<input type='button' class='submitbutton' value='" . ronText('Provest') . "' id='provestHromadneSchvaleniPoklK' />" .
                '<input type="button" class="button" value="' . ronText('Cancel') . '" id="stornoHromadneSchvaleniPoklK" 
                    data-browserlink="?menuid=pokladnikniha&browser=' . $this->seznamPolozekPokladniKnihy->cislo . '" />' .
                '</div>'.
                "</form>\n";

            /** @noinspection BadExpressionStatementJS */
            echo
                '<script type="text/javascript">' .
                'ron.PokladniKniha.hromadneSchvaleni.init(' . json_encode($this ->jsonObject) . ')' .
                '</script>';
        } else {
            $pravo = Prava::vratHodnotuPrava('pkniha_vloz');
            if ($pravo) $this->renderNewRecord();
        }
    }

}