<?php
/**
 * UTF-8
 * Created by PhpStorm.
 * User: pavel.rudol
 * Date: 15.3.2016
 * Time: 13:21
 */
namespace RonPortal\PokladniKniha;

use Exception;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBException;
use Ron\DB\DBUtils;
use Ron\DB\Parameters;
use Ron\DB\SqlStatement;
use Ron\DB\SQLUtils;
use Ron\Controls\FormField;
use RonPortal\Controls\OsobaFormField;
use RonPortal\NeniPravoException;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\RonPortalDBRecord;
use RonPortal\Controls\RonPortalForm;
use RonPortal\RonPortalSettings;
use RonPortal\RonPortalXMLEvents;
use RonPortal\Controls\UserLoginFormField;
use Ron\StringUtils;
use RonPortal\RozdeleniOsob;
use RonPortal\TypRozdeleni;
use RonPortal\User;
use function Ron\ronText;


class CastkaFormField extends FormField
{
    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        echo '<span id="castka_pokladni_knihy">' . $value . '</span>';
    }
}

class PracovniPomerPoklKnihaFormField extends OsobaFormField
{
    private $idPP;

    public function __construct($caption, $prava, $datumPlatnostiPP, $idPP = null)
    {
        parent::__construct($prava, $datumPlatnostiPP);
        $this->setCaption($caption);

        $this->idPP = $idPP;
    }


    function showFieldContent($database, $value, $canEdit, $insert, $rowData)
    {
        if (!is_null($this->idPP))
            $value = $this->idPP;

        parent::showFieldContent($database, $value, $canEdit, $insert, $rowData);
    }

}

class PokladniKnihaForm extends RonPortalForm
{
    public $id;

    /** @var  SeznamPolozekPokladniKnihy */
    private $seznamPolozekPokladniKnihy;
    private $idPP;

    /**
     * PokladniKnihaForm constructor.
     * @param Database $database
     * @param string $formAction
     * @param SeznamPolozekPokladniKnihy $browser
     * @param int $idPP ID vybraneho PP (pri vytvareni se nastavi ze session)
     * @param bool $existujiRadky
     * @throws Exception
     */
    public function __construct($database, $formAction, $browser, $idPP, $existujiRadky = false)
    {
        parent::__construct($database, $formAction);

        $this->seznamPolozekPokladniKnihy = $browser;

        $this->code = 'pokl_kniha_' . $browser->cislo;

        $this ->htmlAction = '?menuid=pokladnikniha';
        $this ->stornoAction = '?menuid=pokladnikniha&browser='.$this->seznamPolozekPokladniKnihy->cislo;

        $this ->title = $formAction == 'new' ? ronText('New_record') //. ' ' . \Ron\ronText('pokladni_kniha' . ($browser->cislo != 1 ? "_$browser->cislo" : ''))
            : ronText('PoklK_editace_zaznamu' . ($browser->cislo != 1 ? "_$browser->cislo" : ''));

        $this ->sqlTableName = 'PokladniKniha';
        $this ->dbLogSrcDescription = StringUtils::fromUTF8('Pokladní kniha');

        $this->idPP = $idPP;

        $this->sqlSelectPart =
            'PokladniKniha.ID, PokladniKniha.DatumPohybu, PokladniKniha.RC, PokladniKniha.OsobniCislo, PokladniKniha.DatumNastupu, PokladniKniha.Castka, ' . //0 - 5
            'PokladniKniha.Mena, PokladniKniha.TypPolozky, PokladniKniha.Popis, PokladniKniha.Poznamka, ' . //6 - 9
            'PokladniKniha.IDDokumentu, PokladniKniha.Schvaleni1, PokladniKniha.Schvalil1, PokladniKniha.Schvaleni2, PokladniKniha.Schvalil2, ' . //10 - 14
            'PokladniKniha.Schvaleni3, PokladniKniha.Schvalil3, PokladniKniha.PoznamkaKVraceni, PokladniKniha.HistorieSchvalovani, ' . //15 - 18
            'PokladniKniha.ObdobiVyuctovani, PokladniKniha.Insert_UserName, PokladniKniha.Insert_Timestamp, PokladniKniha.Update_UserName, PokladniKniha.Update_Timestamp, ' .  //19 - 23
            'PokladniKniha.CastkaBezDPH, PokladniKniha.CastkaDPH, PokladniKniha.KodTypuRozdeleni, PokladniKniha.Rozdeleni, PokladniKniha.KodTypuRozdeleni2, PokladniKniha.Rozdeleni2, ' . //24 - 29
            'PokladniKniha.KodTypuRozdeleni3, PokladniKniha.Rozdeleni3, PokladniKniha.ZpusobPlatby, PokladniKniha.KodTypuRozdeleni4, PokladniKniha.Rozdeleni4'; //30 - 34

        $this->addField(FormField::FIELD_TYPE_ID)
            ->setFieldName('ID')->setPrimary(true)->setSqlFieldIndex(0);

        $this->addField(new PracovniPomerPoklKnihaFormField(ronText('Osoba'), 'pkniha_vloz', time(), $idPP))
            ->setCode('field-osoba');

        $this->addField(FormField::FIELD_TYPE_DATE)
            ->setCaption('datum_pohybu')->setRequired(true)->setSqlFieldIndex(1)->setSqlFieldName('DatumPohybu')
            ->setFieldName('DatumPohybu')->setMaxLength(10)->setDefaultValue(StringUtils::format('date',DateUtils::startOfTheDay(time())))
            ->setCode('field-datum');

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setCaption('typ_polozky')->setRequired(true)->setRowId('typ_polozky')->setSqlFieldName('TypPolozky')
            ->setFieldName('TypPolozky')->setSqlFieldIndex(7)
            ->setSelectSql('SELECT TypPolozkyPokladniKnihy.Kod, TypPolozkyPokladniKnihy.Popis FROM TypPolozkyPokladniKnihy ' .
                ' WHERE ('. SQLUtils::getINCondition($this->database, 'TypPolozkyPokladniKnihy.Kod', $this->seznamPolozekPokladniKnihy->seznamTypuSQL) . ' AND %FILTER% )' .
                ' ORDER BY %ORDER%',
                "1 = 1",
                'Popis')
            ->setCode('field-typ-polozky')->setTranslationPrefix('db_typpolozkypokladniknihy_popis_');

        if (RonPortalSettings::getIntValue('typrozdelenipoklk')) {
            $typRozdeleni = TypRozdeleni::getTypRozdeleni($this->database, RonPortalSettings::getIntValue('typrozdelenipoklk'));

            if ($typRozdeleni) {
                $this->pridejRozdeleni($typRozdeleni, 26, 27);
            }
        }

        if (RonPortalSettings::getIntValue('typrozdelenipoklk2')) {
            $typRozdeleni = TypRozdeleni::getTypRozdeleni($database, RonPortalSettings::getIntValue('typrozdelenipoklk2'));

            if ($typRozdeleni) {
                $this->pridejRozdeleni($typRozdeleni, 28, 29, '2');
            }
        }
        
        if (RonPortalSettings::getIntValue('typrozdelenipoklk3')) {
            $typRozdeleni = TypRozdeleni::getTypRozdeleni($database, RonPortalSettings::getIntValue('typrozdelenipoklk3'));

            if ($typRozdeleni) {
                $this->pridejRozdeleni($typRozdeleni, 30, 31, '3');
            }
        }
        
        if (PokladniKnihaController::zobrazitRozdeleni4($this->database) && RonPortalSettings::getIntValue('typrozdelenipoklk4')) {
            $typRozdeleni = TypRozdeleni::getTypRozdeleni($database, RonPortalSettings::getIntValue('typrozdelenipoklk4'));

            if ($typRozdeleni) {
                $this->pridejRozdeleni($typRozdeleni, 30, 31, '4');
            }
        }

        if ($this->seznamPolozekPokladniKnihy->lzeZobrazitCastkuBezDPH()) {
            $this->addField(FormField::FIELD_TYPE_FLOAT)
                ->setCaption('castkaBezDPH')->setRequired(false)->setRowId('row_castka_bez_dph')->setSqlFieldName('CastkaBezDPH')
                ->setFieldName('CastkaBezDPH')->setSqlFieldIndex(24)->setCode('field-castka-bez-dph')->setReadOnly($existujiRadky);
        }

        if ($this->seznamPolozekPokladniKnihy->lzeZobrazitCastkuDPH()) {
            $this->addField(FormField::FIELD_TYPE_FLOAT)
                ->setCaption('castkaDPH')->setRequired(false)->setRowId('row_castka_dph')->setSqlFieldName('CastkaDPH')
                ->setFieldName('CastkaDPH')->setSqlFieldIndex(25)->setCode('field-castka-dph')->setReadOnly($existujiRadky);
        }

        if ($this->seznamPolozekPokladniKnihy->lzeZobrazitCastku()) {
            $this->addField(FormField::FIELD_TYPE_FLOAT)
                ->setCaption('castka')->setRequired(true)->setSqlFieldName('Castka')->setSqlFieldIndex(5)
                ->setFieldName('castka')->setCode('field-castka')->setReadOnly($existujiRadky);
        }
        
        //needitovatelne pole s celkovu castkou - defaultne skryte, zobrazi se misto editovatelne castky (pokud nelze editovat)
        $this->addField(new CastkaFormField(FormField::FIELD_TYPE_FLOAT))
            ->setCaption('castka')->setRowId('poklkniha_castka_noedit')->setRequired(false)->setReadOnly()->setRowClass('hidden ' . $this->defaultRowClass ?: '')
            ->setCode('field-castka-text');

        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setCaption('mena')->setRequired(true)->setSqlFieldName('Mena')->setFieldName('Mena')->setSqlFieldIndex(6)
            ->setSelectSql('SELECT Mena.KodMeny_ISO, Mena.KodISO FROM Mena WHERE (Mena.Aktivni = 1 AND %FILTER% ) ORDER BY %ORDER% ',
                "1 = 1",
                'KodISO')
            ->setDefaultValue(RonPortalSettings::getValue('poklk_vychozi_mena'))
            ->setCode('field-mena');

        $params = $this->database->createParameters();
        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setCaption('PoklK_zpusobplatby')->setRequired(true)->setSqlFieldName('ZpusobPlatby')->setFieldName('ZpusobPlatby')->setSqlFieldIndex(32)
            ->setSelectSql(new SqlStatement('SELECT ZpusobPlatbyPK.Kod, ZpusobPlatbyPK.Popis FROM ZpusobPlatbyPK WHERE RonPortalBrowser = ' . $params->addParameter($browser->cislo) .
                ($this->seznamPolozekPokladniKnihy->cislo == 1 ? ' OR RonPortalBrowser IS NULL ' : '') . ' AND (%FILTER%) ' .
                ' ORDER BY %ORDER%', $params), '1 = 1', 'Popis')
            ->setCode('field-zpusobplatby')->setTranslationPrefix('db_zpusobplatbypk_popis_');

        $this->addField()
            ->setCaption('popis' . ($browser->cislo != 1 ? "_$browser->cislo" : ''))->setRequired(true)->setSqlFieldName('Popis')->setFieldName('Popis')->setSqlFieldIndex(8)
            ->setCode('field-popis' . ($browser->cislo != 1 ? "_$browser->cislo" : ''));

        $this ->addField(FormField::FIELD_TYPE_MEMO)
            ->setFieldName('Poznamka')->setSqlFieldName('Poznamka')->setSqlFieldIndex(9) ->setCaption('Poznamka')
            ->setCode('field-poznamka');

        $this->addField(FormField::FIELD_TYPE_ACTUAL_TIME)
            ->setSqlFieldName('Update_Timestamp')->setFieldName('Update_Timestamp');
        $this->addField(new UserLoginFormField(true, 'Update_UserName'));

        $this->addField( FormField::FIELD_TYPE_ACTUAL_TIME)
            ->setSqlFieldName('Insert_Timestamp')->setFieldName('Insert_Timestamp')->setReadOnly(true);
        $this->addField(new UserLoginFormField(false,'Insert_UserName'));
    }

    /**
     * @param TypRozdeleni $typRozdeleni
     * @param int $indexTypRozdeleni
     * @param int $indexRozdeleni
     * @param string $poradiRozdeleni
     * @throws DBException
     */
    private function pridejRozdeleni($typRozdeleni, $indexTypRozdeleni, $indexRozdeleni, $poradiRozdeleni = '')
    {
        $params = $this->database->createParameters();
        $sql = static::vratSQLrozdeleni($typRozdeleni, RonPortalSettings::getIntValue("rozdelenipoklk{$poradiRozdeleni}_pouzevlastni"), DateUtils::today(), DateUtils::today(), $params);

        $rozdeleni = null;
        // u noveho zaznamu doplni vychozi rozdeleni
        if (RonPortalSettings::getIntValue("rozdelenipoklk{$poradiRozdeleni}_vychozihodnota")) {
            $r = RozdeleniOsob::nactiRozdeleniOsoby($this ->database,
                $this->idPP, $typRozdeleni->kod,
                time(), time(), (RonPortalSettings::getIntValue("rozdelenipoklk{$poradiRozdeleni}_vychozihodnota") == 2) ? 'Hlavni = 1' : null );

            if ($r) {
                $rozdeleni = $r[0]->kod;
            }
        }

        $this->addField(FormField::FIELD_TYPE_HIDDEN)
            ->setFieldName("KodTypuRozdeleni{$poradiRozdeleni}")
            ->setSqlFieldIndex($indexTypRozdeleni)->setSqlFieldName("KodTypuRozdeleni{$poradiRozdeleni}")
            ->setDefaultValue($typRozdeleni->kod);
        $this->addField(FormField::FIELD_TYPE_SELECT)
            ->setFieldName("Rozdeleni{$poradiRozdeleni}")->setSqlFieldIndex($indexRozdeleni)
            ->setCode("field-rozdeleni{$poradiRozdeleni}")
            ->setCaption(StringUtils::htmlTextConvert($typRozdeleni ->popis))->setShowEmptyValue(true)
            ->setFieldId("poklk_rozdeleni{$poradiRozdeleni}")
            ->setFieldCssClass('form-control ronselect')
            ->setOtherHtmlParams('data-columns="text;value"')
            ->setSelectSql(new SqlStatement($sql, $params), '1 = 1', 'Popis')
            ->setDefaultValue($rozdeleni);

    }

    /**
     * @param TypRozdeleni $typRozdeleni
     * @param bool $pouzeVlastni
     * @param int $datumOd
     * @param int $datumDo
     * @param Parameters $params
     * @return string
     * @throws Exception
     */
    public static function vratSQLrozdeleni($typRozdeleni, $pouzeVlastni, $datumOd, $datumDo, $params)
    {
        if ($pouzeVlastni) {
            $sql =
                'SELECT DISTINCT Rozdeleni.Kod, Rozdeleni.Popis, Rozdeleni.cssClass ' .
                '  FROM RozdeleniOsob JOIN Rozdeleni ON RozdeleniOsob.KodTypuRozdeleni = Rozdeleni.KodTypuRozdeleni AND RozdeleniOsob.Rozdeleni = Rozdeleni.Kod '.
                '   JOIN PracovniPomer ON PracovniPomer.RC = RozdeleniOsob.RC AND PracovniPomer.OsobniCislo = RozdeleniOsob.OsobniCislo AND PracovniPomer.DatumNastupu = RozdeleniOsob.DatumNastupu ' .
                ' WHERE Rozdeleni.KodTypuRozdeleni = ' . $params ->addParameter($typRozdeleni->kod) .
                '   AND (Rozdeleni.DatumZruseni >= ' . $params ->addDateParameter($datumOd) . ' OR Rozdeleni.DatumZruseni IS NULL)'.
                '   AND (RozdeleniOsob.PlatnostDo >= ' . $params ->addDateParameter($datumDo) . ' OR RozdeleniOsob.PlatnostDo IS NULL)'.
                '   AND RozdeleniOsob.PlatnostOd <= ' . $params ->addDateParameter($datumOd) .
                '   AND PracovniPomer.RC = ' . $params->addParameter(User::getRC()) .
                '   AND PracovniPomer.DatumNastupu <= ' . $params->addDateParameter($datumDo) .
                '   AND (PracovniPomer.DatumUkonceni = ' . $params ->addDateParameter($datumOd) . ' OR PracovniPomer.DatumUkonceni IS NULL) ' .
                "   AND (cssClass IS NULL  OR  cssClass <> 'hidden')" .
                "   AND (%FILTER%) " .
                "ORDER BY %ORDER%";
        } else {
            $sql =
                'SELECT Kod, Popis, cssClass FROM Rozdeleni' .
                ' WHERE KodTypuRozdeleni = ' . $params->addParameter($typRozdeleni->kod) .
                '   AND ( ' .
                '   (DatumZruseni >= ' . $params->addDateParameter($datumDo) . ' OR DatumZruseni IS NULL)' .
                '   AND DatumVytvoreni <= ' . $params->addDateParameter($datumOd) .
                ')' .
                " AND (cssClass IS NULL  OR  cssClass <> 'hidden') ".
                " AND (%FILTER%) " .
                ' ORDER BY %ORDER%';
        }

        return $sql;
    }

    function createDBRecordFields($rec, $insert)
    {
        $pracPomer = PracovniPomer::nactiPP($this->database, (int)$_POST['osoba']);

        $rec->addField('RC','',true,15, $pracPomer->rc);
        $rec->addField('OsobniCislo','',true,15, $pracPomer->osobniCislo);
        $rec->addField('DatumNastupu','timestamp',true,0, $pracPomer->datumNastupu);

        parent::createDBRecordFields($rec, $insert);
    }


    /** Pokud je typ pohybu vydej, nastavi castku na zapornou.
     * @param RonPortalDBRecord $rec
     * @return RonPortalDBRecord
     * @throws DBException
     */
    private function nastavPodleTypuPohybu($rec)
    {
        $params = $this->database->createParameters();
        $jeVydej = (int)DBUtils::getSqlValue($this->database, "SELECT DruhPohybu FROM TypPolozkyPokladniKnihy WHERE Kod = " . $params->addParameter($rec['TypPolozky']), $params) == 2;
        //pro vydej bude castka zaporna
        if ($jeVydej) {
            $rec['Castka'] = - abs($rec['Castka']);
        }
        else {
            $rec['Castka'] = abs($rec['Castka']);
        }

        return $rec;
    }

    /** Overi, zda jsou zadana pole, ktera jsou u typu polozky povolena a pokud je povolena castka bez DPH a castka DPH secte je do pole castka.
     * @param RonPortalDBRecord $rec
     * @return RonPortalDBRecord
     * @throws Exception
     */
    private function overVypoctiPodleTypuPohybu($rec)
    {
        $typPolozky = $this->seznamPolozekPokladniKnihy->vratTypPolozky($rec['TypPolozky']);
        
        if ($typPolozky->lzeZobrazitCastkuBezDPH() && ($rec['CastkaBezDPH'] == '')) {
            throw new Exception(ronText('Nezadana_castka_bez_dph'));
        }

        if ($typPolozky->lzeZobrazitCastkuDPH() && ($rec['CastkaDPH'] == '')) { 
            throw new Exception(ronText('Nezadana_castka_dph'));
        }

        //zadava se castka bez dph a celkova castka, castka dph se dopocte
        if ($typPolozky->lzeZobrazitCastkuBezDPH() && !$typPolozky->lzeZobrazitCastkuDPH()) {
            if ($rec['CastkaBezDPH'] > $rec['Castka']) {
                throw new Exception(ronText('Castka_bez_dph_vyssi'));
            }
            $rec['CastkaDPH'] = $rec['Castka'] - $rec['CastkaBezDPH'];
        }

        //zadava se castka dph a celkova castka, castka bez dph se dopocte
        if (!$typPolozky->lzeZobrazitCastkuBezDPH() && $typPolozky->lzeZobrazitCastkuDPH()) {
            if ($rec['CastkaDPH'] > $rec['Castka']) {
                throw new Exception(ronText('Castka_dph_vyssi'));
            }
            $rec['CastkaBezDPH'] = $rec['Castka'] - $rec['CastkaDPH'];
        }

        //zadava se castka bez dph a castka dph, celkova castka se dopocte
        if ($typPolozky->lzeZobrazitCastkuBezDPH() && $typPolozky->lzeZobrazitCastkuDPH()) {
            $rec['Castka'] = $rec['CastkaBezDPH'] + $rec['CastkaDPH'];
        }
        
        return $rec;
    }

    /**
     * @param RonPortalDBRecord $rec
     */
    private function nastavTypRozdeleni($rec)
    {
        if (RonPortalSettings::getIntValue('typrozdelenipoklk')) {
            if ($_POST['Rozdeleni'] != '') {
                $rec->findFieldByName('KodTypuRozdeleni')->setNewValue(RonPortalSettings::getIntValue('typrozdelenipoklk'));
                $rec->findFieldByName('Rozdeleni')->setNewValue($_POST['Rozdeleni']);
            } else {
                $rec->findFieldByName('KodTypuRozdeleni')->setNewValue(null);
                $rec->findFieldByName('Rozdeleni')->setNewValue(null);
            }
        }
        if (RonPortalSettings::getIntValue('typrozdelenipoklk2')) {
            if ($_POST['Rozdeleni2'] != '') {
                $rec->findFieldByName('KodTypuRozdeleni2')->setNewValue(RonPortalSettings::getIntValue('typrozdelenipoklk2'));
                $rec->findFieldByName('Rozdeleni2')->setNewValue($_POST['Rozdeleni2']);
            } else {
                $rec->findFieldByName('KodTypuRozdeleni2')->setNewValue(null);
                $rec->findFieldByName('Rozdeleni2')->setNewValue(null);
            }
        }
        if (RonPortalSettings::getIntValue('typrozdelenipoklk3')) {
            if ($_POST['Rozdeleni3'] != '') {
                $rec->findFieldByName('KodTypuRozdeleni3')->setNewValue(RonPortalSettings::getIntValue('typrozdelenipoklk3'));
                $rec->findFieldByName('Rozdeleni3')->setNewValue($_POST['Rozdeleni3']);
            } else {
                $rec->findFieldByName('KodTypuRozdeleni3')->setNewValue(null);
                $rec->findFieldByName('Rozdeleni3')->setNewValue(null);
            }
        }
        if (PokladniKnihaController::zobrazitRozdeleni4($this->database) && RonPortalSettings::getIntValue('typrozdelenipoklk4')) {
            if ($_POST['Rozdeleni4'] != '') {
                $rec->findFieldByName('KodTypuRozdeleni4')->setNewValue(RonPortalSettings::getIntValue('typrozdelenipoklk4'));
                $rec->findFieldByName('Rozdeleni4')->setNewValue($_POST['Rozdeleni4']);
            } else {
                $rec->findFieldByName('KodTypuRozdeleni4')->setNewValue(null);
                $rec->findFieldByName('Rozdeleni4')->setNewValue(null);
            }
        }
    }

    /**
     * @param RonPortalDBRecord $rec
     */
    private function zkontrolujPravoNaTyp($rec)
    {
        $prava = Prava::vratPrava('pkniha_vloz', 'pkniha_vloz_' . $rec->findFieldByName('TypPolozky')->value);

        if (!$prava->jsouNejakePrava())
            throw new NeniPravoException();
    }

    /**
     * @param RonPortalDBRecord $rec
     * @return bool
     * @throws Exception
     * @throws DBException
     */
    public function beforeInsertRecord($rec)
    {
        $this->zkontrolujPravoNaTyp($rec);

        $rec = $this->overVypoctiPodleTypuPohybu($rec);
        
        $rec = $this->nastavPodleTypuPohybu($rec);

        $this->nastavTypRozdeleni($rec);

        return $rec->executeXmlEvents(RonPortalXMLEvents::POKLK_BEFORE_POST,'insert',null,false);
    }

    /**
     * @param RonPortalDBRecord $rec
     * @return bool
     * @throws Exception
     * @throws DBException
     */
    public function beforeUpdateRecord($rec)
    {
        $this->zkontrolujPravoNaTyp($rec);

        $rec = $this->overVypoctiPodleTypuPohybu($rec);
        
        $rec = $this->nastavPodleTypuPohybu($rec);

        $this->nastavTypRozdeleni($rec);

        return $rec->executeXmlEvents(RonPortalXMLEvents::POKLK_BEFORE_POST,'update',null,false);
    }

    /**
     * @param RonPortalDBRecord $rec
     * @return bool
     * @throws Exception
     */
    public function afterInsertRecord($rec)
    {
        $this->id = $rec['ID'];
        
        return $rec->executeXmlEvents(RonPortalXMLEvents::POKLK_AFTER_POST,'insert',null,false);
    }

    /**
     * @param RonPortalDBRecord $rec
     * @return bool
     * @throws Exception
     */
    public function afterUpdateRecord($rec)
    {
        $this->id = $rec['ID'];

        return $rec->executeXmlEvents(RonPortalXMLEvents::POKLK_AFTER_POST,'update',null,false);
    }

    public function renderCommonFields()
    {
        parent::renderCommonFields();
        
        echo '<input type="hidden" name="browser" value="' . $this->seznamPolozekPokladniKnihy->cislo . '" />';
        echo "<input type='hidden' name='typy' id='seznamtypu' value='" . json_encode($this->seznamPolozekPokladniKnihy->jsonObject) . "' />";
    }

}