<?php
/**
 * Seznam dlouhodobych skoleni s moznosti editace terminu a zaznamenani ucasti
 */

namespace RonPortal\Skoleni;

use Ron\Controls\BrowserField;
use Ron\Controls\ComboBox;
use Ron\Controls\DateInput;
use Ron\Controls\LinksList;
use Ron\DateUtils;
use Ron\DB\Database;
use Ron\DB\DBUtils;
use Ron\DB\SqlStatement;
use Ron\GuiUtils;
use Ron\StringUtils;
use Ron\TextManager;
use RonPortal\Controls\RonPortalBrowser;


/**
 * Class UcastNaSkoleniFiltr
 * @package Ron\RonPortal
 */
class UcastNaSkoleniFiltr extends SkoleniFilterForm
{
    public $typObdobi = 1;
    public $vybraneObdobiSkoleni = 1; // aktualni mesic
    public $pocetDnu = 60;

    public $jenNevyplnene = false;



//    public $timestamp;
//    public $rozsahZobrazeni;

    /**
     * UcastNaSkoleni constructor.
     * @param Database $database
     * @internal param string $action
     */
    public function __construct(Database $database)
    {
        parent::__construct($database, ['menuid' => 'skoleni', 'action' => 'skoleniucast'], true);
    }


    public function getDatumSkoleniOd()
    {
        if ($this ->typObdobi == 2) {
            return $this ->datumOd;
        } else {
            switch ($this ->vybraneObdobiSkoleni) {
                case 3 :
                    return DateUtils::firstDayOfQuarter(time());

                case 6 :
                    return DateUtils::firstDayOfHalfYear(time());

                default :
                    return DateUtils::firstDayOfMonth(time());
            }
        }
    }


    /**
     * Pozor! Vraci pocatek nasledujiciho dne po obdobi!
     * @return int
     */
    public function getDatumSkoleniDo()
    {
        if ($this ->typObdobi == 2) {
            return DateUtils::incDay($this ->datumOd, $this ->pocetDnu);
        } else {
            switch ($this ->vybraneObdobiSkoleni) {
                case 3 :
                    return DateUtils::incMonth(DateUtils::firstDayOfQuarter(time()), 3);

                case 6 :
                    return DateUtils::incMonth(DateUtils::firstDayOfHalfYear(time()), 6);

                default :
                    return DateUtils::lastDayOfMonth(time());
            }
        }
    }



    function renderVyberSkoleni()
    {
        $this ->renderFilterCisloSkoleni(
            'SELECT DISTINCT CisloSkoleni, Popis FROM Skoleni WHERE DlouhodobyKurz = 1', null,
            TextManager::_('nevybrano'));
    }


    function renderNevyplneneFormulare(){
        echo '<div class="filterrow">';

        echo '<span class="filtercaption"></span>';

        LinksList::renderLinks([
            0 => TextManager::_('vsechny_formulare_skoleni'),
            1 => TextManager::_('jen_nevyplnene_formulare_skoleni')],
            $this ->jenNevyplnene, 'jennevyplnene', '', false,
            true); // hiddenfield

        echo '</div>';
    }


    function renderAktualniObdobi()
    {
        echo "<div class='filterrow' id='filterrow_obdobi_skoleni'>";

        $checkedStr = ($this ->typObdobi === 2) ? '' : 'checked="checked"';
        echo
            "<span class='filtercaption'>" .
            "<label><input type='radio' id='ucast_typ_obdobi_1' class='clickAutoSubmit' name='typobdobi' value='1' $checkedStr>" . ' ' . \Ron\ronText('zobrazit_terminy_za') . '</label>' .
            '</span>';

        $combo = new ComboBox('obdobiskoleni', [
            1 => TextManager::_('aktualni_mesic'),
            3 => TextManager::_('aktualni_ctvrtleti'),
            6 => TextManager::_('aktualni_pololeti')
        ]);
        $combo ->className = 'adsfilterforminput_full';
        $combo ->htmlParams = ['data-ron-autosubmit' => '0', 'id' => 'ucast_obdobi_skoleni']; // neodesilat - jeste se zaskrtava radio

        $combo ->selectedOption = $combo ->findOption($this ->vybraneObdobiSkoleni);
        $combo ->render();

        echo '</div>';
    }


    function renderOmezeniTerminu()
    {
        echo '<div class="filterrow">';

        $checkedStr = ($this ->typObdobi === 2) ? 'checked="checked"' : '';
        echo
            "<span class='filtercaption'>" .
            "<label><input type='radio' id='ucast_typ_obdobi_2' class='clickAutoSubmit' name='typobdobi' value='2' $checkedStr>" . ' ' . \Ron\ronText('zobrazit_terminy_za') . '</label>' .
            '</span>';

        echo '<div>'; // class="popis_filtru_terminu">';
        $combo = new ComboBox('pocetdnu', [
            1 => TextManager::_('1_den'),
            10 => TextManager::_('10_dni'),
            20 => TextManager::_('20_dni'),
            31 => TextManager::_('31_dni'),
            60 => TextManager::_('60_dni')
        ]);

        $combo->className = 'adsfilterforminput_kod';
        $combo ->htmlParams = ['data-ron-autosubmit' => '0', 'id' => 'ucast_pocetdnu_skoleni']; // neodesilat - jeste se zaskrtava radio
        //$combo->htmlParams = 'onChange="submit();"';
        $combo->selectedOption = $combo->findOption($this ->pocetDnu);
        $combo->render();
//        echo '<span>' . \Ron\ronText('zvolena_doba') . ': ' . $this->rozsahZobrazeni . '</span>';
//        echo '</div>';

//        echo '<div class="popis_filtru_terminu">';
//        echo '&nbsp;&nbsp;' . \Ron\ronText('Od') . '&nbsp;&nbsp;&nbsp;';
        echo ' ' . TextManager::_('Od') . ' ';
        DateInput::renderDateSelect('datumod', date('j.n.Y', $this->datumOd), true);
        //echo '<span>' . TextManager::_('pouze_dnesni_den') . '</span>';
        echo '</div>';

        echo '</div>';
    }



    function renderDetailDlouhodobehoKurzu()
    {
        $params = $this ->database ->createParameters();
        $sql =
            'SELECT Skolitel, DatumPocatkuSkoleni, CasPocatkuSkoleni, DatumUkonceniSkoleni, CasUkonceniSkoleni' .        //0-4
            ' FROM Skoleni' .
            ' WHERE Skoleni.CisloSkoleni = ' . $params ->addParameter($this ->cisloSkoleni);
        $cursor = $this->database->getSqlResult($sql, $params);
        $row = $cursor->fetch();
        $cursor ->close();

        /**
         * Popis zacatku - konce casem od - do skoleni napr.        1.7.2017 - 30.9.2017     8:00 - 16:00
         **/
        echo '<div class="filterrow">';
        echo  '<span class="filtercaption margin-top-2">' . \Ron\ronText('zacatek_konec_skoleni').  '</span>';
        echo  '<div>';

        echo
            '<strong>' .
                StringUtils::format('date', $this->database->stringToTimestamp($row[1])) .
                ($row[3]
                    ? " - " .
                    StringUtils::format('date', $this->database->stringToTimestamp($row[3]))
                    : ''
                ) .
            '</strong>';

        if ($row[2]  ||  $row[4]) {
            echo
                ' <span class="margin-left-2">' .
                StringUtils::format('time', $this->database->stringToTimestamp($row[2])) .
                ' - ' .
                StringUtils::format('time', $this->database->stringToTimestamp($row[4])) .
                '</span>';
        }
        echo '</div>';
        echo '</div>';

        // jmeno skolitele
        if ($row[0]) {
            echo '<div class="filterrow" id="filter_row_jmeno_skolitele">';
            echo '<span class="filtercaption">' . \Ron\ronText('skolitel') . '</span>';
            echo '<strong><span class="filtercaption">' . StringUtils::htmlTextConvert($row[0]) . '</span></strong>';
            echo '</div>';
        }
    }


    private function loadNevyplneneFormulare()
    {
        if (isset($_GET['jennevyplnene'])) {
            $this ->jenNevyplnene = (int)$_GET['jennevyplnene'] ? 1 : 0;
            $_SESSION['jen_nevyplnene_formulare_skoleni'] = $this ->jenNevyplnene;
        } else {
            $this ->jenNevyplnene = $_SESSION['jen_nevyplnene_formulare_skoleni'] ?? 0;
        }
    }


    function loadVolbaObdobi()
    {
        // typ obdobi
        if (isset($_GET['typobdobi'])) {
            $this ->typObdobi = ((int)$_GET['typobdobi'] == 2) ? 2 : 1;
            $_SESSION['filtrObdobiUcastiNaSkoleni'] = $this ->typObdobi;
        } else {
            $this ->typObdobi = (($_SESSION['filtrObdobiUcastiNaSkoleni'] ?? 1) == 2) ? 2 : 1;
        }

        // obdobi
        if (isset($_GET['obdobiskoleni'])) {
            $this ->vybraneObdobiSkoleni = (int)$_GET['obdobiskoleni'];
            $_SESSION['vybraneObdobiUcastiNaSkoleni'] = $this ->vybraneObdobiSkoleni;
        } else {
            $this ->vybraneObdobiSkoleni = $_SESSION['vybraneObdobiUcastiNaSkoleni'] ?? 1;
        }

        switch ($this ->vybraneObdobiSkoleni) {
            case 1: // mesic
            case 3: // ctvrtleti
            case 6: // pololeti
                break;

            default :
                $this ->vybraneObdobiSkoleni = 1;
        }

        // pocet dnu
        if (isset($_GET['pocetdnu'])) {
            $this ->pocetDnu = (int)$_GET['pocetdnu'];
            $_SESSION['pocetDnuUcastiNaSkoleni'] = $this ->pocetDnu;
        } else {
            $this ->pocetDnu = $_SESSION['pocetDnuUcastiNaSkoleni'] ?? 60;
        }

        switch ($this ->pocetDnu) {
            case 1 :
            case 10 :
            case 20 :
            case 31 :
            case 60 :
                break;

            default :
                $this ->pocetDnu = 60;
        }

        $loadSessionDate = true;
        if (isset($_GET['datumod'])  &&  $_GET['datumod']) {
            try {
                $this->datumOd = DateUtils::dateStringToTimestamp($_GET['datumod']);
                $loadSessionDate = false;
                $_SESSION['filtr_datum_skoleni_od'] = $this ->datumOd;
            } catch (\Exception $e) {
                // ignoruje
            }
        }
        if ($loadSessionDate) {
            $this ->datumOd = $_SESSION['filtr_datum_skoleni_od'] ?? DateUtils::today();
        }
    }


    function loadData()
    {
        $this->loadCisloSkoleni('kodDlouhodobehoKurzu');

        $this->loadVolbaObdobi();
        //$this->loadAktualniObdobi();
        //$this->loadDatumFiltru();
        //$this->loadRozsahPocetDnu();
        //$this->loadZobrazenaDoba();
        $this->loadNevyplneneFormulare(); // vola vsechny/jen nevyplnene
        //$this->loadNastaveniZobrazeniRozsahu();
    }


    function renderFilters()
    {
        $this->renderVyberSkoleni();
        $this->renderAktualniObdobi();
        $this->renderOmezeniTerminu();
        $this->renderNevyplneneFormulare();
        $this->renderDetailDlouhodobehoKurzu();

        echo
            '<div class="filterrow">' .
            '<div class="col margin-top-2"><a href="?menuid=skoleni&action=skolenikurzy">' . TextManager::_('Zpet_na_seznam') . '</a></div>' .
            '</div>';
//        $this->renderDokumentyTerminu();
    }
}



class EditaceTerminuSkoleniField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $datum = $this ->database ->stringToTimestamp($rowData['col-datum']);

        return
            '<i class="inline-glyph glyph-button fa glyph-edit open-dialog"' .
            ' data-dialog="#termin_skoleni_dialog"' .
            ' data-ronform-record="' . StringUtils::htmlTextConvert($rowData['col-cisloskoleni']) . '"' .
            //' data-ronform-url="?menuid=skolenijs&action=terminskoleni&cisloskoleni=' . StringUtils::htmlTextConvert($rowData[0]) . '&datum=' . StringUtils::format('date', $datum)  . '"' .
            ' data-ronform-url="?menuid=skolenijs&action=terminskoleni&datum=' . StringUtils::format('date', $datum)  . '&cisloskoleni="' .
            ' title="' . TextManager::_('Editovat_zaznam') . '">' .
            '</i>';
    }
}


class EditaceUcastnikuSkoleniField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $datum = $this ->database ->stringToTimestamp($rowData['col-datum']);

        return
            '<span class="js open-dialog"' .
            ' data-dialog="#ucast_na_skoleni_dialog"' .
            ' data-ronform-record="' . StringUtils::htmlTextConvert($rowData['col-cisloskoleni']) . '"' .
            //' data-ronform-url="?menuid=skolenijs&action=terminskoleni&cisloskoleni=' . StringUtils::htmlTextConvert($rowData[0]) . '&datum=' . StringUtils::format('date', $datum)  . '"' .
            ' data-ronform-url="?menuid=skolenijs&action=ucastnaskoleni&datum=' . StringUtils::format('date', $datum)  . '&cisloskoleni="' .
            '>' . TextManager::_('zaznamenat_ucast') .
            '</span>';
    }
}


class UcastNaSkoleniEditovanaField extends BrowserField
{
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        return TextManager::_(($rowData['col-ucast']  ||   $rowData['col-neucast'])
            ? 'Str_Yes'
            : 'Str_No');
    }
}



class UcastNaSkoleniPercentField extends BrowserField
{
    private $pocetPrihlasenych;

    /**
     * UcastNaSkoleniPercentField constructor.
     * @param Database $database
     * @param int $pocetPrihlasenych
     */
    public function __construct($database, $pocetPrihlasenych)
    {
        parent::__construct($database);
        $this->pocetPrihlasenych = $pocetPrihlasenych;
    }


    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $result = parent::getViewText($value, $rowNumber, $recordId, $rowData);

        if ($this ->pocetPrihlasenych) {
            $result .= ' (' . (int)($value / $this ->pocetPrihlasenych * 100) . '%)';
        }

        return $result;
    }
}



/**
 * Class SkoleniUcastBrowser
 * @package Ron\RonPortal
 */
class UcastNaSkoleniBrowser extends RonPortalBrowser
{
    private $pocetPrihlasenych = null;

    /** @var bool */
    private $jePravoEditovatUcast;

    /**
     * SkoleniUcastBrowser constructor.
     * @param Database $database
     */
    public function __construct(Database $database)
    {
        parent::__construct($database, 'skoleni');

        //$this->prava = Prava::vratPrava('skoleni_prihlasit');

        $this->code = 'browserskoleniucast';
        $this->cssClass = "browser";
        $this->tableId = 'browserskoleniucast';
        $this->browserUrl = "?menuid=skoleni&action=skoleniucast";
        $this->title = TextManager::_('ucast_na_skoleni');

        $this->sqlFromPart = 'TerminSkoleni';

        $this->filterForm = new UcastNaSkoleniFiltr($database);
        $this->filterForm->loadData();

        $this ->jePravoEditovatUcast = SkoleniJsController::jePravoNaZapisUcasti($this ->filterForm ->cisloSkoleni);

        $this ->pocetPrihlasenych = $this ->getPocetPrihlasenych();

        $this->defaultSort = '-col-datum';

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('CisloSkoleni')
            ->setCode('col-cisloskoleni');

        $this->addField()
            ->setSqlFieldName('Datum') ->setHeaderText('Datum') ->setFormat('finedate') ->setOrdering(true)
            ->setCode('col-datum')
            ->setAlign('center');

        $this->addField()
            ->setSqlFieldName('Cas') ->setHeaderText('cas') ->setFormat('time') ->setAlign('center')
            ->setCode('col-cas');

        //$this->addField(new UcastZaznamenanaEditovan($database, null));

        //$this->addField(new ProcentualniUcastNaSkoleniField($database, null));

        //$this->addField(new ProcentualniNeucastNaSkoleniField($database, null));

        $this ->addField(new UcastNaSkoleniEditovanaField())
            ->setCode('col-formular-vyplnen')
            ->setAlign('center')
            ->setHeaderText('ucast_zaznamenana');

        $this ->addField(new UcastNaSkoleniPercentField($this ->database, $this ->pocetPrihlasenych))
            ->setSqlFieldName(
                '(SELECT COUNT (*) FROM UcastNaTerminuSkoleni' .
                ' WHERE UcastNaTerminuSkoleni.CisloSkoleni = TerminSkoleni.CisloSkoleni' .
                ' AND UcastNaTerminuSkoleni.Datum = TerminSkoleni.Datum' .
                ' AND Pritomen = 1) AS PocetPritomnych')
            ->setCode('col-ucast')
            ->setHeaderText('Ucast')
            ->setAlign('center');

        $this ->addField(new UcastNaSkoleniPercentField($this ->database, $this ->pocetPrihlasenych))
            ->setSqlFieldName(
                '(SELECT COUNT (*) FROM UcastNaTerminuSkoleni' .
                ' WHERE UcastNaTerminuSkoleni.CisloSkoleni = TerminSkoleni.CisloSkoleni' .
                ' AND UcastNaTerminuSkoleni.Datum = TerminSkoleni.Datum' .
                ' AND Pritomen = 0  AND  DuvodNepritomnosti = 1) AS PocetOmluvenych')
            ->setCode('col-neucast')
            ->setHeaderText('neucast')
            ->setAlign('center');

        //$this->addField(new UcastNaSkoleniField($database, null));

        if ($this ->jePravoEditovatUcast) {
            $this->addField(new EditaceUcastnikuSkoleniField($database))
                ->setCode('col-ucastnici');

            $this->addField(new EditaceTerminuSkoleniField($database))
                ->setCode('col-edit');
        }

        $this ->addField(BrowserField::FIELD_TYPE_COMMENT)
            ->setSqlFieldName('{fn_castwidememo(Poznamka)}') ->setHeaderColSpan(1)
            ->setClassName('poznamka');

    }

    function setFilterForm()
    {
        //parent::setFilterForm();      Nedela nic, byl nastaven v konstruktoru
    }


    private function getPocetPrihlasenych()
    {
        $params = $this ->database ->createParameters();
        return DBUtils::getSqlValue($this ->database,
            'SELECT COUNT(*)' .
            ' FROM GenerovaneSkoleni GS' .
            ' WHERE GS.CisloSkoleni = ' . $params->addParameter($this ->filterForm ->cisloSkoleni) .  // TODO: predelat
            '  AND (GS.Potvrzeni<5)',
            $params);
    }


    function getSQLFilter()
    {
        $params = $this ->database ->createParameters();
        $result =
            'TerminSkoleni.CisloSkoleni = ' . $params->addParameter($this ->filterForm ->cisloSkoleni) .
            ' AND TerminSkoleni.Datum >= ' . $params ->addDateParameter($this ->filterForm ->getDatumSkoleniOd()) .
            ' AND TerminSkoleni.Datum < ' . $params ->addDateParameter($this ->filterForm ->getDatumSkoleniDo());

        if ($this ->filterForm ->jenNevyplnene) {
            $result .=
                ' AND NOT EXISTS (' .
                '  SELECT * FROM UcastNaTerminuSkoleni' .
                '  WHERE UcastNaTerminuSkoleni.Datum = TerminSkoleni.Datum' .
                '  AND UcastNaTerminuSkoleni.CisloSkoleni = TerminSkoleni.CisloSkoleni' .
                '  AND (Pritomen = 1  OR  DuvodNepritomnosti = 1)' .
                ' )';
        }

        return new SqlStatement($result, $params);
    }

    function renderFilter()
    {
        parent::renderFilter();

        if ($this ->filterForm ->cisloSkoleni  &&  $this ->jePravoEditovatUcast) {
            echo
                '<div class="col margin-top-2">' .
                // odkaz pro zobrazeni dialogu
                '<span class="js open-dialog"' .
                ' data-dialog="#termin_skoleni_dialog">' .
                TextManager::_('vytvorit_termin') .
                '</span>' .
                '</div>';
        }
    }


    public function render()
    {
        parent::render();

        if ($this ->jePravoEditovatUcast) {
            // dialog pro editaci terminu
            $f = new TerminSkoleniFormular($this->database, $this->filterForm->cisloSkoleni);
            $f->render();

            // dialog pro editaci ucasti
            $f = new UcastNaSkoleniFormular($this->database, $this->filterForm->cisloSkoleni);
            $f->render();
        }
    }
}
