<?php
/**
 * stanislav.elbl, 14.01.2016
 */

namespace RonPortal\Uchazeci;

use Ron\Controls\BrowserField;
use Ron\DB\Database;
use Ron\DB\DBUtils;
use Ron\DB\Parameters;
use Ron\DB\SqlStatement;
use Ron\StringUtils;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\RonPortalSettings;



/**
 * Class RozdeleniUchazeceField
 * @package Ron\Personalistika
 */
class RozdeleniUchazeceField extends BrowserField
{
    private $kodTypuRozdeleni;

    /**
     * RozdeleniUchazeceField constructor.
     * @param Database $database
     * @param int $kodTypuRozdeleni
     */
    public function __construct($database, $kodTypuRozdeleni)
    {
        parent::__construct($database);

        $this ->kodTypuRozdeleni = $kodTypuRozdeleni;

        $params = $this->database->createParameters();
        $popis = DBUtils::getSqlValue($this->database,
            'SELECT Popis FROM TypRozdeleni' .
            ' WHERE Kod = '. $params->addParameter($this ->kodTypuRozdeleni), $params);

        $this->header = StringUtils::htmlTextConvert($popis);
    }

    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return string
     * @throws \Exception
     */
    public function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        $params = $this->database->createParameters();

        $sql =
            'SELECT Rozdeleni.Kod, Rozdeleni.Popis ' .
            ' FROM P_Uchazec, Rozdeleni, RozdeleniUchazece ' .
            ' WHERE P_Uchazec.RC = RozdeleniUchazece.RC ' .
            ' AND RozdeleniUchazece.Rozdeleni = Rozdeleni.Kod ' .
            ' AND RozdeleniUchazece.KodTypuRozdeleni = Rozdeleni.KodTypuRozdeleni ' .
            ' AND RozdeleniUchazece.KodTypuRozdeleni = ' . $params->addParameter($this ->kodTypuRozdeleni) .
            ' AND P_Uchazec.RC = ' . $params->addParameter($rowData['col-rc']) . ' '.
            ' ORDER BY Rozdeleni.Popis';

        $cursor = $this->database->getSqlResult($sql, $params);

        $result = '';

        while ($row = $cursor->fetch()) {
            if ($result) {
                $result .= ', ';
            }
            $result .= "<span title='" . $row[0]  . "'>" . $row[1] . '</span>';
        }
        $cursor ->close();

        return $result;
    }
}



/**
 * Class UchazeciBrowser
 * @package Ron\Personalistika
 */
class UchazeciBrowser extends RonPortalBrowser
{
    public $typRozdeleniFiltr2;
    public $typRozdeleniFiltr3;

    /**
     * UchazeciBrowser constructor.
     * @param Database $database
     * @throws \Exception
     */
    public function __construct($database)
    {
        parent::__construct($database, '');

        $this->typRozdeleniFiltr2 = RonPortalSettings::getIntValue('uchazecityprozdeleniprofiltr');
        $this->typRozdeleniFiltr3 = RonPortalSettings::getIntValue('uchazecityprozdeleniprofiltr2');

        $this ->sqlFromPart =
            'P_Uchazec' .
            ' LEFT OUTER JOIN ZdrojVznikuPP ON P_Uchazec.ZdrojVzniku = ZdrojVznikuPP.Kod';

        $this ->title = \Ron\ronText('Seznam_uchazecu');

        $this ->browserUrl = '?menuid=uchazeci&';
        $this ->rowClickLink = '?menuid=uchazeci&action=zobrazuchazece';
        $this ->cssClass .= ' seznamUchazecu';

        $this->primaryField = 0;

        $this->addField()
            ->setSqlFieldName('P_Uchazec.RC') ->setHeaderText('ID') ->setOrdering(true)
            ->setCode('col-rc');

        $this->addField()
            ->setSqlFieldName('P_Uchazec.Titul') ->setHeaderText('Titul') ->setOrdering(true)
            ->setCode('col-titul');

        $this->addField()
            ->setSqlFieldName('P_Uchazec.Jmeno') ->setHeaderText('Jmeno') ->setOrdering(true)
            ->setCode('col-jmeno')
            ->setLink('?menuid=uchazeci&action=zobrazuchazece');

        $this->addField()
            ->setSqlFieldName('P_Uchazec.Prijmeni') ->setHeaderText('Prijmeni') ->setOrdering(true)
            ->setCode('col-prijmeni')
            ->setLink('?menuid=uchazeci&action=zobrazuchazece');
        $this ->defaultSort = count($this ->fields);

        $this->addField()
            ->setSqlFieldName('P_Uchazec.TitulZa') ->setHeaderText('Titul_za') ->setOrdering(true)
            ->setCode('col-titulza');

        $this->addField()
            ->setSqlFieldName('P_Uchazec.VEvidenciOd') ->setFormat('finedate') ->setHeaderText('V_evidenci_od') ->setOrdering(true)
            ->setCode('col-vevidenciod');

        $this->addField( new RozdeleniUchazeceField($this->database, RonPortalSettings::getIntValue('uchazecityprozdeleniprofiltr')))
            ->setCode('col-rozdeleni');

        if ($this->typRozdeleniFiltr3) {
            $this->addField( new RozdeleniUchazeceField($this->database, $this ->typRozdeleniFiltr3))
                ->setCode('col-rozdeleni');
        }

        $this->addField()
            ->setSqlFieldName('ZdrojVznikuPP.Popis') ->setHeaderText('zdrojVzniku') ->setOrdering(true)
            ->setCode('col-zdrojvznikupp');

        for ($i = 1; $i <= 20; $i++) {
            $this->addField()
                ->setSqlFieldName('uzivatelska' . $i) ->setHeaderText(RonPortalSettings::getValue('uchazeciuzivatelska' . $i))
                ->setVisible((bool)RonPortalSettings::getValue('uchazeciuzivatelska' . $i))
                ->setCode('col-uzivatelska' . $i);
        }
    }


    public function setFilterForm()
    {
        //$this->filterForm = new UchazeciFilterForm($this->database, $this->typRozdeleniFiltr2, $this->typRozdeleniFiltr3);
        $this->filterForm = new RonPortalFilterForm($this ->database, 'uchazeci', true);

        $this ->filterForm ->nastavRozdeleni(true, -1, 0, '', true);
        $this ->filterForm ->showVcetnePodrizenychRozdeleni = false;

        if ($this ->typRozdeleniFiltr2) {
            $this ->filterForm ->nastavRozdeleni2(true, $this ->typRozdeleniFiltr2, 0, '', true);
        }

        if ($this ->typRozdeleniFiltr3) {
            $this ->filterForm ->nastavRozdeleni3(true, $this ->typRozdeleniFiltr3, 0, '');
        }
    }


    /**
     * @param int $kodTypuRozdeleni
     * @param string $kodRozdeleni
     * @param int $vcetnePodrizenych
     * @param Parameters $params
     * @return string
     */
    private function getFiltrRozdeleni($kodTypuRozdeleni, $kodRozdeleni, $vcetnePodrizenych, $params)
    {
        $tabulka = ($kodTypuRozdeleni === -1)
            ? 'UchazecNaPM'
            : 'RozdeleniUchazece';

        $fieldNameKod = ($kodTypuRozdeleni === -1)
            ? 'KodRozdeleni'
            : 'Rozdeleni';

        switch ($vcetnePodrizenych) {
            case 1 : // vcetne podrizenych
            case 2 : // jen podrizene
                return
                    'EXISTS (' .
                    " SELECT * FROM $tabulka, HierarchieRozdeleni" .
                    ' WHERE HierarchieRozdeleni.TypNadrazene = ' . $params->addParameter($kodTypuRozdeleni) .
                    '   AND HierarchieRozdeleni.KodNadrazene = ' . $params->addParameter($kodRozdeleni) .

                    (($vcetnePodrizenych == 2) ? ' AND HierarchieRozdeleni.KodNadrazene <> HierarchieRozdeleni.KodPodrizene' : '') . // filtr pro jen podrizene

                    "    AND $tabulka.KodTypuRozdeleni = HierarchieRozdeleni.typPodrizene" .
                    "    AND $tabulka.$fieldNameKod = HierarchieRozdeleni.kodPodrizene" .
                    "    AND $tabulka.RC = P_Uchazec.RC" .
                    ')';

            case 3 :
                // jen primo podrizene
                return
                    'EXISTS (' .
                    " SELECT * FROM $tabulka, Rozdeleni " .
                    ' WHERE Rozdeleni.RozdeleniRodice = ' . $params->addParameter($kodTypuRozdeleni) .
                    '   AND Rozdeleni.Rodic = ' . $params->addParameter($kodRozdeleni) .
                    "   AND $tabulka.KodTypuRozdeleni = Rozdeleni.KodTypuRozdeleni" .
                    "   AND $tabulka.$fieldNameKod = Rozdeleni.Kod" .
                    "   AND $tabulka.RC = P_Uchazec.RC" .
                    ')';
            default :
                // jen vybrane
                return
                    'EXISTS (' .
                    " SELECT * FROM $tabulka" .
                    " WHERE $tabulka.KodTypuRozdeleni = " . $params->addParameter($kodTypuRozdeleni) .
                    " AND $tabulka.$fieldNameKod = " . $params->addParameter($kodRozdeleni) .
                    " AND $tabulka.RC = P_Uchazec.RC" .
                    ')';
        }
    }


    public function getSQLFilter()
    {
        $result = '';
        $params = $this ->database ->createParameters();

        if ($this ->filterForm ->kodRozdeleni) { // ::rozdeleni(-1) != '') {
            $result .= $this ->getFiltrRozdeleni($this ->filterForm ->kodTypuRozdeleni,
                $this ->filterForm ->kodRozdeleni, 0, // $this ->filterForm ->vcetnePodrizenychRozdeleni,
                $params);
        }

        if ($this ->filterForm ->kodRozdeleni2) {
            $result .= $this ->getFiltrRozdeleni($this ->filterForm ->kodTypuRozdeleni2,
                $this ->filterForm ->kodRozdeleni2, $this ->filterForm ->vcetnePodrizenychRozdeleni2,
                $params);
        }

        if ($this ->filterForm ->kodRozdeleni3) {
            $result .= $this ->getFiltrRozdeleni($this ->filterForm ->kodTypuRozdeleni3,
                $this ->filterForm ->kodRozdeleni3, $this ->filterForm ->vcetnePodrizenychRozdeleni3,
                $params);
        }

        return new SqlStatement($result, $params);
    }
}
