<?php
/**
 * Roman.Szablatura RON Software 3.10.2017
 */

namespace RonPortal\Zamestnanci;

use Ron\Controls\BrowserField;
use Ron\DB\SqlStatement;
use Ron\GuiUtils;
use Ron\DB\Database;
use Ron\StringUtils;
use Ron\Controls\ComboBox;
use Ron\TextManager;
use RonPortal\Controls\JmenoPrijmeniBrowserField;
use RonPortal\Controls\RonPortalBrowser;
use RonPortal\Controls\RonPortalFilterForm;
use RonPortal\PracovniPomer;
use RonPortal\Prava;
use RonPortal\PravaNaOsoby;
use RonPortal\RonPortalSession;
use RonPortal\RozdeleniUtils;


/**
 * Class NahlaseneZmenyFiltr
 */
class NahlaseneZmenyFiltr extends RonPortalFilterForm
{
    /** @var string|null    Aktualni hodnota filtru "typ zmeny" */
    public $kodTypuZmeny = null;

    /** @var int|null       Aktualni hodnota filtru "stav zmeny" */
    public $stavZmeny = null;

    /** @var int|null       Aktualni hodnota filtru "zpracovatel" */
    public $zpracovatel = null;



    /**
     * NahlaseneZmenyFiltr constructor.
     * @param Database $database
     * @param string $action
     * @param bool $useGet
     */
    public function __construct(Database $database, $action, $useGet = false)
    {
        parent::__construct($database, $action, $useGet);

        $this->f_classname = 'filtercontainer';
        $this->f_formaction = '?menuid=nahlasenezmeny';
        $this->setUseGet(true, 'nahlasenezmeny');

        $this->nastavDatumOdDo(true, false, \Ron\ronText('obdobi_nahlasenych_zmen') . ' ' . \Ron\ronText('Od'));
        $this->nastavOsoby(true, [Prava::vratPrava('ou_zmeny_nahlaseni'), Prava::vratPrava('ou_zmeny_schval')], time(), time(), true);
        $this->povolitVsechyOsoby = true;

        $this->nastavRozdeleni(true);
        $this->nastavRozdeleni2(true);
    }


    /**
     * @throws \Ron\DB\DBException
     */
    function renderTypZmeny()
    {
        echo
            '<div class="filterrow">';

        echo
            '<span class="filtercaption">' .
            TextManager::_('typ_zmeny') .
            '</span>';

        $combo = new ComboBox('filtrtypzmeny'); // nepouzivat "typzmeny" - pouziva se v controlleru (20.6.2022)
        $combo
            ->setClassName('adsfilterforminput_full')
            ->setEmptyRow(TextManager::_('_vse_'))
            ->setHtmlParams(['data-ron-autosubmit' => '1']);

        $combo->loadData($this->database,
            'SELECT DISTINCT kod, popis FROM TypNahlaseneZmeny ORDER BY popis',
            $this->kodTypuZmeny);

        $combo->render();

        echo '</div>';
    }



    /**
     * @throws \Ron\DB\DBException
     */
    function renderZpracovatel()
    {
        echo '<div class="filterrow">';
        echo
            '<span class="filtercaption">' .
            TextManager::_('zpracoval') .
            '</span>';

        $sql =
            "SELECT DISTINCT MAX(PracovniPomer.ID), Coalesce (Zpracovatel.Prijmeni, '') {fn_concat()} ' ' {fn_concat()} Coalesce (Zpracovatel.Jmeno, '') Zpracovatel" .
            ' FROM NahlasenaZmena, Osoba Zpracovatel, PracovniPomer' .
            ' WHERE NahlasenaZmena.Zpracoval = Zpracovatel.RC' .
            ' AND Zpracovatel.RC = PracovniPomer.RC' .
            ' GROUP BY Zpracovatel.Prijmeni, Zpracovatel.Jmeno' .
            ' ORDER BY Zpracovatel';

        $combo = new ComboBox('zpracovatel');
        $combo
            ->setClassName('adsfilterforminput_full')
            ->setEmptyRow(TextManager::_('_vsichni_'))
            ->setDataTypeComparison(false)
            ->setHtmlParams(['data-ron-autosubmit' => '1']);
            //->setHtmlParams(['data-ron-autosubmit' => '1', 'data-ron-autosubmit-check' => '#jen_zpracovatelcheckbox']);

        $combo->loadData($this->database, $sql, $this->zpracovatel);
        $combo->render();

        echo '</div>';
    }


    function renderStavZmeny()
    {
        echo '<div class="filterrow">';
            echo
                '<span class="filtercaption">' .
                TextManager::_('stav_zmeny') .
                '</span>';

            $combo = new ComboBox('stavzmeny', [
                '' => \Ron\ronText('_vse_'),
                0 => \Ron\ronText('nezpracovano'),
                1 => \Ron\ronText('zpracovano'),
                2 => \Ron\ronText('odmitnuto')
            ]);

            $combo
                ->setClassName('adsfilterforminput_full')
                ->setHtmlParams(['data-ron-autosubmit' => '1'])
                ->setDataTypeComparison(false)
                ->setSelectedOption($this ->stavZmeny);

            $combo->render();

        echo '</div>';
    }


    function renderJenNezapsaneZmeny()
    {
        //Dlouhodoby kurz
        echo '<div class="">';

        echo '<span>';
        echo "<a href='?menuid=osobniudaje'>" . \Ron\ronText('osobni_udaje') . "</a>";
        echo '</span>';
        echo '</div>';
    }


    function loadZpracovatel()
    {
        if(isset($_GET['zpracovatel'])) {
            $this ->zpracovatel = $_GET['zpracovatel'];
            $this ->zpracovatel =
                ($this ->zpracovatel === '')
                    ? null
                    : (int)$this ->zpracovatel;
            RonPortalSession::setValue('filtr_zpracovatelzmeny', $this ->zpracovatel);
        } else {
            $this ->zpracovatel = RonPortalSession::getValue('filtr_zpracovatelzmeny');
        }
    }


    function loadTypZmeny()
    {
        if(isset($_GET['filtrtypzmeny'])){
            $this ->kodTypuZmeny = $_GET['filtrtypzmeny'];
            RonPortalSession::setValue('filtr_kodTypuZmeny', $this ->kodTypuZmeny);
        } else {
            $this ->kodTypuZmeny = RonPortalSession::getValue('filtr_kodTypuZmeny');
        }
    }


    function loadStavZmeny()
    {
        if(isset($_GET['stavzmeny'])){
            $this ->stavZmeny = $_GET['stavzmeny'];
            $this ->stavZmeny = ($this ->stavZmeny === '')
                ? null
                : (int)$this ->stavZmeny;
            RonPortalSession::setValue('filtr_stavZmeny', $this ->stavZmeny);
        } else {
            $this ->stavZmeny = RonPortalSession::getValue('filtr_stavZmeny');
        }
    }


    function loadData(){
        $this->loadTypZmeny();
        $this->loadZpracovatel();
        $this->loadStavZmeny();
        parent::loadData();
    }


    /**
     * @throws \Ron\DB\DBException
     */
    function renderFilters()
    {
        parent::renderFilters();
        $this->renderTypZmeny();
        //$this->renderOsobaZmeny();
        $this->renderZpracovatel();
        $this->renderStavZmeny();
        $this->renderJenNezapsaneZmeny();
    }
}

/**
 * Class ZobrazeniDetailuNahlaseneZmenyField
 */
class ZobrazeniDetailuNahlaseneZmenyField extends BrowserField
{
    /**
     * @param mixed $value
     * @param int $rowNumber
     * @param string $recordId
     * @param array $rowData
     * @return int|string
     */
    function getViewText($value, $rowNumber, $recordId, $rowData)
    {
        return parent::getViewText(
            \Ron\ronText(($rowData['col-stavzmeny']) ? 'detail' : 'detail_zmeny_s_potvrzenim'),
            $rowNumber, $recordId, $rowData);
    }
}



/**
 * Class NahlaseneZmenyBrowser
 */
class NahlaseneZmenyBrowser extends RonPortalBrowser
{

    /** @var  PravaNaOsoby */
    var $pravo_nahlaseni;

    /** @var  PravaNaOsoby */
    var $pravo_schvaleni;


    /**
     * NahlaseneZmenyBrowser constructor.
     * @param Database $database
     * @throws \Exception
     */
    public function __construct(Database $database)
    {
        parent::__construct($database, 'nahlasenezmeny');

        Prava::nastavObdobiProPrava(time(),time());
        $this->pravo_nahlaseni = Prava::vratPrava('ou_zmeny_nahlaseni');
        $this->pravo_schvaleni = Prava::vratPrava('ou_zmeny_schval');

        $this->code = 'browsernahlasenezmeny';
        $this->tableId = 'browsernahlasenezmeny';
        $this->browserUrl = '?menuid=nahlasenezmeny';
        
        $this->title = \Ron\ronText('nahlasene_zmeny');

        $this->sqlSelectPart =
            "NahlasenaZmena.ID, NahlasenaZmena.Stav, NahlasenaZmena.RC";    //0..2

        $this->primaryField = 0;

        $this->sqlFromPart =
            "(NahlasenaZmena LEFT JOIN TypNahlaseneZmeny ON (NahlasenaZmena.TypZmeny = TypNahlaseneZmeny.Kod)) 
                LEFT JOIN Osoba OsobaZmeny ON (NahlasenaZmena.RC = OsobaZmeny.RC) " .
                "LEFT JOIN Osoba Zpracovatel ON (NahlasenaZmena.Zpracoval = Zpracovatel.RC)" .
                 "LEFT JOIN Osoba Nahlasil ON (NahlasenaZmena.Nahlasil = Nahlasil.RC)";
                  //"LEFT JOIN PracovniPomer PP ON (PP.RC = NahlasenaZmena.RC)";

        $this->filterForm = new NahlaseneZmenyFiltr($database, null);
        $this->filterForm->loadData();

        $this->addField()
            ->setSqlFieldName('NahlasenaZmena.DatumCas') ->setHeaderText('datum_hlaseni') ->setFormat('date')
            ->setCode('col-datum')
            ->setOrdering(true) ->setAlign('center');

        $this->defaultSort = -1;
        
        //$this->addField(new StavZmenyField());
        $this->addField(BrowserField::FIELD_TYPE_SELECT)
            ->setSqlFieldName('NahlasenaZmena.Stav') ->setOrdering(true)
            ->setHeaderText('stav_zmeny')
            ->setCode('col-stavzmeny')
            ->setOptions([
                0 => \Ron\ronText('nezpracovano'),
                1 => \Ron\ronText('zpracovano'),
                2 => \Ron\ronText('odmitnuto')
            ]);

        //$this->addField(new StavZmenyField());
        $this->addField()
            ->setSqlFieldName('TypNahlaseneZmeny.Popis') ->setOrdering(true)
            ->setHeaderText('typ_zmeny')
            ->setCode('col-typzmeny')
            ->setLink('?menuid=nahlasenezmeny'); // . ($backMenuId ? '&back=' . $backMenuId : ''))
        //$this->addField(new TypZmenyField());

        $this->addField(new ZobrazeniDetailuNahlaseneZmenyField($this->database))
            ->setAlign('center') ->setHeaderText('detail')
            ->setCode('col-detail')
            ->setLink('?menuid=nahlasenezmeny&record=');

        // osoba zmeny
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('OsobaZmeny.Prijmeni') ->setCode('col-osoba-prijmeni');
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('OsobaZmeny.Jmeno') ->setCode('col-osoba-jmeno');
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('OsobaZmeny.Titul') ->setCode('col-osoba-titul');

        $this ->addField(new JmenoPrijmeniBrowserField($this ->database, 'col-osoba-jmeno', 'col-osoba-prijmeni', null, 'col-osoba-titul'))
            ->setCode('col-osoba')
            ->setSortSQL('OsobaZmeny.Prijmeni, OsobaZmeny.Jmeno, OsobaZmeny.Titul')
            ->setSortSQLDesc('OsobaZmeny.Prijmeni DESC, OsobaZmeny.Jmeno DESC, OsobaZmeny.Titul DESC')
            ->setHeaderText('osoba_zmeny')
            ->setOrdering(true);

        // nahlasil
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Nahlasil.Prijmeni') ->setCode('col-nahlasil-prijmeni');
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Nahlasil.Jmeno') ->setCode('col-nahlasil-jmeno');
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Nahlasil.Titul') ->setCode('col-nahlasil-titul');

        $this ->addField(new JmenoPrijmeniBrowserField($this ->database, 'col-nahlasil-jmeno', 'col-nahlasil-prijmeni', null, 'col-nahlasil-titul'))
            ->setCode('col-nahlasil')
            ->setSortSQL('Nahlasil.Prijmeni, Nahlasil.Jmeno, Nahlasil.Titul')
            ->setSortSQLDesc('Nahlasil.Prijmeni DESC, Nahlasil.Jmeno DESC, Nahlasil.Titul DESC')
            ->setHeaderText('zmenu_nahlasil') ->setClassName('poznamka')
            ->setOrdering(true);

        //$this->addField(new ZpracovatelField($this->database));
        // zpracoval
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Zpracovatel.Prijmeni') ->setCode('col-zpracoval-prijmeni');
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Zpracovatel.Jmeno') ->setCode('col-zpracoval-jmeno');
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('Zpracovatel.Titul') ->setCode('col-zpracoval-titul');
        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('NahlasenaZmena.Zpracoval') ->setCode('col-zpracoval-rc');

        $this ->addField($f = new JmenoPrijmeniBrowserField($this ->database, 'col-zpracoval-jmeno', 'col-zpracoval-prijmeni', null, 'col-zpracoval-titul'))
            ->setCode('col-zpracoval')
            ->setSortSQL('Zpracovatel.Prijmeni, Zpracovatel.Jmeno, Zpracovatel.Titul')
            ->setSortSQLDesc('Zpracovatel.Prijmeni DESC, Zpracovatel.Jmeno DESC, Zpracovatel.Titul DESC')
            ->setHeaderText('zpracoval') ->setClassName('poznamka')
            ->setOrdering(true);
        $f ->setIndexIfNull('col-zpracoval-rc'); // v tabulce neni vazba - pokud nenajde osobu podle RC, tak zobrazi primo hodnotu pole "zpracoval"

        $this->addField()
            ->setSqlFieldName('NahlasenaZmena.DatumCasZpracovani')
            ->setCode('col-datumzpracovani')
            ->setHeaderText('datum_zpracovani') ->setFormat('date') ->setOrdering(true) ->setAlign('center');

        $this ->addField(BrowserField::FIELD_TYPE_HIDDEN)
            ->setSqlFieldName('{fn_CASTWIDEMEMO(NahlasenaZmena.Poznamka)}')
            ->setCode('col-poznamka');
    }

    function setFilterForm()
    {
        // uz nastaveno v konstruktoru
    }


    /**
     * @param array $values
     * @return bool
     * @throws \Exception
     */
    function enableLine($values)
    {
        //Zobrazi pokud ma na osobu zmeny pravo
        if ($this->pravo_nahlaseni->lzeZobrazitRC($values[2]) || $this->pravo_schvaleni->lzeZobrazitRC($values[2])){
            return true;
        }
        return false;
    }


    function getRowAttributes($values, $className, $rowNumber)
    {
        switch ($values[1]) { // stav
            case 1 :
                $className .= ' zpracovana-zmena';
                break;
            case 2 :
                $className .= ' odmitnuta-zmena';
                break;
            default :
                $className .= ' nezpracovana-zmena';
                break;
        }

        return parent::getRowAttributes($values, $className, $rowNumber);
    }


    function renderLine($values, $className, $rowNumber, $edit = false)
    {
        parent::renderLine($values, $className, $rowNumber, $edit);

        if ($values['col-poznamka']) {
            echo
                "<tr class='$className'>" .
                '<td></td><td colspan="' . ($this->visibleFieldsCount() - 1) . '" >' .
                '<span class="browserpoznamka">' . \Ron\ronText("Poznamka") . ': ' . StringUtils::htmlTextConvert($values['col-poznamka']) . '</span>' .
                '</td>' .
                '</tr>';
        }

        $this->renderSeparator();
    }

    /**
     * @return SqlStatement
     * @throws \Ron\DB\DBException
     */
    public function getSQLFilter()
    {
        $result = '';
        $params = $this ->database ->createParameters();

        $pp = null;
        if (!$this ->filterForm ->vsechnyOsoby) {
            $pp = PracovniPomer::nactiPP($this ->database, $this ->filterForm ->vybranaOsoba);
        }


        if ($this ->filterForm ->filtrovatDatum) {
            $result .=
                ' NahlasenaZmena.DatumCas >= ' . $params ->addDateParameter($this ->filterForm ->datumOd) .
                ' AND NahlasenaZmena.DatumCas <= ' . $params ->addDateParameter($this ->filterForm ->datumDo);
        }

        if ($this ->filterForm ->kodTypuZmeny) {
            $result .=
                ($result ? ' AND ' : '') .
                'TypNahlaseneZmeny.kod = ' . $params->addParameter($this ->filterForm ->kodTypuZmeny);
        }

        if (isset($this ->filterForm ->stavZmeny)) {
            $result .=
                ($result ? ' AND ' : '') .
                (!$this ->filterForm ->stavZmeny
                    ? '(NahlasenaZmena.DatumCasZpracovani IS NULL OR NahlasenaZmena.Zpracoval IS NULL)'
                    : '') .
                (($this ->filterForm ->stavZmeny === 1)
                    ? '(NahlasenaZmena.Stav = ' . $params->addParameter($this ->filterForm ->stavZmeny) . ')' .
                      ' AND (NahlasenaZmena.DatumCasZpracovani IS NOT NULL OR NahlasenaZmena.Zpracoval IS NOT NULL)' .
                      ' OR (NahlasenaZmena.Stav =  0 AND (NahlasenaZmena.DatumCasZpracovani IS NOT NULL OR NahlasenaZmena.Zpracoval IS NOT NULL))'
                    : '') .
                (($this ->filterForm ->stavZmeny === 2)
                    ? ' NahlasenaZmena.Stav = ' . $params->addParameter($this ->filterForm ->stavZmeny)
                    : '');
        }

        if($pp) {
            $result .=
                ($result ? ' AND ' : '') .
                'NahlasenaZmena.RC = ' . $params->addParameter($pp->rc);
        } else {
            if ($this ->filterForm ->kodRozdeleni) {
                $rozdeleniFiltr =
                    RozdeleniUtils::sqlFilterRozdeleniPP($this ->database, 'NahlasenaZmena',
                        $this ->filterForm ->kodTypuRozdeleni, $this ->filterForm ->kodRozdeleni,
                        $this ->filterForm ->filtrovatDatum ? $this ->filterForm ->datumOd : time(),
                        $this ->filterForm ->filtrovatDatum ? $this ->filterForm ->datumDo : time(),
                        $this ->filterForm ->vcetnePodrizenychRozdeleni, $params, true); // jenRC = true

                if ($rozdeleniFiltr) {
                    $result .= ($result ? ' AND ' : '') . '(' . $rozdeleniFiltr . ')';
                }
            }

            if ($this ->filterForm ->kodRozdeleni2) {
                $rozdeleniFiltr2 =
                    RozdeleniUtils::sqlFilterRozdeleniPP($this ->database, 'NahlasenaZmena',
                        $this ->filterForm ->kodTypuRozdeleni2, $this ->filterForm ->kodRozdeleni2,
                        $this ->filterForm ->filtrovatDatum ? $this ->filterForm ->datumOd : time(),
                        $this ->filterForm ->filtrovatDatum ? $this ->filterForm ->datumDo : time(),
                        $this ->filterForm ->vcetnePodrizenychRozdeleni2, $params, true); // jenRC = true

                if ($rozdeleniFiltr2) {
                    $result .= ($result ? ' AND ' : '') . '(' . $rozdeleniFiltr2 . ')';
                }
            }
        }

        if (isset($this ->filterForm ->zpracovatel)) {
            try {
                $pracovniPomerZpracovatel = PracovniPomer::nactiPP($this->database, $this->filterForm->zpracovatel);

                $result .=
                    ($result ? ' AND ' : '') .
                    'NahlasenaZmena.Zpracoval = ' . $params->addParameter($pracovniPomerZpracovatel->rc);
            } catch (\Exception $e) {
                // zpracovatel neexistuje
                $result .= ' AND 0 = 1';
            }
        }

        return new SqlStatement($result, $params);
    }
}